@@ -4,9 +4,13 @@
 #ifndef NTP_H
 #define NTP_H
 
+#include <stddef.h>
 #include <math.h>
 
+#include <ntp_fp.h>
 #include <ntp_types.h>
+#include <ntp_lists.h>
+#include <ntp_stdlib.h>
 #ifdef OPENSSL
 #include <ntp_crypto.h>
 #endif /* OPENSSL */
@@ -14,7 +18,6 @@
 #include <ntp_net.h>
 
 #include <isc/boolean.h>
-#include <isc/list.h>
 
 /*
  * Calendar arithmetic - contributed by G. Healton
@@ -126,7 +129,7 @@ typedef char s_char;
 #define	NTP_MINCLOCK	3	/* min survivors */
 #define	NTP_MAXCLOCK	10	/* max candidates */
 #define	NTP_MAXASSOC	50	/* max associations */
-#define MINDISPERSE	.005	/* min dispersion increment */
+#define MINDISPERSE	.001	/* min distance */
 #define MAXDISTANCE	1.5	/* max root distance (select threshold) */
 #define CLOCK_SGATE	3.	/* popcorn spike gate */
 #define HUFFPUFF	900	/* huff-n'-puff sample interval (s) */
@@ -134,12 +137,13 @@ typedef char s_char;
 #define MAX_TTL		8	/* max ttl mapping vector size */
 #define	BEACON		7200	/* manycast beacon interval */
 #define NTP_MAXEXTEN	2048	/* max extension field size */
+#define	NTP_ORPHWAIT	300	/* orphan wair (s) */
 
 /*
  * Miscellaneous stuff
  */
 #define NTP_MAXKEY	65535	/* max authentication key number */
-
+#define	KEY_TYPE_MD5	NID_md5	/* MD5 digest NID */
 /*
  * Limits of things
  */
@@ -154,7 +158,7 @@ typedef char s_char;
  * dispersion component (frequency error plus precision). The frequency
  * error component is computed as CLOCK_PHI times the difference between
  * the epoch of the time measurement and the reference time. The
- * precision componen is computed as the square root of the mean of the
+ * precision component is computed as the square root of the mean of the
  * squares of a zero-mean, uniform distribution of unit maximum
  * amplitude. Whether this makes statistical sense may be arguable.
  */
@@ -168,34 +172,34 @@ typedef char s_char;
 
 #define	EVENT_TIMEOUT	0	/* one second, that is */
 
+
 /*
  * The interface structure is used to hold the addresses and socket
  * numbers of each of the interfaces we are using.
  */
 struct interface {
-	SOCKET fd;			/* socket this is opened on */
-	SOCKET bfd;			/* socket for receiving broadcasts */
-	struct sockaddr_storage sin;	/* interface address */
-	struct sockaddr_storage bcast;	/* broadcast address */
-	struct sockaddr_storage mask;	/* interface mask */
-	char name[32];			/* name of interface */
-	short  family;			/* Address family */
-	int flags;			/* interface flags */
-	int last_ttl;			/* last TTL specified */
-	u_int32 addr_refid;		/* IPv4 addr or IPv6 hash */
-	int num_mcast;			/* No. of IP addresses in multicast socket */
-	u_long starttime;	        /* current_time as of creation of interface structure */
-	volatile long received;		/* number of incoming packets */
-	long sent;			/* number of outgoing packets */
-	long notsent;			/* number of send failures */
-	u_int scopeid;			/* Scope used for Multicasting */
-	u_int ifindex;			/* interface index */
-	u_int ifnum;		        /* sequential interface instance count */
-        u_char phase;		        /* phase in update cycle */
-	isc_boolean_t ignore_packets;	/* Specify whether the packet should be ignored */
-        ISC_LIST(struct peer) peers;    /* list of peers for the interface */
-        u_int peercnt;		        /* number of peers referencinf this interface - informational only */
-        ISC_LINK(struct interface) link;     /* interface list */
+	struct interface *link;	/* interface list link */
+	SOCKET fd;		/* socket this is opened on */
+	SOCKET bfd;		/* socket for receiving broadcasts */
+	sockaddr_u sin;		/* interface address */
+	sockaddr_u bcast;	/* broadcast address */
+	sockaddr_u mask;	/* interface mask */
+	char name[32];		/* name of interface */
+	u_short family;		/* Address family */
+	u_short phase;		/* phase in update cycle */
+	int flags;		/* interface flags */
+	int last_ttl;		/* last TTL specified */
+	u_int32 addr_refid;	/* IPv4 addr or IPv6 hash */
+	int num_mcast;		/* No. of IP addresses in multicast socket */
+	u_long starttime;	/* current_time as of creation of interface structure */
+	volatile long received;	/* number of incoming packets */
+	long sent;		/* number of outgoing packets */
+	long notsent;		/* number of send failures */
+	u_int scopeid;		/* Scope used for Multicasting */
+	u_int ifnum;		/* sequential interface instance count */
+	isc_boolean_t ignore_packets; /* listen-read-drop this? */
+	struct peer *peers;	/* list of peers for the interface */
+	u_int peercnt;		/* peers referencing this interface */
 };
 
 /*
@@ -208,8 +212,9 @@ struct interface {
 #define INT_MULTICAST	0x010	/* can multicast out this interface */
 #define	INT_BCASTOPEN	0x020	/* broadcast socket is open */
 #define INT_MCASTOPEN	0x040	/* multicasting enabled */
-#define INT_WILDCARD    0x080   /* wildcard interface - usually skipped */
-#define INT_MCASTIF     0x100	/* bound directly to MCAST address */
+#define INT_WILDCARD	0x080   /* wildcard interface - usually skipped */
+#define INT_MCASTIF	0x100	/* bound directly to MCAST address */
+
 /*
  * Define flasher bits (tests 1 through 11 in packet procedure)
  * These reveal the state at the last grumble from the peer and are
@@ -244,11 +249,15 @@ struct interface {
  * spec.
  */
 struct peer {
-	struct peer *next;	/* pointer to next association */
-	struct peer *ass_next;	/* link pointer in associd hash */
-	struct sockaddr_storage srcadr; /* address of remote host */
-	struct interface *dstadr; /* pointer to address on local host */
-        ISC_LINK(struct peer) ilink; /* interface link list */
+	struct peer *p_link;	/* link pointer in free & peer lists */
+	struct peer *adr_link;	/* link pointer in address hash */
+	struct peer *aid_link;	/* link pointer in associd hash */
+	struct peer *ilink;	/* list of peers for interface */
+	sockaddr_u srcadr;	/* address of remote host */
+	char *	hostname;	/* if non-NULL, remote name */
+	struct addrinfo *addrs;	/* hostname query result */
+	struct addrinfo *ai;	/* position within addrs */
+	struct interface *dstadr; /* local address (interface) */
 	associd_t associd;	/* association ID */
 	u_char	version;	/* version number */
 	u_char	hmode;		/* local association mode */
@@ -437,7 +446,7 @@ struct peer {
 #define	FLAG_SYSPEER	0x0010	/* system peer */
 #define FLAG_PREFER	0x0020	/* prefer peer */
 #define FLAG_BURST	0x0040	/* burst mode */
-#define FLAG_FIXPOLL	0x0080	/* stick at minpoll */
+#define FLAG_PPS	0x0080	/* steered by PPS */
 #define FLAG_IBURST	0x0100	/* initial burst mode */
 #define FLAG_NOSELECT	0x0200	/* never select */
 #define FLAG_TRUE	0x0400	/* force truechimer */
@@ -445,11 +454,9 @@ struct peer {
 #define	FLAG_XLEAVE	0x1000	/* interleaved protocol */
 #define	FLAG_XB		0x2000	/* interleaved broadcast */
 #define	FLAG_XBOGUS	0x4000	/* interleaved bogus packet */
-#define	FLAG_PPS	0x8000	/* steered by PPS */
 #ifdef	OPENSSL
-#define FLAG_ASSOC	0x10000	/* autokey request */
+#define FLAG_ASSOC	0x8000	/* autokey request */
 #endif /* OPENSSL */
-#define FLAG_ADKEY      0x00010000      /* Authenticated (or wants reply to be authenticated) using AD authentication */
 
 /*
  * Definitions for the clear() routine.  We use memset() to clear
@@ -465,11 +472,6 @@ struct peer {
 #define LEN_CRYPTO_TO_ZERO	(END_CRYPTO_TO_ZERO((struct peer *)0) \
 				    - CRYPTO_TO_ZERO((struct peer *)0))
 
-/*
- * Reference clock identifiers (for pps signal)
- */
-#define PPSREFID (u_int32)"PPS "	/* used when pps controls stratum>1 */
-
 /*
  * Reference clock types.  Added as necessary.
  */
@@ -519,92 +521,6 @@ struct peer {
 #define REFCLK_NEOCLOCK4X	44	/* NeoClock4X DCF77 or TDF receiver */
 #define REFCLK_MAX		44	/* NeoClock4X DCF77 or TDF receiver */
 
- /*
- * Macro for sockaddr_storage structures operations
- */
-#ifdef ISC_PLATFORM_HAVESCOPEID
-#define SOCKSCOPE(sock1, sock2) (((struct sockaddr_in6 *)sock1)->sin6_scope_id == ((struct sockaddr_in6 *)sock2)->sin6_scope_id)
-#else
-#define SOCKSCOPE(sock1, sock2) 1
-#endif
-
-#define SOCKCMP(sock1, sock2) \
-	(((struct sockaddr_storage *)sock1)->ss_family \
-	    == ((struct sockaddr_storage *)sock2)->ss_family ? \
- 	((struct sockaddr_storage *)sock1)->ss_family == AF_INET ? \
- 	memcmp(&((struct sockaddr_in *)sock1)->sin_addr, \
-	    &((struct sockaddr_in *)sock2)->sin_addr, \
-	    sizeof(struct in_addr)) == 0 : \
-	memcmp(&((struct sockaddr_in6 *)sock1)->sin6_addr, \
-	    &((struct sockaddr_in6 *)sock2)->sin6_addr, \
-	       sizeof(struct in6_addr)) == 0 && SOCKSCOPE(sock1, sock2) : \
-	0)
-
-#define SOCKNUL(sock1) \
-	(((struct sockaddr_storage *)sock1)->ss_family == AF_INET ? \
- 	(((struct sockaddr_in *)sock1)->sin_addr.s_addr == 0) : \
- 	(IN6_IS_ADDR_UNSPECIFIED(&((struct sockaddr_in6 *)sock1)->sin6_addr)))
-
-#define SOCKLEN(sock) \
-	(((struct sockaddr_storage *)sock)->ss_family == AF_INET ? \
- 	(sizeof(struct sockaddr_in)) : (sizeof(struct sockaddr_in6)))
-
-#define ANYSOCK(sock) \
-	memset(((struct sockaddr_storage *)sock), 0, \
-	    sizeof(struct sockaddr_storage))
-
-#define ANY_INTERFACE_CHOOSE(sock) \
-	(((struct sockaddr_storage *)sock)->ss_family == AF_INET ? \
- 	any_interface : any6_interface)
-
-/*
- * We tell reference clocks from real peers by giving the reference
- * clocks an address of the form 127.127.t.u, where t is the type and
- * u is the unit number.  We define some of this here since we will need
- * some sanity checks to make sure this address isn't interpretted as
- * that of a normal peer.
- */
-#define	REFCLOCK_ADDR	0x7f7f0000	/* 127.127.0.0 */
-#define	REFCLOCK_MASK	0xffff0000	/* 255.255.0.0 */
-
-#define	ISREFCLOCKADR(srcadr)	((SRCADR(srcadr) & REFCLOCK_MASK) \
-					== REFCLOCK_ADDR)
-
-/*
- * Macro for checking for invalid addresses.  This is really, really
- * gross, but is needed so no one configures a host on net 127 now that
- * we're encouraging it the the configuration file.
- */
-#define	LOOPBACKADR	0x7f000001
-#define	LOOPNETMASK	0xff000000
-
-#define	ISBADADR(srcadr)	(((SRCADR(srcadr) & LOOPNETMASK) \
-				    == (LOOPBACKADR & LOOPNETMASK)) \
-				    && (SRCADR(srcadr) != LOOPBACKADR))
-
-/*
- * Utilities for manipulating addresses and port numbers
- */
-#define	NSRCADR(src)	(((struct sockaddr_in *)src)->sin_addr.s_addr) /* address in net byte order */
-#define	NSRCPORT(src)	(((struct sockaddr_in *)src)->sin_port)	/* port in net byte order */
-#define	SRCADR(src)	(ntohl(NSRCADR((src))))	/* address in host byte order */
-#define	SRCPORT(src)	(ntohs(NSRCPORT((src))))	/* host port */
-
-#define CAST_V4(src)	((struct sockaddr_in *)&(src))
-#define CAST_V6(src)	((struct sockaddr_in6 *)&(src))
-#define GET_INADDR(src)  (CAST_V4(src)->sin_addr.s_addr)
-#define GET_INADDR6(src) (CAST_V6(src)->sin6_addr)
-
-#define SET_HOSTMASK(addr, family)	\
-	do { \
-		memset((char *)(addr), 0, sizeof(struct sockaddr_storage)); \
-		(addr)->ss_family = (family); \
-		if ((family) == AF_INET) \
-			GET_INADDR(*(addr)) = 0xffffffff; \
-		else \
-			memset(&GET_INADDR6(*(addr)), 0xff, \
-			    sizeof(struct in6_addr)); \
-	} while(0)
 
 /*
  * NTP packet format.  The mac field is optional.  It isn't really
@@ -627,10 +543,10 @@ struct pkt {
 	l_fp	rec;		/* receive time stamp */
 	l_fp	xmt;		/* transmit time stamp */
 
-#define	LEN_PKT_NOMAC	12 * sizeof(u_int32) /* min header length */
-#define	LEN_PKT_MAC	LEN_PKT_NOMAC +  sizeof(u_int32)
-#define MIN_MAC_LEN	3 * sizeof(u_int32)	/* DES */
-#define MAX_MAC_LEN	5 * sizeof(u_int32)	/* MD5 */
+#define	LEN_PKT_NOMAC	(12 * sizeof(u_int32)) /* min header length */
+#define MIN_MAC_LEN	(1 * sizeof(u_int32))	/* crypto_NAK */
+#define MAX_MD5_LEN	(5 * sizeof(u_int32))	/* MD5 */
+#define	MAX_MAC_LEN	(6 * sizeof(u_int32))	/* SHA */
 
 	/*
 	 * The length of the packet less MAC must be a multiple of 64
@@ -658,10 +574,11 @@ struct pkt {
 #define	PKT_LEAP(li_vn_mode)	((u_char)(((li_vn_mode) >> 6) & 0x3))
 
 /*
- * Stuff for putting things back into li_vn_mode
+ * Stuff for putting things back into li_vn_mode in packets and vn_mode
+ * in ntp_monitor.c's mon_entry.
  */
-#define	PKT_LI_VN_MODE(li, vn, md) \
-	((u_char)((((li) << 6) & 0xc0) | (((vn) << 3) & 0x38) | ((md) & 0x7)))
+#define VN_MODE(v, m)		((((v) & 7) << 3) | ((m) & 0x7))
+#define	PKT_LI_VN_MODE(l, v, m) ((((l) & 3) << 6) | VN_MODE((v), (m)))
 
 
 /*
@@ -697,6 +614,10 @@ struct pkt {
 #define	EVNT_LEAP	11	/* leap event */
 #define	EVNT_CLOCKRESET	12	/* clock step */
 #define	EVNT_KERN	13	/* kernel event */
+#define	EVNT_TAI	14	/* TAI */
+#define	EVNT_LEAPVAL	15	/* stale leapsecond values */
+#define	EVNT_CLKHOP	16	/* clockhop */
+
 /*
  * Peer event codes
  */
@@ -709,7 +630,7 @@ struct pkt {
 #define	PEVNT_RATE	(7 | PEER_EVENT) /* rate exceeded */
 #define	PEVNT_DENY	(8 | PEER_EVENT) /* access denied */
 #define PEVNT_ARMED	(9 | PEER_EVENT) /* leap armed */
-#define	PEVNT_NEWPEER	(10 | PEER_EVENT) /* sys peer*/
+#define	PEVNT_NEWPEER	(10 | PEER_EVENT) /* sys peer */
 #define	PEVNT_CLOCK	(11 | PEER_EVENT) /* clock event */
 #define	PEVNT_AUTH	(12 | PEER_EVENT) /* bad auth */
 #define	PEVNT_POPCORN	(13 | PEER_EVENT) /* popcorn */
@@ -738,9 +659,9 @@ struct pkt {
  * To speed lookups, peers are hashed by the low order bits of the
  * remote IP address. These definitions relate to that.
  */
-#define	NTP_HASH_SIZE	128
-#define	NTP_HASH_MASK	(NTP_HASH_SIZE-1)
-#define	NTP_HASH_ADDR(src)	sock_hash(src)
+#define	NTP_HASH_SIZE		128
+#define	NTP_HASH_MASK		(NTP_HASH_SIZE-1)
+#define	NTP_HASH_ADDR(src)	(sock_hash(src) & NTP_HASH_MASK)
 
 /*
  * min, min3 and max.  Makes it easier to transliterate the spec without
@@ -780,6 +701,7 @@ struct pkt {
 #define	PROTO_MAXHOP		24
 #define	PROTO_BEACON		25
 #define	PROTO_ORPHAN		26
+#define	PROTO_ORPHWAIT		27
 
 /*
  * Configuration items for the loop filter
@@ -816,94 +738,114 @@ struct pkt {
 /*
  * Structure used optionally for monitoring when this is turned on.
  */
+typedef struct mon_data	mon_entry;
 struct mon_data {
-	struct mon_data *hash_next;	/* next structure in hash list */
-	struct mon_data *mru_next;	/* next structure in MRU list */
-	struct mon_data *mru_prev;	/* previous structure in MRU list */
-	int	flags;			/* restrict flags */
-	int	leak;			/* leaky bucket accumulator */
-	int	count;			/* total packet count */
-	u_long	firsttime;		/* first time found */
-	u_long	lasttime;		/* last time found */
-	struct sockaddr_storage rmtadr;	/* address of remote host */
-	struct interface *interface;	/* interface on which this arrived */
-	u_short	rmtport;		/* remote port last came from */
-	u_char	mode;			/* packet mode */
-	u_char	version;		/* packet version */
-	u_char	cast_flags;		/* flags MDF_?CAST */
+	mon_entry *	hash_next;	/* next structure in hash list */
+	DECL_DLIST_LINK(mon_entry, mru);/* MRU list link pointers */
+	struct interface * lcladr;	/* address on which this arrived */
+	l_fp		first;		/* first time seen */
+	l_fp		last;		/* last time seen */
+	int		leak;		/* leaky bucket accumulator */
+	int		count;		/* total packet count */
+	u_short		flags;		/* restrict flags */
+	u_char		vn_mode;	/* packet mode & version */
+	u_char		cast_flags;	/* flags MDF_?CAST */
+	sockaddr_u	rmtadr;		/* address of remote host */
 };
 
 /*
- * Values for cast_flags
+ * Values for cast_flags in mon_entry and struct peer.  mon_entry uses
+ * only the first three, MDF_UCAST, MDF_MCAST, and MDF_BCAST.
  */
-#define	MDF_UCAST	0x01		/* unicast */
-#define	MDF_MCAST	0x02		/* multicast */
-#define	MDF_BCAST	0x04		/* broadcast */
-#define	MDF_LCAST	0x08		/* localcast */
-#define MDF_ACAST	0x10		/* manycast */
-#define	MDF_BCLNT	0x20		/* broadcast client */
-#define MDF_ACLNT	0x40		/* manycast client */
-
+#define	MDF_UCAST	0x01	/* unicast client */
+#define	MDF_MCAST	0x02	/* multicast server */
+#define	MDF_BCAST	0x04	/* broadcast server */
+#define	MDF_POOL	0x08	/* pool client solicitor */
+#define MDF_ACAST	0x10	/* manycast client solicitor */
+#define	MDF_BCLNT	0x20	/* eph. broadcast/multicast client */
+#define MDF_UCLNT	0x40	/* eph. manycast or pool client */
+/*
+ * In the context of struct peer in ntpd, three of the cast_flags bits
+ * represent configured associations which never receive packets, and
+ * whose reach is always 0: MDF_BCAST, MDF_MCAST, and MDF_ACAST.  The
+ * last can be argued as responses are received, but those responses do
+ * not affect the MDF_ACAST association's reach register, rather they
+ * (may) result in mobilizing ephemeral MDF_ACLNT associations.
+ */
+#define MDF_TXONLY_MASK	(MDF_BCAST | MDF_MCAST | MDF_ACAST | MDF_POOL)
+/*
+ * manycastclient-like solicitor association cast_flags bits
+ */
+#define MDF_SOLICIT_MASK	(MDF_ACAST | MDF_POOL)
 /*
  * Values used with mon_enabled to indicate reason for enabling monitoring
  */
-#define MON_OFF    0x00			/* no monitoring */
-#define MON_ON     0x01			/* monitoring explicitly enabled */
-#define MON_RES    0x02			/* implicit monitoring for RES_LIMITED */
+#define MON_OFF		0x00		/* no monitoring */
+#define MON_ON		0x01		/* monitoring explicitly enabled */
+#define MON_RES		0x02		/* implicit monitoring for RES_LIMITED */
 /*
  * Structure used for restrictlist entries
  */
-struct restrictlist {
-	struct restrictlist *next;	/* link to next entry */
-	u_int32 addr;			/* Ipv4 host address (host byte order) */
-	u_int32 mask;			/* Ipv4 mask for address (host byte order) */
-	u_long count;			/* number of packets matched */
-	u_short flags;			/* accesslist flags */
-	u_short mflags;			/* match flags */
-};
+typedef struct res_addr4_tag {
+	u_int32		addr;		/* IPv4 addr (host order) */
+	u_int32		mask;		/* IPv4 mask (host order) */
+} res_addr4;
 
-struct restrictlist6 {
-	struct restrictlist6 *next;	/* link to next entry */
-	struct in6_addr addr6;		/* Ipv6 host address */
-	struct in6_addr mask6;		/* Ipv6 mask address */
-	u_long count;			/* number of packets matched */
-	u_short flags;			/* accesslist flags */
-	u_short mflags;			/* match flags */
-};
+typedef struct res_addr6_tag {
+	struct in6_addr addr;		/* IPv6 addr (net order) */
+	struct in6_addr mask;		/* IPv6 mask (net order) */
+} res_addr6;
 
+typedef struct restrict_u_tag	restrict_u;
+struct restrict_u_tag {
+	restrict_u *		link;	/* link to next entry */
+	u_int32			count;	/* number of packets matched */
+	u_short			flags;	/* accesslist flags */
+	u_short			mflags;	/* match flags */
+	u_long			expire;	/* valid until time */
+	union {				/* variant starting here */
+		res_addr4 v4;
+		res_addr6 v6;
+	} u;
+};
+#define	V4_SIZEOF_RESTRICT_U	(offsetof(restrict_u, u)	\
+				 + sizeof(res_addr4))
+#define	V6_SIZEOF_RESTRICT_U	(offsetof(restrict_u, u)	\
+				 + sizeof(res_addr6))
 
 /*
  * Access flags
  */
-#define	RES_IGNORE		0x001	/* ignore packet */
-#define	RES_DONTSERVE		0x002	/* access denied */
-#define	RES_DONTTRUST		0x004	/* authentication required */
-#define	RES_VERSION		0x008	/* version mismatch */
-#define	RES_NOPEER		0x010	/* new association denied */
-#define RES_LIMITED		0x020	/* packet rate exceeded */
-
+#define	RES_IGNORE		0x0001	/* ignore packet */
+#define	RES_DONTSERVE		0x0002	/* access denied */
+#define	RES_DONTTRUST		0x0004	/* authentication required */
+#define	RES_VERSION		0x0008	/* version mismatch */
+#define	RES_NOPEER		0x0010	/* new association denied */
+#define RES_LIMITED		0x0020	/* packet rate exceeded */
 #define RES_FLAGS		(RES_IGNORE | RES_DONTSERVE |\
 				    RES_DONTTRUST | RES_VERSION |\
 				    RES_NOPEER | RES_LIMITED)
 
-#define	RES_NOQUERY		0x040	/* mode 6/7 packet denied */
-#define	RES_NOMODIFY		0x080	/* mode 6/7 modify denied */
-#define	RES_NOTRAP		0x100	/* mode 6/7 set trap denied */
-#define	RES_LPTRAP		0x200	/* mode 6/7 low priority trap */
+#define	RES_NOQUERY		0x0040	/* mode 6/7 packet denied */
+#define	RES_NOMODIFY		0x0080	/* mode 6/7 modify denied */
+#define	RES_NOTRAP		0x0100	/* mode 6/7 set trap denied */
+#define	RES_LPTRAP		0x0200	/* mode 6/7 low priority trap */
 
-#define RES_KOD			0x400	/* send kiss of death packet */
-#define RES_TIMEOUT		0x800	/* timeout this entry */
+#define RES_KOD			0x0400	/* send kiss of death packet */
+#define	RES_MSSNTP		0x0800	/* enable MS-SNTP authentication */
+#define RES_FLAKE		0x1000	/* flakeway - drop 10% */
 
-#define	RES_ALLFLAGS		(RES_FLAGS | RES_NOQUERY |\
-				    RES_NOMODIFY | RES_NOTRAP |\
-				    RES_LPTRAP | RES_KOD |\
-				    RES_TIMEOUT)
+#define	RES_ALLFLAGS		(RES_FLAGS | RES_NOQUERY |	\
+				 RES_NOMODIFY | RES_NOTRAP |	\
+				 RES_LPTRAP | RES_KOD |		\
+				 RES_MSSNTP | RES_FLAKE)
 
 /*
  * Match flags
  */
 #define	RESM_INTERFACE		0x1000	/* this is an interface */
-#define	RESM_NTPONLY		0x2000	/* match ntp port only */
+#define	RESM_NTPONLY		0x2000	/* match source port 123 */
+#define RESM_SOURCE		0x4000	/* from "restrict source" */
 
 /*
  * Restriction configuration ops
@@ -911,7 +853,7 @@ struct restrictlist6 {
 #define	RESTRICT_FLAGS		1	/* add flags to restrict entry */
 #define	RESTRICT_UNFLAG		2	/* remove flags from restrict entry */
 #define	RESTRICT_REMOVE		3	/* remove a restrict entry */
-#define	RESTRICT_REMOVEIF       4	/* remove an interface restrict entry */
+#define	RESTRICT_REMOVEIF	4	/* remove an interface restrict entry */
 
 /*
  * Endpoint structure for the select algorithm
@@ -929,7 +871,7 @@ struct endpoint {
 #define AM_PROCPKT	1		/* server/symmetric packet */	
 #define AM_BCST		2		/* broadcast packet */	
 #define AM_FXMIT	3		/* client packet */
-#define AM_MANYCAST	4		/* manycast packet */
+#define AM_MANYCAST	4		/* manycast or pool */
 #define AM_NEWPASS	5		/* new passive */
 #define AM_NEWBCL	6		/* new broadcast */
 #define	AM_POSSBCL	7		/* discard broadcast */
@@ -939,4 +881,6 @@ struct endpoint {
 #define NETINFO_CONFIG_DIR "/config/ntp"
 #endif
 
+/* ntpq -c mrulist rows per request limit in ntpd */
+#define MRU_ROW_LIMIT	256
 #endif /* NTP_H */
