@@ -5,44 +5,14 @@
 #ifndef NTP_REFCLOCK_H
 #define NTP_REFCLOCK_H
 
-#include "ntp_types.h"
-
-#if defined(HAVE_BSD_TTYS)
-#include <sgtty.h>
-#endif /* HAVE_BSD_TTYS */
-
-#if defined(HAVE_SYSV_TTYS)
-#include <termio.h>
-#endif /* HAVE_SYSV_TTYS */
-
-#if defined(HAVE_TERMIOS)
-# ifdef TERMIOS_NEEDS__SVID3
-#  define _SVID3
-# endif
-# include <termios.h>
-# ifdef TERMIOS_NEEDS__SVID3
-#  undef _SVID3
-# endif
-#endif
-
 #if defined(HAVE_SYS_MODEM_H)
 #include <sys/modem.h>
 #endif
 
-#if 0 /* If you need that, include ntp_io.h instead */
-#if defined(STREAM)
-#include <stropts.h>
-#if defined(CLK) /* This is never defined, except perhaps by a system header file */
-#include <sys/clkdefs.h>
-#endif /* CLK */
-#endif /* STREAM */
-#endif
-
+#include "ntp_types.h"
+#include "ntp_tty.h"
 #include "recvbuff.h"
 
-#if !defined(SYSV_TTYS) && !defined(STREAM) & !defined(BSD_TTYS)
-#define BSD_TTYS
-#endif /* SYSV_TTYS STREAM BSD_TTYS */
 
 #define SAMPLE(x)	pp->coderecv = (pp->coderecv + 1) % MAXSTAGE; \
 			pp->filter[pp->coderecv] = (x); \
@@ -129,9 +99,10 @@ struct refclockio {
 				due to small bursts
 				of refclock input data */
 	caddr_t	srcclock;	/* pointer to clock structure */
-	int	datalen;	/* lenth of data */
-	SOCKET	fd;		/* file descriptor */
+	int	datalen;	/* length of data */
+	int	fd;		/* file descriptor */
 	u_long	recvcount;	/* count of receive completions */
+	int	active;		/* nonzero when in use */
 };
 
 /*
@@ -185,7 +156,7 @@ struct refclockproc {
 	const char *clockdesc;	/* clock description */
 
 	char	a_lastcode[BMAX]; /* last timecode received */
-	u_short	lencode;	/* length of last timecode */
+	int	lencode;	/* length of last timecode */
 
 	int	year;		/* year of eternity */
 	int	day;		/* day of year */
@@ -234,8 +205,8 @@ struct refclock {
 	int (*clock_start)	(int, struct peer *);
 	void (*clock_shutdown)	(int, struct peer *);
 	void (*clock_poll)	(int, struct peer *);
-	void (*clock_control)	(int, struct refclockstat *,
-				    struct refclockstat *, struct peer *);
+	void (*clock_control)	(int, const struct refclockstat *,
+				 struct refclockstat *, struct peer *);
 	void (*clock_init)	(void);
 	void (*clock_buginfo)	(int, struct refclockbug *, struct peer *);
 	void (*clock_timer)	(int, struct peer *);
@@ -244,30 +215,25 @@ struct refclock {
 /*
  * Function prototypes
  */
-/*
- * auxiliary PPS interface (implemented by refclock_atom())
- */
-extern	int	pps_sample (l_fp *);
-extern	int	io_addclock_simple (struct refclockio *);
 extern	int	io_addclock	(struct refclockio *);
 extern	void	io_closeclock	(struct refclockio *);
 
 #ifdef REFCLOCK
-extern	void	refclock_buginfo (struct sockaddr_storage *,
-				    struct refclockbug *);
-extern	void	refclock_control (struct sockaddr_storage *,
-				    struct refclockstat *,
-				    struct refclockstat *);
+extern	void	refclock_buginfo(sockaddr_u *,
+				 struct refclockbug *);
+extern	void	refclock_control(sockaddr_u *,
+				 const struct refclockstat *,
+				 struct refclockstat *);
 extern	int	refclock_open	(char *, u_int, u_int);
 extern	int	refclock_setup	(int, u_int, u_int);
 extern	void	refclock_timer	(struct peer *);
-extern	void	refclock_transmit (struct peer *);
-extern	int	refclock_ioctl	(int, u_int);
-extern 	int	refclock_process (struct refclockproc *);
-extern 	void	refclock_process_offset (struct refclockproc *, l_fp, l_fp, double);
+extern	void	refclock_transmit(struct peer *);
+extern 	int	refclock_process(struct refclockproc *);
+extern 	int	refclock_process_f(struct refclockproc *, double);
+extern 	void	refclock_process_offset(struct refclockproc *, l_fp, l_fp, double);
 extern	void	refclock_report	(struct peer *, int);
 extern	int	refclock_gtlin	(struct recvbuf *, char *, int, l_fp *);
-extern	int	refclock_gtraw  (struct recvbuf *, char *, int, l_fp *);
+extern	int	refclock_gtraw	(struct recvbuf *, char *, int, l_fp *);
 #endif /* REFCLOCK */
 
 #endif /* NTP_REFCLOCK_H */
