@@ -2,12 +2,8 @@
  * Define string ops: strchr strrchr memcmp memmove memset 
  */
 
-#ifndef  _ntp_string_h
-#define  _ntp_string_h
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#ifndef NTP_STRING_H
+#define NTP_STRING_H
 
 #ifdef HAVE_MEMORY_H
 # include <memory.h>
@@ -36,13 +32,20 @@ char *strchr(), *strrchr();
 #endif /* STDC_HEADERS */
 
 #ifdef NTP_NEED_BOPS
-# define memcmp(a,b,c) bcmp(a,b,(int)c)
-# define memmove(t,f,c) bcopy(f,t,(int)c)
-# define memcpy(t,f,c) bcopy(f,t,(int)c)
-# define memset(a,x,c) if (x == 0x00) bzero(a,(int)c); else ntp_memset((char*)a,x,c)
 
-void ntp_memset (char *, int, int);
+#ifdef HAVE_STRINGS_H
+# include <strings.h>		/* bcmp, bcopy, bzero */
+#endif
 
+void	ntp_memset	(char *, int, int);
+
+#define memcmp(a, b, c)		bcmp(a, b, (int)(c))
+#define memmove(t, f, c)	bcopy(f, t, (int)(c))
+#define memcpy(t, f, c)		bcopy(f, t, (int)(c))
+#define memset(a, x, c)		if (0 == (x)) \
+					bzero(a, (int)(c)); \
+				else \
+					ntp_memset((char *)(a), x, c)
 #endif /*  NTP_NEED_BOPS */
 
-#endif /* _ntp_string_h */
+#endif	/* NTP_STRING_H */
