@@ -6,10 +6,6 @@
 #ifndef NTP_SYSCALL_H
 #define NTP_SYSCALL_H
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef HAVE_SYS_TIMEX_H
 # include <sys/timex.h>
 #endif
@@ -24,6 +20,15 @@ extern	int	__adjtimex	(struct timex *);
 
 #  define ntp_adjtime(t)	__adjtimex((t))
 
+#ifndef HAVE_STRUCT_NTPTIMEVAL
+struct ntptimeval
+{
+  struct timeval time;  /* current time (ro) */
+  long int maxerror;    /* maximum error (us) (ro) */
+  long int esterror;    /* estimated error (us) (ro) */
+};
+#endif
+
 static inline int
 ntp_gettime(
 	struct ntptimeval *ntv
