@@ -39,15 +39,13 @@ static char sccsid[] = "@(#)random.c	8.2 (Berkeley) 5/19/95";
 #endif /* LIBC_SCCS and not lint */
 
 #include "config.h"
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
+#include <sys/types.h>
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
 #include <stdio.h>
 
-#include <ntp_types.h>
+#include <l_stdlib.h>
 #include <ntp_random.h>
 #include <ntp_unixtime.h>
 
@@ -142,15 +140,8 @@ static char sccsid[] = "@(#)random.c	8.2 (Berkeley) 5/19/95";
 #define	DEG_4		63
 #define	SEP_4		1
 
-/*
- * Array versions of the above information to make code run faster --
- * relies on fact that TYPE_i == i.
- */
 #define	MAX_TYPES	5		/* max number of types above */
 
-static long degrees[MAX_TYPES] =	{ DEG_0, DEG_1, DEG_2, DEG_3, DEG_4 };
-static long seps [MAX_TYPES] =	{ SEP_0, SEP_1, SEP_2, SEP_3, SEP_4 };
-
 /*
  * Initially, everything is set up as if from:
  *
@@ -270,19 +261,22 @@ ntp_srandom(
 	unsigned long x
 	)
 {
-	register long i;
+	long i;
 
-	if (rand_type == TYPE_0)
+	if (rand_type == TYPE_0) {
 		state[0] = x;
-	else {
+	} else {
 		state[0] = x;
 		for (i = 1; i < rand_deg; i++)
 			state[i] = good_rand(state[i - 1]);
 		fptr = &state[rand_sep];
 		rptr = &state[0];
 		for (i = 0; i < 10 * rand_deg; i++)
-			(void)ntp_random();
+			x = ntp_random();
 	}
+
+	/* seed the likely faster (and poorer) rand() as well */
+	srand((u_int)x);
 }
 
 /*
@@ -309,6 +303,24 @@ ntp_srandomdev( void )
 }
 #endif
 
+
+/*
+ * ntp_initstate() and ntp_setstate() are unused in our codebase and
+ * trigger warnings due to casting to a more-strictly-aligned pointer
+ * on alignment-sensitive platforms.  #ifdef them away to save noise,
+ * build time, and binary space, but retain the code in case we find a
+ * use.
+ */
+#ifdef COMPILE_UNUSED_FUNCTIONS
+/*
+ * Array versions of the above information to make code run faster --
+ * relies on fact that TYPE_i == i.
+ */
+#define	MAX_TYPES	5		/* max number of types above */
+
+static long degrees[MAX_TYPES] =	{ DEG_0, DEG_1, DEG_2, DEG_3, DEG_4 };
+static long seps [MAX_TYPES] =	{ SEP_0, SEP_1, SEP_2, SEP_3, SEP_4 };
+
 /*
  * initstate:
  *
@@ -437,6 +449,8 @@ ntp_setstate(
 	end_ptr = &state[rand_deg];		/* set end_ptr too */
 	return(ostate);
 }
+#endif	/* COMPILE_UNUSED_FUNCTIONS */
+
 
 /*
  * random:
