@@ -1,55 +1,32 @@
 /*
- *  $Id: configfile.c,v 1.21 2007/04/15 19:01:18 bkorb Exp $
- *  Time-stamp:      "2007-04-15 11:22:46 bkorb"
+ *  $Id: f1650b45a91ec95af830ff76041cc4f0048e60f0 $
+ *  Time-stamp:      "2009-01-18 10:21:58 bkorb"
  *
  *  configuration/rc/ini file handling.
- */
-
-/*
- *  Automated Options copyright 1992-2007 Bruce Korb
  *
- *  Automated Options is free software.
- *  You may redistribute it and/or modify it under the terms of the
- *  GNU General Public License, as published by the Free Software
- *  Foundation; either version 2, or (at your option) any later version.
+ *  This file is part of AutoOpts, a companion to AutoGen.
+ *  AutoOpts is free software.
+ *  AutoOpts is copyright (c) 1992-2009 by Bruce Korb - all rights reserved
  *
- *  Automated Options is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  AutoOpts is available under any one of two licenses.  The license
+ *  in use must be one of these two and the choice is under the control
+ *  of the user of the license.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with Automated Options.  See the file "COPYING".  If not,
- *  write to:  The Free Software Foundation, Inc.,
- *             51 Franklin Street, Fifth Floor,
- *             Boston, MA  02110-1301, USA.
+ *   The GNU Lesser General Public License, version 3 or later
+ *      See the files "COPYING.lgplv3" and "COPYING.gplv3"
  *
- * As a special exception, Bruce Korb gives permission for additional
- * uses of the text contained in his release of AutoOpts.
+ *   The Modified Berkeley Software Distribution License
+ *      See the file "COPYING.mbsd"
  *
- * The exception is that, if you link the AutoOpts library with other
- * files to produce an executable, this does not by itself cause the
- * resulting executable to be covered by the GNU General Public License.
- * Your use of that executable is in no way restricted on account of
- * linking the AutoOpts library code into it.
+ *  These files have the following md5sums:
  *
- * This exception does not however invalidate any other reasons why
- * the executable file might be covered by the GNU General Public License.
- *
- * This exception applies only to the code released by Bruce Korb under
- * the name AutoOpts.  If you copy code from other sources under the
- * General Public License into a copy of AutoOpts, as the General Public
- * License permits, the exception does not apply to the code that you add
- * in this way.  To avoid misleading anyone as to the status of such
- * modified files, you must delete this exception notice from them.
- *
- * If you write modifications of your own for AutoOpts, it is your choice
- * whether to permit this exception to apply to your modifications.
- * If you do not wish that, delete this exception notice.
+ *  43b91e8ca915626ed3818ffb1b71248b pkg/libopts/COPYING.gplv3
+ *  06a1a2e4760c90ea5e1dad8dfaac4d39 pkg/libopts/COPYING.lgplv3
+ *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by :mkfwd */
+/* static forward declarations maintained by mk-fwd */
 static void
 filePreset(
     tOptions*     pOpts,
@@ -89,11 +66,6 @@ parseKeyWordType(
     char*         pzText,
     tOptionValue* pType );
 
-static char*
-parseLoadMode(
-    char*               pzText,
-    tOptionLoadMode*    pMode );
-
 static char*
 parseSetMemType(
     tOptions*     pOpts,
@@ -444,7 +416,7 @@ filePreset(
     int           direction )
 {
     tmap_info_t   cfgfile;
-    tOptState     st = OPTSTATE_INITIALIZER(PRESET);
+    tOptState     optst = OPTSTATE_INITIALIZER(PRESET);
     char*         pzFileText =
         text_mmap( pzFileName, PROT_READ|PROT_WRITE, MAP_PRIVATE, &cfgfile );
 
@@ -452,8 +424,8 @@ filePreset(
         return;
 
     if (direction == DIRECTION_CALLED) {
-        st.flags  = OPTST_DEFINED;
-        direction = DIRECTION_PROCESS;
+        optst.flags = OPTST_DEFINED;
+        direction   = DIRECTION_PROCESS;
     }
 
     /*
@@ -463,18 +435,19 @@ filePreset(
      *  and we consider stuff set herein to be "set" by the client program.
      */
     if ((pOpts->fOptSet & OPTPROC_PRESETTING) == 0)
-        st.flags = OPTST_SET;
+        optst.flags = OPTST_SET;
 
     do  {
-        while (isspace( (int)*pzFileText ))  pzFileText++;
+        while (IS_WHITESPACE_CHAR(*pzFileText))  pzFileText++;
 
-        if (isalpha( (int)*pzFileText )) {
-            pzFileText = handleConfig( pOpts, &st, pzFileText, direction );
+        if (IS_VAR_FIRST_CHAR(*pzFileText)) {
+            pzFileText = handleConfig(pOpts, &optst, pzFileText, direction);
 
         } else switch (*pzFileText) {
         case '<':
-            if (isalpha( (int)pzFileText[1] ))
-                pzFileText = handleStructure(pOpts, &st, pzFileText, direction);
+            if (IS_VAR_FIRST_CHAR(pzFileText[1]))
+                pzFileText =
+                    handleStructure(pOpts, &optst, pzFileText, direction);
 
             else switch (pzFileText[1]) {
             case '?':
@@ -549,8 +522,8 @@ handleConfig(
     if (pzEnd == NULL)
         return pzText + strlen(pzText);
 
-    while (ISNAMECHAR( (int)*pzText ))  pzText++;
-    while (isspace( (int)*pzText )) pzText++;
+    while (IS_VALUE_NAME_CHAR(*pzText)) pzText++;
+    while (IS_WHITESPACE_CHAR(*pzText)) pzText++;
     if (pzText > pzEnd) {
     name_only:
         *pzEnd++ = NUL;
@@ -564,10 +537,10 @@ handleConfig(
      *  is an invalid format and we give up parsing the text.
      */
     if ((*pzText == '=') || (*pzText == ':')) {
-        while (isspace( (int)*++pzText ))   ;
+        while (IS_WHITESPACE_CHAR(*++pzText))   ;
         if (pzText > pzEnd)
             goto name_only;
-    } else if (! isspace((int)pzText[-1]))
+    } else if (! IS_WHITESPACE_CHAR(pzText[-1]))
         return NULL;
 
     /*
@@ -631,7 +604,7 @@ handleDirective(
     size_t name_len;
 
     if (  (strncmp( pzText+2, zProg, title_len ) != 0)
-       || (! isspace( (int)pzText[title_len+2] )) )  {
+       || (! IS_WHITESPACE_CHAR(pzText[title_len+2])) )  {
         pzText = strchr( pzText+2, '>' );
         if (pzText != NULL)
             pzText++;
@@ -645,8 +618,8 @@ handleDirective(
     do  {
         pzText += title_len;
 
-        if (isspace((int)*pzText)) {
-            while (isspace((int)*pzText))  pzText++;
+        if (IS_WHITESPACE_CHAR(*pzText)) {
+            while (IS_WHITESPACE_CHAR(*++pzText))  ;
             if (  (strneqvcmp( pzText, pOpts->pzProgName, (int)name_len) == 0)
                && (pzText[name_len] == '>'))  {
                 pzText += name_len + 1;
@@ -706,13 +679,13 @@ handleStructure(
     int           direction )
 {
     tOptionLoadMode mode = option_load_mode;
-    tOptionValue     valu;
+    tOptionValue    valu;
 
     char* pzName = ++pzText;
     char* pzData;
     char* pcNulPoint;
 
-    while (ISNAMECHAR( *pzText ))  pzText++;
+    while (IS_VALUE_NAME_CHAR(*pzText))  pzText++;
     pcNulPoint = pzText;
     valu.valType = OPARG_TYPE_STRING;
 
@@ -779,6 +752,36 @@ handleStructure(
      */
     memset(pcNulPoint, ' ', pzData - pcNulPoint);
 
+    /*
+     *  If we are getting a "string" value, the process the XML-ish
+     *  %XX hex characters.
+     */
+    if (valu.valType == OPARG_TYPE_STRING) {
+        char * pzSrc = pzData;
+        char * pzDst = pzData;
+        char bf[4];
+        bf[2] = NUL;
+
+        for (;;) {
+            int ch = ((int)*(pzSrc++)) & 0xFF;
+            switch (ch) {
+            case NUL: goto string_fixup_done;
+
+            case '%':
+                bf[0] = *(pzSrc++);
+                bf[1] = *(pzSrc++);
+                if ((bf[0] == NUL) || (bf[1] == NUL))
+                    goto string_fixup_done;
+                ch = strtoul(bf, NULL, 16);
+                /* FALLTHROUGH */
+
+            default:
+                *(pzDst++) = ch;
+            }
+        } string_fixup_done:;
+        *pzDst = NUL;
+    }
+
     /*
      *  "pzName" points to what looks like text for one option/configurable.
      *  It is NUL terminated.  Process it.
@@ -798,13 +801,22 @@ handleStructure(
 LOCAL void
 internalFileLoad( tOptions* pOpts )
 {
-    int     idx;
-    int     inc = DIRECTION_PRESET;
-    char    zFileName[ AG_PATH_MAX+1 ];
+    uint32_t  svfl;
+    int       idx;
+    int       inc;
+    char      zFileName[ AG_PATH_MAX+1 ];
 
     if (pOpts->papzHomeList == NULL)
         return;
 
+    svfl = pOpts->fOptSet;
+    inc  = DIRECTION_PRESET;
+
+    /*
+     *  Never stop on errors in config files.
+     */
+    pOpts->fOptSet &= ~OPTPROC_ERRSTOP;
+
     /*
      *  Find the last RC entry (highest priority entry)
      */
@@ -877,6 +889,8 @@ internalFileLoad( tOptions* pOpts )
             }
         }
     } /* twice for every path in the home list, ... */
+
+    pOpts->fOptSet = svfl;
 }
 
 
@@ -939,35 +953,38 @@ optionFileLoad( tOptions* pOpts, char const* pzProgram )
 void
 optionLoadOpt( tOptions* pOpts, tOptDesc* pOptDesc )
 {
+    struct stat sb;
+
     /*
      *  IF the option is not being disabled, THEN load the file.  There must
      *  be a file.  (If it is being disabled, then the disablement processing
      *  already took place.  It must be done to suppress preloading of ini/rc
      *  files.)
      */
-    if (! DISABLED_OPT( pOptDesc )) {
-        struct stat sb;
-        if (stat( pOptDesc->optArg.argString, &sb ) != 0) {
-            if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
-                return;
+    if (  DISABLED_OPT(pOptDesc)
+       || ((pOptDesc->fOptState & OPTST_RESET) != 0))
+        return;
 
-            fprintf( stderr, zFSErrOptLoad, errno, strerror( errno ),
-                     pOptDesc->optArg.argString );
-            exit(EX_NOINPUT);
-            /* NOT REACHED */
-        }
+    if (stat( pOptDesc->optArg.argString, &sb ) != 0) {
+        if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
+            return;
 
-        if (! S_ISREG( sb.st_mode )) {
-            if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
-                return;
-
-            fprintf( stderr, zNotFile, pOptDesc->optArg.argString );
-            exit(EX_NOINPUT);
-            /* NOT REACHED */
-        }
-
-        filePreset(pOpts, pOptDesc->optArg.argString, DIRECTION_CALLED);
+        fprintf( stderr, zFSErrOptLoad, errno, strerror( errno ),
+                 pOptDesc->optArg.argString );
+        exit(EX_NOINPUT);
+        /* NOT REACHED */
     }
+
+    if (! S_ISREG( sb.st_mode )) {
+        if ((pOpts->fOptSet & OPTPROC_ERRSTOP) == 0)
+            return;
+
+        fprintf( stderr, zNotFile, pOptDesc->optArg.argString );
+        exit(EX_NOINPUT);
+        /* NOT REACHED */
+    }
+
+    filePreset(pOpts, pOptDesc->optArg.argString, DIRECTION_CALLED);
 }
 
 
@@ -982,45 +999,65 @@ parseAttributes(
     tOptionLoadMode*    pMode,
     tOptionValue*       pType )
 {
-    size_t lenLoadType = strlen( zLoadType );
-    size_t lenKeyWords = strlen( zKeyWords );
-    size_t lenSetMem   = strlen( zSetMembers );
+    size_t len;
 
     do  {
-        switch (*pzText) {
-        case '/': pType->valType = OPARG_TYPE_NONE;
-        case '>': return pzText;
+        if (! IS_WHITESPACE_CHAR(*pzText))
+            switch (*pzText) {
+            case '/': pType->valType = OPARG_TYPE_NONE;
+            case '>': return pzText;
+
+            default:
+            case NUL: return NULL;
+            }
+
+        while (IS_WHITESPACE_CHAR(*++pzText))     ;
+        len = 0;
+        while (IS_LOWER_CASE_CHAR(pzText[len]))   len++;
+
+        switch (find_xat_attribute_id(pzText, len)) {
+        case XAT_KWD_TYPE:
+            pzText = parseValueType( pzText+len, pType );
+            break;
+
+        case XAT_KWD_WORDS:
+            pzText = parseKeyWordType( pOpts, pzText+len, pType );
+            break;
+
+        case XAT_KWD_MEMBERS:
+            pzText = parseSetMemType( pOpts, pzText+len, pType );
+            break;
+
+        case XAT_KWD_COOKED:
+            pzText += len;
+            if (! IS_END_XML_TOKEN_CHAR(*pzText))
+                goto invalid_kwd;
+
+            *pMode = OPTION_LOAD_COOKED;
+            break;
+
+        case XAT_KWD_UNCOOKED:
+            pzText += len;
+            if (! IS_END_XML_TOKEN_CHAR(*pzText))
+                goto invalid_kwd;
+
+            *pMode = OPTION_LOAD_UNCOOKED;
+            break;
+
+        case XAT_KWD_KEEP:
+            pzText += len;
+            if (! IS_END_XML_TOKEN_CHAR(*pzText))
+                goto invalid_kwd;
+
+            *pMode = OPTION_LOAD_KEEP;
+            break;
 
         default:
-        case NUL: return NULL;
-
-        case ' ':
-        case '\t':
-        case '\n':
-        case '\f':
-        case '\r':
-        case '\v':
-            break;
+        case XAT_KWD_INVALID:
+        invalid_kwd:
+            pType->valType = OPARG_TYPE_NONE;
+            return skipUnknown( pzText );
         }
-
-        while (isspace( (int)*++pzText ))   ;
-
-        if (strncmp( pzText, zLoadType, lenLoadType ) == 0) {
-            pzText = parseValueType( pzText+lenLoadType, pType );
-            continue;
-        }
-
-        if (strncmp( pzText, zKeyWords, lenKeyWords ) == 0) {
-            pzText = parseKeyWordType( pOpts, pzText+lenKeyWords, pType );
-            continue;
-        }
-
-        if (strncmp( pzText, zSetMembers, lenSetMem ) == 0) {
-            pzText = parseSetMemType( pOpts, pzText+lenSetMem, pType );
-            continue;
-        }
-
-        pzText = parseLoadMode( pzText, pMode );
     } while (pzText != NULL);
 
     return pzText;
@@ -1042,61 +1079,6 @@ parseKeyWordType(
 }
 
 
-/*  parseLoadMode
- *
- *  "pzText" points to some name character.  We check for "cooked" or
- *  "uncooked" or "keep".  This function should handle any attribute
- *  that does not have an associated value.
- */
-static char*
-parseLoadMode(
-    char*               pzText,
-    tOptionLoadMode*    pMode )
-{
-    {
-        size_t len = strlen(zLoadCooked);
-        if (strncmp( pzText, zLoadCooked, len ) == 0) {
-            if (  (pzText[len] == '>')
-               || (pzText[len] == '/')
-               || isspace((int)pzText[len])) {
-                *pMode = OPTION_LOAD_COOKED;
-                return pzText + len;
-            }
-            goto unknown;
-        }
-    }
-
-    {
-        size_t len = strlen(zLoadUncooked);
-        if (strncmp( pzText, zLoadUncooked, len ) == 0) {
-            if (  (pzText[len] == '>')
-               || (pzText[len] == '/')
-               || isspace((int)pzText[len])) {
-                *pMode = OPTION_LOAD_UNCOOKED;
-                return pzText + len;
-            }
-            goto unknown;
-        }
-    }
-
-    {
-        size_t len = strlen(zLoadKeep);
-        if (strncmp( pzText, zLoadKeep, len ) == 0) {
-            if (  (pzText[len] == '>')
-               || (pzText[len] == '/')
-               || isspace((int)pzText[len])) {
-                *pMode = OPTION_LOAD_KEEP;
-                return pzText + len;
-            }
-            goto unknown;
-        }
-    }
-
-  unknown:
-    return skipUnknown( pzText );
-}
-
-
 /*  parseSetMemType
  *
  *  "pzText" points to the character after "members="
@@ -1122,75 +1104,52 @@ parseValueType(
     char*         pzText,
     tOptionValue* pType )
 {
-    {
-        size_t len = strlen(zLtypeString);
-        if (strncmp( pzText, zLtypeString, len ) == 0) {
-            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
-                pType->valType = OPARG_TYPE_STRING;
-                return pzText + len;
-            }
-            goto unknown;
-        }
+    size_t len = 0;
+
+    if (*(pzText++) != '=')
+        goto woops;
+
+    while (IS_OPTION_NAME_CHAR(pzText[len]))  len++;
+    pzText += len;
+
+    if ((len == 0) || (! IS_END_XML_TOKEN_CHAR(*pzText))) {
+    woops:
+        pType->valType = OPARG_TYPE_NONE;
+        return skipUnknown( pzText );
     }
 
-    {
-        size_t len = strlen(zLtypeInteger);
-        if (strncmp( pzText, zLtypeInteger, len ) == 0) {
-            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
-                pType->valType = OPARG_TYPE_NUMERIC;
-                return pzText + len;
-            }
-            goto unknown;
-        }
+    switch (find_value_type_id(pzText - len, len)) {
+    default:
+    case VTP_KWD_INVALID: goto woops;
+
+    case VTP_KWD_STRING:
+        pType->valType = OPARG_TYPE_STRING;
+        break;
+
+    case VTP_KWD_INTEGER:
+        pType->valType = OPARG_TYPE_NUMERIC;
+        break;
+
+    case VTP_KWD_BOOL:
+    case VTP_KWD_BOOLEAN:
+        pType->valType = OPARG_TYPE_BOOLEAN;
+        break;
+
+    case VTP_KWD_KEYWORD:
+        pType->valType = OPARG_TYPE_ENUMERATION;
+        break;
+
+    case VTP_KWD_SET:
+    case VTP_KWD_SET_MEMBERSHIP:
+        pType->valType = OPARG_TYPE_MEMBERSHIP;
+        break;
+
+    case VTP_KWD_NESTED:
+    case VTP_KWD_HIERARCHY:
+        pType->valType = OPARG_TYPE_HIERARCHY;
     }
 
-    {
-        size_t len = strlen(zLtypeBool);
-        if (strncmp( pzText, zLtypeBool, len ) == 0) {
-            if ((pzText[len] == '>') || isspace(pzText[len])) {
-                pType->valType = OPARG_TYPE_BOOLEAN;
-                return pzText + len;
-            }
-            goto unknown;
-        }
-    }
-
-    {
-        size_t len = strlen(zLtypeKeyword);
-        if (strncmp( pzText, zLtypeKeyword, len ) == 0) {
-            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
-                pType->valType = OPARG_TYPE_ENUMERATION;
-                return pzText + len;
-            }
-            goto unknown;
-        }
-    }
-
-    {
-        size_t len = strlen(zLtypeSetMembership);
-        if (strncmp( pzText, zLtypeSetMembership, len ) == 0) {
-            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
-                pType->valType = OPARG_TYPE_MEMBERSHIP;
-                return pzText + len;
-            }
-            goto unknown;
-        }
-    }
-
-    {
-        size_t len = strlen(zLtypeNest);
-        if (strncmp( pzText, zLtypeNest, len ) == 0) {
-            if ((pzText[len] == '>') || isspace((int)pzText[len])) {
-                pType->valType = OPARG_TYPE_HIERARCHY;
-                return pzText + len;
-            }
-            goto unknown;
-        }
-    }
-
-  unknown:
-    pType->valType = OPARG_TYPE_NONE;
-    return skipUnknown( pzText );
+    return pzText;
 }
 
 
@@ -1202,12 +1161,8 @@ static char*
 skipUnknown( char* pzText )
 {
     for (;; pzText++) {
-        if (isspace( (int)*pzText ))  return pzText;
-        switch (*pzText) {
-        case NUL: return NULL;
-        case '/':
-        case '>': return pzText;
-        }
+        if (IS_END_XML_TOKEN_CHAR(*pzText))  return pzText;
+        if (*pzText == NUL) return NULL;
     }
 }
 
@@ -1233,7 +1188,14 @@ validateOptionsStruct( tOptions* pOpts, char const* pzProgram )
      *  is available, then go do it.
      */
     if (  ((pOpts->fOptSet & OPTPROC_TRANSLATE) != 0)
-       && (pOpts->pTransProc != 0) ) {
+       && (pOpts->pTransProc != NULL) ) {
+        /*
+         *  If option names are not to be translated at all, then do not do
+         *  it for configuration parsing either.  (That is the bit that really
+         *  gets tested anyway.)
+         */
+        if ((pOpts->fOptSet & OPTPROC_NO_XLAT_MASK) == OPTPROC_NXLAT_OPT)
+            pOpts->fOptSet |= OPTPROC_NXLAT_OPT_CFG;
         (*pOpts->pTransProc)();
         pOpts->fOptSet &= ~OPTPROC_TRANSLATE;
     }
@@ -1248,8 +1210,7 @@ validateOptionsStruct( tOptions* pOpts, char const* pzProgram )
           || (pOpts->structVersion < OPTIONS_MINIMUM_VERSION )
        )  )  {
 
-        fprintf( stderr, zAO_Err, pOpts->origArgVect[0],
-                 NUM_TO_VER( pOpts->structVersion ));
+        fprintf(stderr, zAO_Err, pzProgram, NUM_TO_VER(pOpts->structVersion));
         if (pOpts->structVersion > OPTIONS_STRUCT_VERSION )
             fputs( zAO_Big, stderr );
         else
