@@ -6,12 +6,13 @@
 #endif
 
 #include "ntpd.h"
-#include "ntp_io.h"
 #include "ntp_unixtime.h"
 #include "ntp_filegen.h"
 #include "ntp_if.h"
 #include "ntp_stdlib.h"
 #include "ntp_assert.h"
+#include "ntp_calendar.h"
+#include "lib_strbuf.h"
 
 #include <stdio.h>
 #include <ctype.h>
@@ -19,6 +20,10 @@
 #ifdef HAVE_SYS_IOCTL_H
 # include <sys/ioctl.h>
 #endif
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+#include <sys/stat.h>
 
 #ifdef HAVE_IEEEFP_H
 # include <ieeefp.h>
@@ -27,12 +32,6 @@
 # include <math.h>
 #endif
 
-#ifdef  DOSYNCTODR
-# if !defined(VMS)
-#  include <sys/resource.h>
-# endif /* VMS */
-#endif
-
 #if defined(VMS)
 # include <descrip.h>
 #endif /* VMS */
@@ -55,12 +54,13 @@
  * File names
  */
 static	char *key_file_name;		/* keys file name */
-char	*leapseconds_file_name;		/* leapseconds file name */
+char	*leapseconds_file;		/* leapseconds file name */
+struct stat leapseconds_file_sb1;	/* leapseconds file stat() buffer */
+struct stat leapseconds_file_sb2;	/* leapseconds file stat() buffer */
 char	*stats_drift_file;		/* frequency file name */
 static	char *stats_temp_file;		/* temp frequency file name */
-double wander_resid;			/* wander threshold */
-double	wander_threshold = 1e-7;	/* initial wander threshold */
-int	drift_file_sw;			/* clock update switch */
+static double wander_resid;		/* last frequency update */
+double	wander_threshold = 1e-7;	/* initial frequency threshold */
 
 /*
  * Statistics file stuff
@@ -73,24 +73,16 @@ int	drift_file_sw;			/* clock update switch */
 # endif /* SYS_WINNT */
 #endif
 
-#ifndef MAXPATHLEN
-# define MAXPATHLEN 256
-#endif
 
-#ifdef DEBUG_TIMING
-static FILEGEN timingstats;
-#endif
-#ifdef OPENSSL
-static FILEGEN cryptostats;
-#endif /* OPENSSL */
-
-static	char statsdir[MAXPATHLEN] = NTP_VAR;
+char statsdir[MAXFILENAME] = NTP_VAR;
 static FILEGEN peerstats;
 static FILEGEN loopstats;
 static FILEGEN clockstats;
 static FILEGEN rawstats;
 static FILEGEN sysstats;
 static FILEGEN protostats;
+static FILEGEN cryptostats;
+static FILEGEN timingstats;
 
 /*
  * This controls whether stats are written to the fileset. Provided
@@ -99,16 +91,16 @@ static FILEGEN protostats;
 int stats_control;
 
 /*
- * Initial frequency offset later passed to the loopfilter.
+ * Last frequency written to file.
  */
-double	old_drift = 1e9;		/* current frequency */
 static double prev_drift_comp;		/* last frequency update */
 
 /*
- * Static prototypes
+ * Function prototypes
  */
-static int leap_file(FILE *);
-static void record_sys_stats(void);
+static	int	leap_file(FILE *);
+static	void	record_sys_stats(void);
+	void	ntpd_time_stepped(void);
 
 /* 
  * Prototypes
@@ -125,6 +117,9 @@ void	uninit_util(void);
 void
 uninit_util(void)
 {
+#if defined(_MSC_VER) && defined (_DEBUG)
+	_CrtCheckMemory();
+#endif
 	if (stats_drift_file) {
 		free(stats_drift_file);
 		free(stats_temp_file);
@@ -141,39 +136,41 @@ uninit_util(void)
 	filegen_unregister("rawstats");
 	filegen_unregister("sysstats");
 	filegen_unregister("protostats");
-#ifdef OPENSSL
+#ifdef AUTOKEY
 	filegen_unregister("cryptostats");
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 #ifdef DEBUG_TIMING
 	filegen_unregister("timingstats");
-#endif /* DEBUG_TIMING */
+#endif	/* DEBUG_TIMING */
+
+#if defined(_MSC_VER) && defined (_DEBUG)
+	_CrtCheckMemory();
+#endif
 }
 #endif /* DEBUG */
 
 
 /*
- * init_util - initialize the utilities (ntpd included)
+ * init_util - initialize the util module of ntpd
  */
 void
 init_util(void)
 {
-	stats_drift_file = NULL;
-	stats_temp_file = NULL;
-	key_file_name = NULL;
-	filegen_register(statsdir, "peerstats",   &peerstats);
-	filegen_register(statsdir, "loopstats",   &loopstats);
+	filegen_register(statsdir, "peerstats",	  &peerstats);
+	filegen_register(statsdir, "loopstats",	  &loopstats);
 	filegen_register(statsdir, "clockstats",  &clockstats);
-	filegen_register(statsdir, "rawstats",    &rawstats);
-	filegen_register(statsdir, "sysstats",    &sysstats);
+	filegen_register(statsdir, "rawstats",	  &rawstats);
+	filegen_register(statsdir, "sysstats",	  &sysstats);
 	filegen_register(statsdir, "protostats",  &protostats);
-#ifdef OPENSSL
 	filegen_register(statsdir, "cryptostats", &cryptostats);
-#endif /* OPENSSL */
-#ifdef DEBUG_TIMING
 	filegen_register(statsdir, "timingstats", &timingstats);
-#endif /* DEBUG_TIMING */
+	/*
+	 * register with libntp ntp_set_tod() to call us back
+	 * when time is stepped.
+	 */
+	step_callback = &ntpd_time_stepped;
 #ifdef DEBUG
-	atexit(uninit_util);
+	atexit(&uninit_util);
 #endif /* DEBUG */
 }
 
@@ -185,14 +182,13 @@ void
 write_stats(void)
 {
 	FILE	*fp;
-	double	ftemp;
 #ifdef DOSYNCTODR
 	struct timeval tv;
 #if !defined(VMS)
 	int	prio_set;
 #endif
 #ifdef HAVE_GETCLOCK
-        struct timespec ts;
+	struct timespec ts;
 #endif
 	int	o_prio;
 
@@ -214,8 +210,8 @@ write_stats(void)
 	 * thing to do.
 	 *
 	 * CAVEAT: settimeofday() steps the sun clock by about 800 us,
-	 *         so setting DOSYNCTODR seems a bad idea in the
-	 *         case of us resolution
+	 *	   so setting DOSYNCTODR seems a bad idea in the
+	 *	   case of us resolution
 	 */
 
 #if !defined(VMS)
@@ -244,9 +240,9 @@ write_stats(void)
 		prio_set = 1;	/* overdrive */
 #endif /* VMS */
 #ifdef HAVE_GETCLOCK
-        (void) getclock(TIMEOFDAY, &ts);
-        tv.tv_sec = ts.tv_sec;
-        tv.tv_usec = ts.tv_nsec / 1000;
+	(void) getclock(TIMEOFDAY, &ts);
+	tv.tv_sec = ts.tv_sec;
+	tv.tv_usec = ts.tv_nsec / 1000;
 #else /*  not HAVE_GETCLOCK */
 	GETTIMEOFDAY(&tv,(struct timezone *)NULL);
 #endif /* not HAVE_GETCLOCK */
@@ -258,74 +254,71 @@ write_stats(void)
 #endif /* VMS */
 #endif /* DOSYNCTODR */
 	record_sys_stats();
-	ftemp = fabs(prev_drift_comp - drift_comp); 
-	prev_drift_comp = drift_comp;
-	if (ftemp > clock_phi)
-		return;
-
-	if (stats_drift_file != 0 && drift_file_sw) {
+	if (stats_drift_file != 0) {
 
 		/*
 		 * When the frequency file is written, initialize the
-		 * wander threshold to a configured initial value.
-		 * Thereafter reduce it by a factor of 0.5. When it
-		 * drops below the frequency wander, write the frequency
-		 * file. This adapts to the prevailing wander yet
-		 * minimizes the file writes.
+		 * prev_drift_comp and wander_resid. Thereafter,
+		 * reduce the wander_resid by half each hour. When
+		 * the difference between the prev_drift_comp and
+		 * drift_comp is less than the wander_resid, update
+		 * the frequncy file. This minimizes the file writes to
+		 * nonvolaile storage.
 		 */
-		drift_file_sw = FALSE;
-		wander_resid *= 0.5;
 #ifdef DEBUG
 		if (debug)
-			printf("write_stats: wander %.6lf thresh %.6lf, freq %.6lf\n",
-			    clock_stability * 1e6, wander_resid * 1e6,
-			    drift_comp * 1e6);
+			printf("write_stats: frequency %.6lf thresh %.6lf, freq %.6lf\n",
+			    (prev_drift_comp - drift_comp) * 1e6, wander_resid *
+			    1e6, drift_comp * 1e6);
 #endif
- 		if (sys_leap != LEAP_NOTINSYNC && clock_stability >
-		    wander_resid) {
-			wander_resid = wander_threshold;
-			if ((fp = fopen(stats_temp_file, "w")) == NULL)
-			    {
-				msyslog(LOG_ERR,
-				    "frequency file %s: %m",
-				    stats_temp_file);
-				return;
-			}
-			fprintf(fp, "%.3f\n", drift_comp * 1e6);
-			(void)fclose(fp);
-			/* atomic */
+		if (fabs(prev_drift_comp - drift_comp) < wander_resid) {
+			wander_resid *= 0.5;
+			return;
+		}
+		prev_drift_comp = drift_comp;
+		wander_resid = wander_threshold;
+		if ((fp = fopen(stats_temp_file, "w")) == NULL) {
+			msyslog(LOG_ERR, "frequency file %s: %m",
+			    stats_temp_file);
+			return;
+		}
+		fprintf(fp, "%.3f\n", drift_comp * 1e6);
+		(void)fclose(fp);
+		/* atomic */
 #ifdef SYS_WINNT
-			(void) _unlink(stats_drift_file); /* rename semantics differ under NT */
+		if (_unlink(stats_drift_file)) /* rename semantics differ under NT */
+			msyslog(LOG_WARNING, 
+				"Unable to remove prior drift file %s, %m", 
+				stats_drift_file);
 #endif /* SYS_WINNT */
 
 #ifndef NO_RENAME
-			(void) rename(stats_temp_file,
-			    stats_drift_file);
+		if (rename(stats_temp_file, stats_drift_file))
+			msyslog(LOG_WARNING, 
+				"Unable to rename temp drift file %s to %s, %m", 
+				stats_temp_file, stats_drift_file);
 #else
-			/* we have no rename NFS of ftp in use */
-			if ((fp = fopen(stats_drift_file, "w")) ==
-			    NULL) {
-				msyslog(LOG_ERR,
-				    "frequency file %s: %m",
-				    stats_drift_file);
-				return;
-			}
+		/* we have no rename NFS of ftp in use */
+		if ((fp = fopen(stats_drift_file, "w")) ==
+		    NULL) {
+			msyslog(LOG_ERR,
+			    "frequency file %s: %m",
+			    stats_drift_file);
+			return;
+		}
 #endif
 
 #if defined(VMS)
-			/* PURGE */
-			{
-				$DESCRIPTOR(oldvers,";-1");
-				struct dsc$descriptor driftdsc = {
-					strlen(stats_drift_file), 0, 0,
-					    stats_drift_file };
-				while(lib$delete_file(&oldvers,
-				    &driftdsc) & 1);
-			}
-#endif
-		} else {
-			/* XXX: Log a message at INFO level */
+		/* PURGE */
+		{
+			$DESCRIPTOR(oldvers,";-1");
+			struct dsc$descriptor driftdsc = {
+				strlen(stats_drift_file), 0, 0,
+				    stats_drift_file };
+			while(lib$delete_file(&oldvers,
+			    &driftdsc) & 1);
 		}
+#endif
 	}
 }
 
@@ -339,9 +332,16 @@ stats_config(
 	const char *invalue	/* only one type so far */
 	)
 {
-	FILE *fp;
+	FILE	*fp;
 	const char *value;
-	int len;
+	int	len;
+	double	old_drift;
+	l_fp	now;
+#ifndef VMS
+	const char temp_ext[] = ".TEMP";
+#else
+	const char temp_ext[] = "-TEMP";
+#endif
 
 	/*
 	 * Expand environment strings under Windows NT, since the
@@ -351,71 +351,58 @@ stats_config(
 	char newvalue[MAX_PATH], parameter[MAX_PATH];
 
 	if (!ExpandEnvironmentStrings(invalue, newvalue, MAX_PATH)) {
- 		switch(item) {
-		    case STATS_FREQ_FILE:
-			strcpy(parameter,"STATS_FREQ_FILE");
+		switch (item) {
+		case STATS_FREQ_FILE:
+			strlcpy(parameter, "STATS_FREQ_FILE",
+				sizeof(parameter));
 			break;
 
-		    case STATS_LEAP_FILE:
-			strcpy(parameter,"STATS_LEAP_FILE");
+		case STATS_LEAP_FILE:
+			strlcpy(parameter, "STATS_LEAP_FILE",
+				sizeof(parameter));
 			break;
 
-		    case STATS_STATSDIR:
-			strcpy(parameter,"STATS_STATSDIR");
+		case STATS_STATSDIR:
+			strlcpy(parameter, "STATS_STATSDIR",
+				sizeof(parameter));
 			break;
 
-		    case STATS_PID_FILE:
-			strcpy(parameter,"STATS_PID_FILE");
+		case STATS_PID_FILE:
+			strlcpy(parameter, "STATS_PID_FILE",
+				sizeof(parameter));
 			break;
 
-		    default:
-			strcpy(parameter,"UNKNOWN");
+		default:
+			strlcpy(parameter, "UNKNOWN",
+				sizeof(parameter));
 			break;
 		}
 		value = invalue;
 		msyslog(LOG_ERR,
-		    "ExpandEnvironmentStrings(%s) failed: %m\n",
-		    parameter);
+			"ExpandEnvironmentStrings(%s) failed: %m\n",
+			parameter);
 	} else {
 		value = newvalue;
 	}
-#else    
+#else	 
 	value = invalue;
 #endif /* SYS_WINNT */
 
-	switch(item) {
+	switch (item) {
 
 	/*
 	 * Open and read frequency file.
 	 */
 	case STATS_FREQ_FILE:
-		if (stats_drift_file) {
-			free(stats_drift_file);
-			free(stats_temp_file);
-			stats_drift_file = NULL;
-			stats_temp_file = NULL;
-		}
+		if (!value || (len = strlen(value)) == 0)
+			break;
 
-		if (value == 0 || (len = strlen(value)) == 0)
-		    break;
-
-		stats_drift_file = (char*)emalloc((u_int)(len + 1));
-#if !defined(VMS)
-		stats_temp_file = (char*)emalloc((u_int)(len +
-		    sizeof(".TEMP")));
-#else
-		stats_temp_file = (char*)emalloc((u_int)(len +
-		    sizeof("-TEMP")));
-#endif /* VMS */
-		memmove(stats_drift_file, value, (unsigned)(len+1));
-		memmove(stats_temp_file, value, (unsigned)len);
-#if !defined(VMS)
-		memmove(stats_temp_file + len, ".TEMP",
-		    sizeof(".TEMP"));
-#else
-		memmove(stats_temp_file + len, "-TEMP",
-		    sizeof("-TEMP"));
-#endif /* VMS */
+		stats_drift_file = erealloc(stats_drift_file, len + 1);
+		stats_temp_file = erealloc(stats_temp_file, 
+		    len + sizeof(".TEMP"));
+		memcpy(stats_drift_file, value, (size_t)(len+1));
+		memcpy(stats_temp_file, value, (size_t)len);
+		memcpy(stats_temp_file + len, temp_ext, sizeof(temp_ext));
 
 		/*
 		 * Open drift file and read frequency. If the file is
@@ -426,15 +413,15 @@ stats_config(
 
 		if (fscanf(fp, "%lf", &old_drift) != 1) {
 			msyslog(LOG_ERR,
-			    "format error frequency file %s", 
-			    stats_drift_file);
+				"format error frequency file %s", 
+				stats_drift_file);
 			fclose(fp);
 			break;
 
 		}
 		fclose(fp);
-		old_drift /= 1e6;
-		prev_drift_comp = old_drift;
+		loop_config(LOOP_FREQ, old_drift);
+		prev_drift_comp = drift_comp;
 		break;
 
 	/*
@@ -442,87 +429,30 @@ stats_config(
 	 */
 	case STATS_STATSDIR:
 
-		/*
-		 * HMS: the following test is insufficient:
-		 * - value may be missing the DIR_SEP
-		 * - we still need the filename after it
-		 */
-		if (strlen(value) >= sizeof(statsdir)) {
+		/* - 1 since value may be missing the DIR_SEP. */
+		if (strlen(value) >= sizeof(statsdir) - 1) {
 			msyslog(LOG_ERR,
 			    "statsdir too long (>%d, sigh)",
-			    (int)sizeof(statsdir) - 1);
+			    (int)sizeof(statsdir) - 2);
 		} else {
-			l_fp now;
 			int add_dir_sep;
-			int value_l = strlen(value);
+			int value_l;
 
 			/* Add a DIR_SEP unless we already have one. */
-			if (value_l == 0)
-				add_dir_sep = 0;
+			value_l = strlen(value);
+			if (0 == value_l)
+				add_dir_sep = FALSE;
 			else
 				add_dir_sep = (DIR_SEP !=
 				    value[value_l - 1]);
 
 			if (add_dir_sep)
-			    snprintf(statsdir, sizeof(statsdir),
-				"%s%c", value, DIR_SEP);
+				snprintf(statsdir, sizeof(statsdir),
+				    "%s%c", value, DIR_SEP);
 			else
-			    snprintf(statsdir, sizeof(statsdir),
-				"%s", value);
-
-			get_systime(&now);
-			if(peerstats.prefix == &statsdir[0] &&
-			    peerstats.fp != NULL) {
-				fclose(peerstats.fp);
-				peerstats.fp = NULL;
-				filegen_setup(&peerstats, now.l_ui);
-			}
-			if(loopstats.prefix == &statsdir[0] &&
-			    loopstats.fp != NULL) {
-				fclose(loopstats.fp);
-				loopstats.fp = NULL;
-				filegen_setup(&loopstats, now.l_ui);
-			}
-			if(clockstats.prefix == &statsdir[0] &&
-			    clockstats.fp != NULL) {
-				fclose(clockstats.fp);
-				clockstats.fp = NULL;
-				filegen_setup(&clockstats, now.l_ui);
-			}
-			if(rawstats.prefix == &statsdir[0] &&
-			    rawstats.fp != NULL) {
-				fclose(rawstats.fp);
-				rawstats.fp = NULL;
-				filegen_setup(&rawstats, now.l_ui);
-			}
-			if(sysstats.prefix == &statsdir[0] &&
-			    sysstats.fp != NULL) {
-				fclose(sysstats.fp);
-				sysstats.fp = NULL;
-				filegen_setup(&sysstats, now.l_ui);
-			}
-			if(protostats.prefix == &statsdir[0] &&
-			    protostats.fp != NULL) {
-				fclose(protostats.fp);
-				protostats.fp = NULL;
-				filegen_setup(&protostats, now.l_ui);
-			}
-#ifdef OPENSSL
-			if(cryptostats.prefix == &statsdir[0] &&
-			    cryptostats.fp != NULL) {
-				fclose(cryptostats.fp);
-				cryptostats.fp = NULL;
-				filegen_setup(&cryptostats, now.l_ui);
-			}
-#endif /* OPENSSL */
-#ifdef DEBUG_TIMING
-			if(timingstats.prefix == &statsdir[0] &&
-			    timingstats.fp != NULL) {
-				fclose(timingstats.fp);
-				timingstats.fp = NULL;
-				filegen_setup(&timingstats, now.l_ui);
-			}
-#endif /* DEBUG_TIMING */
+				snprintf(statsdir, sizeof(statsdir),
+				    "%s", value);
+			filegen_statsdir();
 		}
 		break;
 
@@ -543,20 +473,37 @@ stats_config(
 	 * Read leapseconds file.
 	 */
 	case STATS_LEAP_FILE:
-		if ((fp = fopen(value, "r")) == NULL) {
+		if (!value || (len = strlen(value)) == 0)
+			break;
+
+		leapseconds_file = erealloc(leapseconds_file, len + 1);
+		memcpy(leapseconds_file, value, len + 1);
+
+		if ((fp = fopen(leapseconds_file, "r")) == NULL) {
 			msyslog(LOG_ERR, "leapseconds file %s: %m",
-			    value);
+			    leapseconds_file);
 			break;
 		}
 
-		if (leap_file(fp) < 0)
+		if (-1 == fstat(fileno(fp), &leapseconds_file_sb1)) {
+			msyslog(LOG_ERR,
+			    "leapseconds: stat(%s) failed: %m",
+			    leapseconds_file);
+		} else if (leap_file(fp) < 0) {
 			msyslog(LOG_ERR,
 			    "format error leapseconds file %s",
-			    value);
-		else
-			msyslog(LOG_NOTICE,
-			    "leap epoch %lu expire %lu TAI offset %d from %s",
-			    leap_sec, leap_expire, leap_tai, value);
+			    leapseconds_file);
+		} else {
+			get_systime(&now);
+			mprintf_event(EVNT_TAI, NULL,
+				      "%d leap %s %s %s",
+				      leap_tai,
+				      fstostr(leap_sec),
+				      (now.l_ui > leap_expire)
+					  ? "expired"
+					  : "expires",
+				      fstostr(leap_expire));
+		}
 		fclose(fp);
 		break;
 
@@ -582,7 +529,7 @@ stats_config(
 */
 void
 record_peer_stats(
-	struct sockaddr_storage *addr,
+	sockaddr_u *addr,
 	int	status,
 	double	offset,		/* offset */
 	double	delay,		/* delay */
@@ -661,7 +608,7 @@ record_loop_stats(
  */
 void
 record_clock_stats(
-	struct sockaddr_storage *addr,
+	sockaddr_u *addr,
 	const char *text	/* timecode string */
 	)
 {
@@ -683,6 +630,30 @@ record_clock_stats(
 }
 
 
+/*
+ * mprintf_clock_stats - write clock statistics to file with
+ *			msnprintf-style formatting.
+ */
+int
+mprintf_clock_stats(
+	sockaddr_u *addr,
+	const char *fmt,
+	...
+	)
+{
+	va_list	ap;
+	int	rc;
+	char	msg[512];
+
+	va_start(ap, fmt);
+	rc = mvsnprintf(msg, sizeof(msg), fmt, ap);
+	va_end(ap);
+	if (stats_control)
+		record_clock_stats(addr, msg);
+
+	return rc;
+}
+
 /*
  * record_raw_stats - write raw timestamps to file
  *
@@ -695,12 +666,21 @@ record_clock_stats(
  */
 void
 record_raw_stats(
-        struct sockaddr_storage *srcadr,
-        struct sockaddr_storage *dstadr,
+	sockaddr_u *srcadr,
+	sockaddr_u *dstadr,
 	l_fp	*t1,		/* originate timestamp */
 	l_fp	*t2,		/* receive timestamp */
 	l_fp	*t3,		/* transmit timestamp */
-	l_fp	*t4		/* destination timestamp */
+	l_fp	*t4,		/* destination timestamp */
+	int	leap,
+	int	version,
+	int	mode,
+	int	stratum,
+	int	poll,
+	int	precision,
+	double	root_delay,	/* seconds */
+	double	root_dispersion,/* seconds */
+	u_int32	refid
 	)
 {
 	l_fp	now;
@@ -714,10 +694,13 @@ record_raw_stats(
 	day = now.l_ui / 86400 + MJD_1900;
 	now.l_ui %= 86400;
 	if (rawstats.fp != NULL) {
-                fprintf(rawstats.fp, "%lu %s %s %s %s %s %s %s\n", day,
-		    ulfptoa(&now, 3), stoa(srcadr), dstadr ? 
-		    stoa(dstadr) : "-",	ulfptoa(t1, 9), ulfptoa(t2, 9),
-		    ulfptoa(t3, 9), ulfptoa(t4, 9));
+		fprintf(rawstats.fp, "%lu %s %s %s %s %s %s %s %d %d %d %d %d %d %.6f %.6f %s\n",
+		    day, ulfptoa(&now, 3),
+		    stoa(srcadr), dstadr ?  stoa(dstadr) : "-",
+		    ulfptoa(t1, 9), ulfptoa(t2, 9),
+		    ulfptoa(t3, 9), ulfptoa(t4, 9),
+		    leap, version, mode, stratum, poll, precision,
+		    root_delay, root_dispersion, refid_str(refid, stratum));
 		fflush(rawstats.fp);
 	}
 }
@@ -755,7 +738,7 @@ record_sys_stats(void)
 	day = now.l_ui / 86400 + MJD_1900;
 	now.l_ui %= 86400;
 	if (sysstats.fp != NULL) {
-                fprintf(sysstats.fp,
+		fprintf(sysstats.fp,
 		    "%lu %s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
 		    day, ulfptoa(&now, 3), current_time - sys_stattime,
 		    sys_received, sys_processed, sys_newversion,
@@ -799,7 +782,7 @@ record_proto_stats(
 }
 
 
-#ifdef OPENSSL
+#ifdef AUTOKEY
 /*
  * record_crypto_stats - write crypto statistics to file
  *
@@ -811,7 +794,7 @@ record_proto_stats(
  */
 void
 record_crypto_stats(
-	struct sockaddr_storage *addr,
+	sockaddr_u *addr,
 	const char *text	/* text message */
 	)
 {
@@ -835,7 +818,7 @@ record_crypto_stats(
 		fflush(cryptostats.fp);
 	}
 }
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 
 #ifdef DEBUG_TIMING
@@ -873,6 +856,48 @@ record_timing_stats(
 #endif
 
 
+/*
+ *
+ */
+void
+check_leap_file(
+	void
+	)
+{
+	struct stat *sp1 = &leapseconds_file_sb1;
+	struct stat *sp2 = &leapseconds_file_sb2;
+
+	if (leapseconds_file) {
+		if (stat(leapseconds_file, &leapseconds_file_sb2)) {
+			msyslog(LOG_ERR,
+			    "check_leap_file: stat(%s): %m",
+			    leapseconds_file);
+			return;
+		}
+		if (   (sp1->st_mtime != sp2->st_mtime)
+		    || (sp1->st_ctime != sp2->st_ctime)) {
+			FILE *fp;
+
+			if ((fp = fopen(leapseconds_file, "r")) == NULL) {
+				msyslog(LOG_ERR,
+				    "check_leap_file: fopen(%s): %m",
+				    leapseconds_file);
+			} else {
+				leapseconds_file_sb1 = leapseconds_file_sb2;
+				if (leap_file(fp) < 0) {
+					msyslog(LOG_ERR,
+					    "format error leapseconds file %s",
+					    leapseconds_file);
+				}
+				fclose(fp);
+			}
+		}
+	}
+
+	return;
+}
+
+
 /*
  * leap_file - read leapseconds file
  *
@@ -897,7 +922,7 @@ leap_file(
 	 * integers followed by junk or comments. The first integer is
 	 * the NTP seconds at the leap, the second is the TAI offset
 	 * after the leap.
- 	 */
+	 */
 	offset = 0;
 	leap = 0;
 	expire = 0;
@@ -954,47 +979,25 @@ leap_month(
 	u_long	sec		/* current NTP second */
 	)
 {
-	u_long	ltemp;
-	u_long	*ptr;
-	u_long	year[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30,
-		    31}; 
-	u_long	lyear[] = {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30,
-		    31}; 
+	int	     leap;
+	int32	     year, month;
+	u_int32	     ndays;
+	ntpcal_split tmp;
+	vint64	     tvl;
 
-	/*
-	 * Find current leap cycle.
-	 */
-	ltemp = sec;
-	while (ltemp >= L_CENT)
-		ltemp -= L_CENT;
-	while (ltemp >= L_4YEAR)
-		ltemp -= L_4YEAR;
-
-	/*
-	 * We are within four years of the target. If in leap year, use
-	 * leap year month table; otherwise, use year month table.
-	 */
-	if (ltemp < L_LYEAR) {
-		ptr = lyear;
-	} else {
-		ptr = year;
-		ltemp -= L_LYEAR;
-		while (ltemp >= L_YEAR)
-			ltemp -= L_YEAR;
-	}
-
-	/*
-	 * We are within one year of the target. Find the month of the
-	 * leap.
-	 */
-	while (ltemp >= *ptr * L_DAY)
-		ltemp -= *ptr++ * L_DAY;
-
-	/*
-	 * The result is the number of seconds until the end of the
-	 * month when the leap is to occur.
-	 */
-	return (*ptr * L_DAY - ltemp - L_DAY);
+	/* --*-- expand time and split to days */
+	tvl   = ntpcal_ntp_to_ntp(sec, NULL);
+	tmp   = ntpcal_daysplit(&tvl);
+	/* --*-- split to years and days in year */
+	tmp   = ntpcal_split_eradays(tmp.hi + DAY_NTP_STARTS - 1, &leap);
+	year  = tmp.hi;
+	/* --*-- split days of year to month */
+	tmp   = ntpcal_split_yeardays(tmp.lo, leap);
+	month = tmp.hi;
+	/* --*-- get nominal start of next month */
+	ndays = ntpcal_edate_to_eradays(year, month+1, 0) + 1 - DAY_NTP_STARTS;
+	
+	return (u_int32)(ndays*SECSPERDAY - sec);
 }
 
 
@@ -1009,29 +1012,26 @@ getauthkeys(
 	int len;
 
 	len = strlen(keyfile);
-	if (len == 0)
+	if (!len)
 		return;
 	
-	if (key_file_name != NULL) {
-		free(key_file_name);
-		key_file_name = NULL;
-	}
-	if (key_file_name == NULL) {
 #ifndef SYS_WINNT
-		key_file_name = emalloc(len + 1);
-#else
-		key_file_name = emalloc(MAXPATHLEN);
-#endif
-	}
-#ifndef SYS_WINNT
- 	memmove(key_file_name, keyfile, len + 1);
+	key_file_name = erealloc(key_file_name, len + 1);
+	memcpy(key_file_name, keyfile, len + 1);
 #else
+	key_file_name = erealloc(key_file_name, _MAX_PATH);
+	if (len + 1 > _MAX_PATH)
+		return;
 	if (!ExpandEnvironmentStrings(keyfile, key_file_name,
-	    MAXPATHLEN)) {
+				      _MAX_PATH)) {
 		msyslog(LOG_ERR,
-		    "ExpandEnvironmentStrings(KEY_FILE) failed: %m\n");
+			"ExpandEnvironmentStrings(KEY_FILE) failed: %m");
+		strlcpy(key_file_name, keyfile, _MAX_PATH);
 	}
+	key_file_name = erealloc(key_file_name,
+				 1 + strlen(key_file_name));
 #endif /* SYS_WINNT */
+
 	authreadkeys(key_file_name);
 }
 
@@ -1042,58 +1042,11 @@ getauthkeys(
 void
 rereadkeys(void)
 {
-	if (key_file_name != NULL)
+	if (NULL != key_file_name)
 		authreadkeys(key_file_name);
 }
 
 
-/*
- * sock_hash - hash an sockaddr_storage structure
- */
-int
-sock_hash(
-	struct sockaddr_storage *addr
-	)
-{
-	int hashVal;
-	int i;
-	int len;
-	char *ch;
-	hashVal = 0;
-	len = 0;
-
-	/*
-	 * We can't just hash the whole thing because there are hidden
-	 * fields in sockaddr_in6 that might be filled in by recvfrom(),
-	 * so just use the family, port and address.
-	 */
-	ch = (char *)&addr->ss_family;
-	hashVal = 37 * hashVal + (int)*ch;
-	if (sizeof(addr->ss_family) > 1) {
-		ch++;
-		hashVal = 37 * hashVal + (int)*ch;
-	}
-	switch(addr->ss_family) {
-	case AF_INET:
-		ch = (char *)&((struct sockaddr_in *)addr)->sin_addr;
-		len = sizeof(struct in_addr);
-		break;
-
-	case AF_INET6:
-		ch = (char *)&((struct sockaddr_in6 *)addr)->sin6_addr;
-		len = sizeof(struct in6_addr);
-		break;
-	}
-
-	for (i = 0; i < len ; i++)
-		hashVal = 37 * hashVal + (int)*(ch + i);
-	hashVal = hashVal % 128;  /* % MON_HASH_SIZE hardcoded */
-	if (hashVal < 0)
-		hashVal += 128;
-	return hashVal;
-}
-
-
 #if notyet
 /*
  * ntp_exit - document explicitly that ntpd has exited
@@ -1105,3 +1058,54 @@ ntp_exit(int retval)
 	exit(retval);
 }
 #endif
+
+/*
+ * fstostr - prettyprint NTP seconds
+ */
+char * fstostr(
+	time_t	ntp_stamp
+	)
+{
+	char	*	buf;
+	struct tm *	tm;
+	time_t		unix_stamp;
+
+	LIB_GETBUF(buf);
+	unix_stamp = ntp_stamp - JAN_1970;
+	tm = gmtime(&unix_stamp);
+	if (NULL == tm)
+		msnprintf(buf, LIB_BUFLENGTH, "gmtime(%ld): %m",
+			  (long)unix_stamp);
+	else
+		snprintf(buf, LIB_BUFLENGTH, "%04d%02d%02d%02d%02d",
+			 tm->tm_year + 1900, tm->tm_mon + 1,
+			 tm->tm_mday, tm->tm_hour, tm->tm_min);
+
+	return buf;
+}
+
+
+/*
+ * ntpd_time_stepped is called back by step_systime(), allowing ntpd
+ * to do any one-time processing necessitated by the step.
+ */
+void
+ntpd_time_stepped(void)
+{
+	u_int saved_mon_enabled;
+
+	/*
+	 * flush the monitor MRU list which contains l_fp timestamps
+	 * which should not be compared across the step.
+	 */
+	if (MON_OFF != mon_enabled) {
+		saved_mon_enabled = mon_enabled;
+		mon_stop(MON_OFF);
+		mon_start(saved_mon_enabled);
+	}
+
+	/* inform interpolating Windows code to allow time to go back */
+#ifdef SYS_WINNT
+	win_time_stepped();
+#endif
+}
