@@ -5,6 +5,8 @@
 #include <config.h>
 #endif
 
+#include "ntp_types.h"
+
 #if defined(REFCLOCK) && defined(CLOCK_CHU)
 
 #include "ntpd.h"
@@ -32,17 +34,18 @@
  * This driver synchronizes the computer time using data encoded in
  * radio transmissions from Canadian time/frequency station CHU in
  * Ottawa, Ontario. Transmissions are made continuously on 3330 kHz,
- * 7335 kHz and 14670 kHz in upper sideband, compatible AM mode. An
+ * 7850 kHz and 14670 kHz in upper sideband, compatible AM mode. An
  * ordinary shortwave receiver can be tuned manually to one of these
  * frequencies or, in the case of ICOM receivers, the receiver can be
  * tuned automatically as propagation conditions change throughout the
  * day and season.
  *
- * The driver receives, demodulates and decodes the radio signals when
- * connected to the audio codec of a suported workstation hardware and
- * operating system. These include Solaris, SunOS, FreeBSD, NetBSD and
- * Linux. In this implementation, only one audio driver and codec can be
- * supported on a single machine.
+ * The driver requires an audio codec or sound card with sampling rate 8
+ * kHz and mu-law companding. This is the same standard as used by the
+ * telephone industry and is supported by most hardware and operating
+ * systems, including Solaris, SunOS, FreeBSD, NetBSD and Linux. In this
+ * implementation, only one audio driver and codec can be supported on a
+ * single machine.
  *
  * The driver can be compiled to use a Bell 103 compatible modem or
  * modem chip to receive the radio signal and demodulate the data.
@@ -51,7 +54,7 @@
  * latter case, the driver implements the modem using DSP routines, so
  * the radio can be connected directly to either the microphone on line
  * input port. In either case, the driver decodes the data using a
- * maximum likelihood technique which exploits the considerable degree
+ * maximum-likelihood technique which exploits the considerable degree
  * of redundancy available to maximize accuracy and minimize errors.
  *
  * The CHU time broadcast includes an audio signal compatible with the
@@ -163,7 +166,7 @@
  * dst	Canadian daylight code (opaque)
  * t	number of minutes since last synchronized
  * agc	audio gain (0 - 255)
- * ident identifier (CHU0 3330 kHz, CHU1 7335 kHz, CHU2 14670 kHz)
+ * ident identifier (CHU0 3330 kHz, CHU1 7850 kHz, CHU2 14670 kHz)
  * m	signal metric (0 - 100)
  * b	number of timecodes for the previous minute (0 - 59)
  *
@@ -194,6 +197,11 @@
  * 9600 bps if the high order 0x80 bit of the mode is zero and 1200 bps
  * if one. The C-IV trace is turned on if the debug level is greater
  * than one.
+ *
+ * Alarm codes
+ *
+ * CEVNT_BADTIME	invalid date or time
+ * CEVNT_PROP		propagation failure - no stations heard
  */
 /*
  * Interface definitions
@@ -218,7 +226,7 @@
 #define BAUD		300	/* modulation rate (bps) */
 #define OFFSET		128	/* companded sample offset */
 #define SIZE		256	/* decompanding table size */
-#define	MAXSGL		6000.	/* maximum signal level */
+#define	MAXAMP		6000.	/* maximum signal level */
 #define	MAXCLP		100	/* max clips above reference per s */
 #define	SPAN		800.	/* min envelope span */
 #define LIMIT		1000.	/* soft limiter threshold */
@@ -236,21 +244,27 @@
  */
 #define CHAR		(11. / 300.) /* character time (s) */
 #define BURST		11	/* max characters per burst */
-#define MINCHAR		9	/* min characters per burst */
+#define MINCHARS		9	/* min characters per burst */
 #define MINDIST		28	/* min burst distance (of 40)  */
 #define MINSYNC		8	/* min sync distance (of 16) */
 #define MINSTAMP	20	/* min timestamps (of 60) */
-#define METRIC		50	/* min channel metric (of 160) */
+#define MINMETRIC	50	/* min channel metric (of 160) */
 
 /*
- * The offset to the last stop bit of the first character, which defines
- * the timecode offset 170 ms. To this is added the receiver delay 4.7
- * ms, codec delay 1 ms and modem delay 0.8 ms. The receiver delay was
- * measured, the codec delay came from the AC97 spec sheet and the modem
- * delay was measured in simulation. The result is within 0.5 ms as
- * verified by on-air testing. 
+ * The on-time synchronization point for the driver is the last stop bit
+ * of the first character 170 ms. The modem delay is 0.8 ms, while the
+ * receiver delay is approxmately 4.7 ms at 2125 Hz. The fudge value 1.3
+ * ms due to the codec and other causes was determined by calibrating to
+ * a PPS signal from a GPS receiver. The additional propagation delay
+ * specific to each receiver location can be programmed in the fudge
+ * time1. 
+ *
+ * The resulting offsets with a 2.4-GHz P4 running FreeBSD 6.1 are
+ * generally within 0.5 ms short term with 0.3 ms jitter. The long-term
+ * offsets vary up to 0.3 ms due to ionospheric layer height variations.
+ * The processor load due to the driver is 0.4 percent.
  */
-#define	FUDGE		.1765	/* offset to first stop bit (s) */
+#define	PDELAY	((170 + .8 + 4.7 + 1.3) / 1000)	/* system delay (s) */
 
 /*
  * Status bits (status)
@@ -266,6 +280,7 @@
 #define AVALID		0x0100	/* valid A frame */
 #define BVALID		0x0200	/* valid B frame */
 #define INSYNC		0x0400	/* clock synchronized */
+#define	METRIC		0x0800	/* one or more stations heard */
 
 /*
  * Alarm status bits (alarm)
@@ -283,7 +298,7 @@
 
 #ifdef HAVE_AUDIO
 /*
- * Maximum likelihood UART structure. There are eight of these
+ * Maximum-likelihood UART structure. There are eight of these
  * corresponding to the number of phases.
  */ 
 struct surv {
@@ -312,7 +327,7 @@ struct xmtr {
  * CHU unit control structure
  */
 struct chuunit {
-	u_char	decode[20][16];	/* maximum likelihood decoding matrix */
+	u_char	decode[20][16];	/* maximum-likelihood decoding matrix */
 	l_fp	cstamp[BURST];	/* character timestamps */
 	l_fp	tstamp[MAXSTAGE]; /* timestamp samples */
 	l_fp	timestamp;	/* current buffer timestamp */
@@ -372,7 +387,7 @@ struct chuunit {
 	int	discptr;	/* discriminator pointer */
 
 	/*
-	 * Maximum likelihood UART variables
+	 * Maximum-likelihood UART variables
 	 */
 	double	baud;		/* baud interval */
 	struct surv surv[8];	/* UART survivor structures */
@@ -423,7 +438,7 @@ static char hexchar[] = "0123456789abcdef_*=";
  * transmits on USB with carrier so we can use AM and the narrow SSB
  * filter.
  */
-static double qsy[NCHAN] = {3.330, 7.335, 14.670}; /* freq (MHz) */
+static double qsy[NCHAN] = {3.330, 7.850, 14.670}; /* freq (MHz) */
 #endif /* ICOM */
 
 /*
@@ -462,7 +477,7 @@ chu_start(
 	double	step;		/* codec adjustment */
 
 	/*
-	 * Open audio device.
+	 * Open audio device. Don't complain if not there.
 	 */
 	fd_audio = audio_init(DEVICE_AUDIO, AUDIO_BUFSIZ, unit);
 #ifdef DEBUG
@@ -471,12 +486,12 @@ chu_start(
 #endif
 
 	/*
-	 * Open serial port in raw mode.
+	 * If audio is unavailable, Open serial port in raw mode.
 	 */
 	if (fd_audio > 0) {
 		fd = fd_audio;
 	} else {
-		sprintf(device, DEVICE, unit);
+		snprintf(device, sizeof(device), DEVICE, unit);
 		fd = refclock_open(device, SPEED232, LDISC_RAW);
 	}
 #else /* HAVE_AUDIO */
@@ -484,7 +499,7 @@ chu_start(
 	/*
 	 * Open serial port in raw mode.
 	 */
-	sprintf(device, DEVICE, unit);
+	snprintf(device, sizeof(device), DEVICE, unit);
 	fd = refclock_open(device, SPEED232, LDISC_RAW);
 #endif /* HAVE_AUDIO */
 	if (fd <= 0)
@@ -493,21 +508,18 @@ chu_start(
 	/*
 	 * Allocate and initialize unit structure
 	 */
-	if (!(up = (struct chuunit *)
-	      emalloc(sizeof(struct chuunit)))) {
-		close(fd);
-		return (0);
-	}
-	memset((char *)up, 0, sizeof(struct chuunit));
+	up = emalloc_zero(sizeof(*up));
 	pp = peer->procptr;
-	pp->unitptr = (caddr_t)up;
+	pp->unitptr = up;
 	pp->io.clock_recv = chu_receive;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
 		close(fd);
+		pp->io.fd = -1;
 		free(up);
+		pp->unitptr = NULL;
 		return (0);
 	}
 
@@ -516,7 +528,7 @@ chu_start(
 	 */
 	peer->precision = PRECISION;
 	pp->clockdesc = DESCRIPTION;
-	strcpy(up->ident, "CHU");
+	strlcpy(up->ident, "CHU", sizeof(up->ident));
 	memcpy(&pp->refid, up->ident, 4); 
 	DTOLFP(CHAR, &up->charstamp);
 #ifdef HAVE_AUDIO
@@ -556,16 +568,11 @@ chu_start(
 	}
 	if (up->fd_icom > 0) {
 		if (chu_newchan(peer, 0) != 0) {
-			NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
-			    msyslog(LOG_NOTICE,
-			    "icom: radio not found");
-			up->errflg = CEVNT_FAULT;
+			msyslog(LOG_NOTICE, "icom: radio not found");
 			close(up->fd_icom);
 			up->fd_icom = 0;
 		} else {
-			NLOG(NLOG_SYNCEVENT | NLOG_SYSEVENT)
-			    msyslog(LOG_NOTICE,
-			    "icom: autotune enabled");
+			msyslog(LOG_NOTICE, "icom: autotune enabled");
 		}
 	}
 #endif /* ICOM */
@@ -586,7 +593,7 @@ chu_shutdown(
 	struct refclockproc *pp;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (up == NULL)
 		return;
 
@@ -612,9 +619,9 @@ chu_receive(
 	struct refclockproc *pp;
 	struct peer *peer;
 
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * If the audio codec is warmed up, the buffer contains codec
@@ -652,9 +659,9 @@ chu_audio_receive(
 	int	bufcnt;		/* buffer counter */
 	l_fp	ltemp;		/* l_fp temp */
 
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Main loop - read until there ain't no more. Note codec
@@ -668,15 +675,15 @@ chu_audio_receive(
 		sample = up->comp[~*dpt++ & 0xff];
 
 		/*
-		 * Clip noise spikes greater than MAXSgl. If no clips,
+		 * Clip noise spikes greater than MAXAMP. If no clips,
 		 * increase the gain a tad; if the clips are too high, 
 		 * decrease a tad.
 		 */
-		if (sample > MAXSGL) {
-			sample = MAXSGL;
+		if (sample > MAXAMP) {
+			sample = MAXAMP;
 			up->clipcnt++;
-		} else if (sample < -MAXSGL) {
-			sample = -MAXSGL;
+		} else if (sample < -MAXAMP) {
+			sample = -MAXAMP;
 			up->clipcnt++;
 		}
 		chu_rf(peer, sample);
@@ -710,7 +717,7 @@ chu_audio_receive(
  *
  * This routine implements a 300-baud Bell 103 modem with mark 2225 Hz
  * and space 2025 Hz. It uses a bandpass filter followed by a soft
- * limiter, FM discriminator and lowpass filter. A maximum likelihood
+ * limiter, FM discriminator and lowpass filter. A maximum-likelihood
  * decoder samples the baseband signal at eight times the baud rate and
  * detects the start bit of each character.
  *
@@ -739,7 +746,7 @@ chu_rf(
 	int	i, j;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Bandpass filter. 4th-order elliptic, 500-Hz bandpass centered
@@ -821,7 +828,7 @@ chu_rf(
 	lpf += up->lpf[0] = disc * 2.538771e-02;
 
 	/*
-	 * Maximum likelihood decoder. The UART updates each of the
+	 * Maximum-likelihood decoder. The UART updates each of the
 	 * eight survivors and determines the span, slice level and
 	 * tentative decoded character. Valid 11-bit characters are
 	 * framed so that bit 10 and bit 11 (stop bits) are mark and bit
@@ -882,7 +889,7 @@ chu_rf(
 
 
 /*
- * chu_uart - maximum likelihood UART
+ * chu_uart - maximum-likelihood UART
  *
  * This routine updates a shift register holding the last 11 envelope
  * samples. It then computes the slice level and span over these samples
@@ -964,9 +971,9 @@ chu_serial_receive(
 
 	u_char	*dpt;		/* receive buffer pointer */
 
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	dpt = (u_char *)&rbufp->recv_space;
 	chu_decode(peer, *dpt, rbufp->recv_time);
@@ -990,7 +997,7 @@ chu_decode(
 	double	dtemp;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * If the interval since the last character is greater than the
@@ -1038,7 +1045,7 @@ chu_burst(
 	int	i;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Correlate a block of five characters with the next block of
@@ -1046,7 +1053,7 @@ chu_burst(
 	 * of bits that match in the two blocks for format A and that
 	 * match the inverse for format B.
 	 */
-	if (up->ndx < MINCHAR) {
+	if (up->ndx < MINCHARS) {
 		up->status |= RUNT;
 		return;
 	}
@@ -1095,10 +1102,13 @@ chu_b(
 
 	u_char	code[11];	/* decoded timecode */
 	char	tbuf[80];	/* trace buffer */
+	char *	p;
+	size_t	chars;
+	size_t	cb;
 	int	i;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * In a format B burst, a character is considered valid only if
@@ -1107,10 +1117,20 @@ chu_b(
 	 * only if the distance is 40. Note that once a valid frame has
 	 * been found errors are ignored.
 	 */
-	sprintf(tbuf, "chuB %04x %4.0f %2d %2d ", up->status,
-	    up->maxsignal, nchar, -up->burdist);
-	for (i = 0; i < nchar; i++)
-		sprintf(&tbuf[strlen(tbuf)], "%02x", up->cbuf[i]);
+	snprintf(tbuf, sizeof(tbuf), "chuB %04x %4.0f %2d %2d ",
+		 up->status, up->maxsignal, nchar, -up->burdist);
+	cb = sizeof(tbuf);
+	p = tbuf;
+	for (i = 0; i < nchar; i++) {
+		chars = strlen(p);
+		if (cb < chars + 1) {
+			msyslog(LOG_ERR, "chu_b() fatal out buffer");
+			exit(1);
+		}
+		cb -= chars;
+		p += chars;
+		snprintf(p, cb, "%02x", up->cbuf[i]);
+	}
 	if (pp->sloppyclockflag & CLK_FLAG4)
 		record_clock_stats(&peer->srcadr, tbuf);
 #ifdef DEBUG
@@ -1155,13 +1175,16 @@ chu_a(
 	struct chuunit *up;
 
 	char	tbuf[80];	/* trace buffer */
+	char *	p;
+	size_t	chars;
+	size_t	cb;
 	l_fp	offset;		/* timestamp offset */
 	int	val;		/* distance */
 	int	temp;
 	int	i, j, k;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Determine correct burst phase. There are three cases
@@ -1197,12 +1220,22 @@ chu_a(
 	if (temp < 2 || temp > 9 || k + 9 >= nchar || temp !=
 	    ((up->cbuf[k + 9] >> 4) & 0xf))
 		temp = 0;
-	sprintf(tbuf, "chuA %04x %4.0f %2d %2d %2d %2d %1d ",
-	    up->status, up->maxsignal, nchar, up->burdist, k,
-	    up->syndist, temp);
-	for (i = 0; i < nchar; i++)
-		sprintf(&tbuf[strlen(tbuf)], "%02x",
-		    up->cbuf[i]);
+	snprintf(tbuf, sizeof(tbuf),
+		 "chuA %04x %4.0f %2d %2d %2d %2d %1d ", up->status,
+		 up->maxsignal, nchar, up->burdist, k, up->syndist,
+		 temp);
+	cb = sizeof(tbuf);
+	p = tbuf;
+	for (i = 0; i < nchar; i++) {
+		chars = strlen(p);
+		if (cb < chars + 1) {
+			msyslog(LOG_ERR, "chu_a() fatal out buffer");
+			exit(1);
+		}
+		cb -= chars;
+		p += chars;
+		snprintf(p, cb, "%02x", up->cbuf[i]);
+	}
 	if (pp->sloppyclockflag & CLK_FLAG4)
 		record_clock_stats(&peer->srcadr, tbuf);
 #ifdef DEBUG
@@ -1227,7 +1260,7 @@ chu_a(
 		up->status |= AVALID;
 		up->second = pp->second = 30 + temp;
 		offset.l_ui = 30 + temp;
-		offset.l_f = 0;
+		offset.l_uf = 0;
 		i = 0;
 		if (k < 0)
 			offset = up->charstamp;
@@ -1302,7 +1335,7 @@ chu_second(
 	double	dtemp;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * This routine is called once per minute to process the
@@ -1331,7 +1364,7 @@ chu_second(
 		qual |= DECERR;
 	if (up->status & STAMP)
 		qual |= TSPERR;
-	if (up->status & BVALID && dtemp >= METRIC)
+	if (up->status & BVALID && dtemp >= MINMETRIC)
 		up->status |= INSYNC;
 	synchar = leapchar = ' ';
 	if (!(up->status & INSYNC)) {
@@ -1346,7 +1379,7 @@ chu_second(
 	} else {
 		pp->leap = LEAP_NOWARNING;
 	}
-	sprintf(pp->a_lastcode,
+	snprintf(pp->a_lastcode, sizeof(pp->a_lastcode),
 	    "%c%1X %04d %03d %02d:%02d:%02d %c%x %+d %d %d %s %.0f %d",
 	    synchar, qual, pp->year, pp->day, pp->hour, pp->minute,
 	    pp->second, leapchar, up->dst, up->dut, minset, up->gain,
@@ -1356,11 +1389,10 @@ chu_second(
 	/*
 	 * If in sync and the signal metric is above threshold, the
 	 * timecode is ipso fatso valid and can be selected to
-	 * discipline the clock. Be sure not to leave stray timestamps
-	 * around if signals are too weak or the clock time is invalid.
+	 * discipline the clock.
 	 */
 	if (up->status & INSYNC && !(up->status & (DECODE | STAMP)) &&
-	    dtemp > METRIC) {
+	    dtemp > MINMETRIC) {
 		if (!clocktime(pp->day, pp->hour, pp->minute, 0, GMT,
 		    up->tstamp[0].l_ui, &pp->yearstart, &offset.l_ui)) {
 			up->errflg = CEVNT_BADTIME;
@@ -1368,7 +1400,7 @@ chu_second(
 			offset.l_uf = 0;
 			for (i = 0; i < up->ntstamp; i++)
 				refclock_process_offset(pp, offset,
-				    up->tstamp[i], FUDGE +
+				up->tstamp[i], PDELAY +
 				    pp->fudgetime1);
 			pp->lastref = up->timestamp;
 			refclock_receive(peer);
@@ -1410,7 +1442,7 @@ chu_major(
 	int	i, j, k;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Majority decoder. Each burst encodes two replications at each
@@ -1418,7 +1450,7 @@ chu_major(
 	 * matrix encodes the number of occurences of each digit found
 	 * at the corresponding position. The maximum over all
 	 * occurrences at each position is the distance for this
-	 * position and the corresponding digit is the maximum
+	 * position and the corresponding digit is the maximum-
 	 * likelihood candidate. If the distance is not more than half
 	 * the total number of occurences, a majority has not been found
 	 * and the data are discarded. The decoding distance is defined
@@ -1471,14 +1503,14 @@ chu_clear(
 	int	i, j;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Clear stuff for the minute.
 	 */
 	up->ndx = up->prevsec = 0;
 	up->burstcnt = up->ntstamp = 0;
-	up->status &= INSYNC;
+	up->status &= INSYNC | METRIC;
 	for (i = 0; i < 20; i++) {
 		for (j = 0; j < 16; j++)
 			up->decode[i][j] = 0;
@@ -1504,11 +1536,11 @@ chu_newchan(
 	int	i;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * The radio can be tuned to three channels: 0 (3330 kHz), 1
-	 * (7335 kHz) and 2 (14670 kHz). There are five one-minute
+	 * (7850 kHz) and 2 (14670 kHz). There are five one-minute
 	 * dwells in each cycle. During the first dwell the radio is
 	 * tuned to one of the three channels to measure the channel
 	 * metric. The channel is selected as the one least recently
@@ -1532,16 +1564,15 @@ chu_newchan(
 	for (i = 0; i < NCHAN; i++) {
 		up->xmtr[i].probe++;
 		if (up->xmtr[i].metric > metric) {
+			up->status |= METRIC;
 			metric = up->xmtr[i].metric;
 			up->chan = i;
 		}
 	}
 
 	/*
-	 * Start the next dwell. If the first or no stations have been
-	 * heard, continue round-robin scan. If no stations are heard,
-	 * skew the minute by a few seconds in case we landed in the
-	 * middle of a burst.
+	 * Start the next dwell. If the first dwell or no stations have
+	 * been heard, continue round-robin scan.
 	 */
 	up->dwell = (up->dwell + 1) % DWELL;
 	if (up->dwell == 0 || metric == 0) {
@@ -1559,9 +1590,13 @@ chu_newchan(
 	 */
 	rval = icom_freq(up->fd_icom, peer->ttl & 0x7f, qsy[up->chan] +
 	    TUNE);
-	sprintf(up->ident, "CHU%d", up->chan);
+	snprintf(up->ident, sizeof(up->ident), "CHU%d", up->chan);
 	memcpy(&pp->refid, up->ident, 4); 
-	memcpy(&peer->refid, up->ident, 4); 
+	memcpy(&peer->refid, up->ident, 4);
+	if (metric == 0 && up->status & METRIC) {
+		up->status &= ~METRIC;
+		refclock_report(peer, CEVNT_PROP);
+	} 
 	return (rval);
 }
 #endif /* ICOM */
@@ -1603,11 +1638,12 @@ chu_dist(
 /*
  * chu_gain - adjust codec gain
  *
- * This routine is called once each second. If the signal envelope
- * amplitude is too low, the codec gain is bumped up by four units; if
- * too high, it is bumped down. The decoder is relatively insensitive to
- * amplitude, so this crudity works just fine. The input port is set and
- * the error flag is cleared, mostly to be ornery.
+ * This routine is called at the end of each second. During the second
+ * the number of signal clips above the MAXAMP threshold (6000). If
+ * there are no clips, the gain is bumped up; if there are more than
+ * MAXCLP clips (100), it is bumped down. The decoder is relatively
+ * insensitive to amplitude, so this crudity works just peachy. The
+ * routine also jiggles the input port and selectively mutes the
  */
 static void
 chu_gain(
@@ -1618,7 +1654,7 @@ chu_gain(
 	struct chuunit *up;
 
 	pp = peer->procptr;
-	up = (struct chuunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Apparently, the codec uses only the high order bits of the
@@ -1641,5 +1677,5 @@ chu_gain(
 
 
 #else
-int refclock_chu_bs;
+NONEMPTY_TRANSLATION_UNIT
 #endif /* REFCLOCK */
