@@ -35,7 +35,7 @@ extern int async_write(int, const void *, unsigned int);
  *
  * Input format:
  *
- *      hh:mm:ss   <cr>
+ *	hh:mm:ss   <cr>
  *
  * hh:mm:ss -- what you'd expect, with a 24 hour clock.  (Heck, that's the only
  * way it could get stupider.)  We take time on the <cr>.
@@ -64,11 +64,11 @@ extern int async_write(int, const void *, unsigned int);
  * Dumb clock control structure
  */
 struct dumbclock_unit {
-	u_char	  tcswitch;		     /* timecode switch */
-	l_fp	  laststamp;		     /* last receive timestamp */
-	u_char	  lasthour;		     /* last hour (for monitor) */
-	u_char	  linect;		     /* count ignored lines (for monitor */
-        struct tm ymd;			     /* struct tm for y/m/d only */
+	u_char	  tcswitch;	/* timecode switch */
+	l_fp	  laststamp;	/* last receive timestamp */
+	u_char	  lasthour;	/* last hour (for monitor) */
+	u_char	  linect;	/* count ignored lines (for monitor */
+	struct tm ymd;		/* struct tm for y/m/d only */
 };
 
 /*
@@ -115,24 +115,20 @@ dumbclock_start(
 	 * Open serial port. Don't bother with CLK line discipline, since
 	 * it's not available.
 	 */
-	(void)sprintf(device, DEVICE, unit);
+	snprintf(device, sizeof(device), DEVICE, unit);
 #ifdef DEBUG
 	if (debug)
 		printf ("starting Dumbclock with device %s\n",device);
 #endif
 	fd = refclock_open(device, SPEED232, 0);
-	if (fd < 0)
+	if (fd <= 0)
 		return (0);
 
 	/*
 	 * Allocate and initialize unit structure
 	 */
-	up = (struct dumbclock_unit *)emalloc(sizeof(struct dumbclock_unit));
-	if (up == NULL) {
-		(void) close(fd);
-		return (0);
-	}
-	memset((char *)up, 0, sizeof(struct dumbclock_unit));
+	up = emalloc(sizeof(*up));
+	memset(up, 0, sizeof(*up));
 	pp = peer->procptr;
 	pp->unitptr = (caddr_t)up;
 	pp->io.clock_recv = dumbclock_receive;
@@ -140,8 +136,10 @@ dumbclock_start(
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
-		(void) close(fd);
+		close(fd);
+		pp->io.fd = -1;
 		free(up);
+		pp->unitptr = NULL;
 		return (0);
 	}
 
@@ -153,14 +151,10 @@ dumbclock_start(
 	tm_time_p = gmtime(&now);
 #endif
 	if (tm_time_p)
-	{
-	    up->ymd = *tm_time_p;
-	}
+		up->ymd = *tm_time_p;
 	else
-	{
-	    return 0;
-	}
-	    
+		return 0;
+
 	/*
 	 * Initialize miscellaneous variables
 	 */
@@ -185,8 +179,10 @@ dumbclock_shutdown(
 
 	pp = peer->procptr;
 	up = (struct dumbclock_unit *)pp->unitptr;
-	io_closeclock(&pp->io);
-	free(up);
+	if (-1 != pp->io.fd)
+		io_closeclock(&pp->io);
+	if (NULL != up)
+		free(up);
 }
 
 
@@ -202,12 +198,12 @@ dumbclock_receive(
 	struct refclockproc *pp;
 	struct peer *peer;
 
-	l_fp	     trtmp;	/* arrival timestamp */
-	int          hours;	/* hour-of-day */
-	int	     minutes;	/* minutes-past-the-hour */
-	int          seconds;	/* seconds */
-	int	     temp;	/* int temp */
-	int          got_good;	/* got a good time flag */
+	l_fp	trtmp;		/* arrival timestamp */
+	int	hours;		/* hour-of-day */
+	int	minutes;	/* minutes-past-the-hour */
+	int	seconds;	/* seconds */
+	int	temp;		/* int temp */
+	int	got_good;	/* got a good time flag */
 
 	/*
 	 * Initialize pointers and read the timecode and timestamp
@@ -222,7 +218,7 @@ dumbclock_receive(
 			up->tcswitch = 1;
 			up->laststamp = trtmp;
 		} else
-		    up->tcswitch = 0;
+			up->tcswitch = 0;
 		return;
 	}
 	pp->lencode = (u_short)temp;
@@ -255,10 +251,12 @@ dumbclock_receive(
 
 	    /*
 	     * Convert to GMT for sites that distribute localtime.  This
-             * means we have to figure out what day it is.  Easier said
+	     * means we have to figure out what day it is.  Easier said
 	     * than done...
 	     */
 
+	    memset(&asserted_tm, 0, sizeof(asserted_tm));
+
 	    asserted_tm.tm_year  = up->ymd.tm_year;
 	    asserted_tm.tm_mon   = up->ymd.tm_mon;
 	    asserted_tm.tm_mday  = up->ymd.tm_mday;
@@ -383,4 +381,4 @@ dumbclock_poll(
 
 #else
 int refclock_dumbclock_bs;
-#endif /* REFCLOCK */
+#endif	/* defined(REFCLOCK) && defined(CLOCK_DUMBCLOCK) */
