@@ -221,17 +221,19 @@ heath_start(
 	/*
 	 * Open serial port
 	 */
-	sprintf(device, DEVICE, unit);
-	if (!(fd = refclock_open(device, speed[peer->ttl & 0x3],
-	    LDISC_REMOTE)))
+	snprintf(device, sizeof(device), DEVICE, unit);
+	fd = refclock_open(device, speed[peer->ttl & 0x3],
+			   LDISC_REMOTE);
+	if (fd <= 0)
 		return (0);
 	pp = peer->procptr;
 	pp->io.clock_recv = heath_receive;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
-		(void) close(fd);
+		close(fd);
+		pp->io.fd = -1;
 		return (0);
 	}
 
@@ -239,9 +241,8 @@ heath_start(
 	 * Initialize miscellaneous variables
 	 */
 	peer->precision = PRECISION;
-	peer->burst = NSTAGE;
 	pp->clockdesc = DESCRIPTION;
-	memcpy((char *)&pp->refid, REFID, 4);
+	memcpy(&pp->refid, REFID, 4);
 	return (1);
 }
 
@@ -258,7 +259,8 @@ heath_shutdown(
 	struct refclockproc *pp;
 
 	pp = peer->procptr;
-	io_closeclock(&pp->io);
+	if (-1 != pp->io.fd)
+		io_closeclock(&pp->io);
 }
 
 
@@ -280,7 +282,7 @@ heath_receive(
 	/*
 	 * Initialize pointers and read the timecode and timestamp
 	 */
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
 	pp->lencode = refclock_gtlin(rbufp, pp->a_lastcode, BMAX,
 	    &trtmp);
@@ -428,8 +430,6 @@ heath_poll(
 	if (write(pp->io.fd, "T", 1) != 1)
 		refclock_report(peer, CEVNT_FAULT);
 	ioctl(pp->io.fd, TIOCMBIS, (char *)&bits);
-	if (peer->burst > 0)
-		return;
 	if (pp->coderecv == pp->codeproc) {
 		refclock_report(peer, CEVNT_TIMEOUT);
 		return;
@@ -442,7 +442,6 @@ heath_poll(
 	    printf("heath: timecode %d %s\n", pp->lencode,
 		   pp->a_lastcode);
 #endif
-	peer->burst = MAXSTAGE;
 	pp->polls++;
 }
 
