@@ -52,17 +52,12 @@
 # include "ppsapi_timepps.h"
 #endif
 
-#ifdef XNTP_BIG_ENDIAN
+#ifdef WORDS_BIGENDIAN
 #define getshort(s) ((((s) & 0xff) << 8) | (((s) >> 8) & 0xff))
 #define putshort(s) ((((s) & 0xff) << 8) | (((s) >> 8) & 0xff))
 #else
-#define getshort(s) (s)
-#define putshort(s) (s)
-#endif
-
-/* XXX */
-#ifdef sun
-char *strerror(int);
+#define getshort(s) ((u_short)(s))
+#define putshort(s) ((u_short)(s))
 #endif
 
 /*
@@ -138,14 +133,15 @@ struct instance {
 static	void	jupiter_canmsg	(struct instance *, u_int);
 static	u_short	jupiter_cksum	(u_short *, u_int);
 static	int	jupiter_config	(struct instance *);
-static	void	jupiter_debug	(struct peer *, char *, char *, ...)
-    __attribute__ ((format (printf, 3, 4)));
+static	void	jupiter_debug	(struct peer *, const char *,
+				 const char *, ...)
+			__attribute__ ((format (printf, 3, 4)));
 static	char *	jupiter_parse_t	(struct instance *, u_short *);
 static	char *	jupiter_parse_gpos	(struct instance *, u_short *);
 static	void	jupiter_platform	(struct instance *, u_int);
 static	void	jupiter_poll	(int, struct peer *);
-static	void	jupiter_control	(int, struct refclockstat *, struct
-				    refclockstat *, struct peer *);
+static	void	jupiter_control	(int, const struct refclockstat *,
+				 struct refclockstat *, struct peer *);
 #ifdef HAVE_PPSAPI
 static	int	jupiter_ppsapi	(struct instance *);
 static	int	jupiter_pps	(struct instance *);
@@ -153,9 +149,9 @@ static	int	jupiter_pps	(struct instance *);
 static	int	jupiter_recv	(struct instance *);
 static	void	jupiter_receive (struct recvbuf *rbufp);
 static	void	jupiter_reqmsg	(struct instance *, u_int, u_int);
-static	void	jupiter_reqonemsg	(struct instance *, u_int);
+static	void	jupiter_reqonemsg(struct instance *, u_int);
 static	char *	jupiter_send	(struct instance *, struct jheader *);
-static	void	jupiter_shutdown	(int, struct peer *);
+static	void	jupiter_shutdown(int, struct peer *);
 static	int	jupiter_start	(int, struct peer *);
 
 /*
@@ -182,39 +178,35 @@ jupiter_start(
 {
 	struct refclockproc *pp;
 	struct instance *instance;
-	int fd = -1;
+	int fd;
 	char gpsdev[20];
 
 	/*
 	 * Open serial port
 	 */
-	(void)sprintf(gpsdev, DEVICE, unit);
+	snprintf(gpsdev, sizeof(gpsdev), DEVICE, unit);
 	fd = refclock_open(gpsdev, SPEED232, LDISC_RAW);
-	if (fd == 0) {
-		jupiter_debug(peer, "jupiter_start", "open %s: %s",
-		    gpsdev, strerror(errno));
+	if (fd <= 0) {
+		jupiter_debug(peer, "jupiter_start", "open %s: %m",
+			      gpsdev);
 		return (0);
 	}
 
 	/* Allocate unit structure */
-	if ((instance = (struct instance *)
-	    emalloc(sizeof(struct instance))) == NULL) {
-		(void) close(fd);
-		return (0);
-	}
-	memset((char *)instance, 0, sizeof(struct instance));
+	instance = emalloc_zero(sizeof(*instance));
 	instance->peer = peer;
 	pp = peer->procptr;
 	pp->io.clock_recv = jupiter_receive;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
-		(void) close(fd);
+		close(fd);
+		pp->io.fd = -1;
 		free(instance);
 		return (0);
 	}
-	pp->unitptr = (caddr_t)instance;
+	pp->unitptr = instance;
 
 	/*
 	 * Initialize miscellaneous variables
@@ -261,7 +253,7 @@ jupiter_shutdown(int unit, struct peer *peer)
 	struct refclockproc *pp;
 
 	pp = peer->procptr;
-	instance = (struct instance *)pp->unitptr;
+	instance = pp->unitptr;
 	if (!instance)
 		return;
 
@@ -272,7 +264,8 @@ jupiter_shutdown(int unit, struct peer *peer)
 	}
 #endif /* HAVE_PPSAPI */
 
-	io_closeclock(&pp->io);
+	if (pp->io.fd != -1)
+		io_closeclock(&pp->io);
 	free(instance);
 }
 
@@ -426,7 +419,7 @@ jupiter_pps(struct instance *instance)
 		return 1;
 	instance->ts = ts;
 
-	tstmp.l_ui = ts.tv_sec + JAN_1970;
+	tstmp.l_ui = (u_int32)ts.tv_sec + JAN_1970;
 	dtemp = ts.tv_nsec * FRAC / 1e9;
 	tstmp.l_uf = (u_int32)dtemp;
 	instance->peer->procptr->lastrec = tstmp;
@@ -444,7 +437,7 @@ jupiter_poll(int unit, struct peer *peer)
 	struct refclockproc *pp;
 
 	pp = peer->procptr;
-	instance = (struct instance *)pp->unitptr;
+	instance = pp->unitptr;
 
 	/*
 	 * You don't need to poll this clock.  It puts out timecodes
@@ -479,7 +472,7 @@ jupiter_poll(int unit, struct peer *peer)
 static void
 jupiter_control(
 	int unit,		/* unit (not used) */
-	struct refclockstat *in, /* input parameters (not used) */
+	const struct refclockstat *in, /* input parameters (not used) */
 	struct refclockstat *out, /* output parameters (not used) */
 	struct peer *peer	/* peer structure pointer */
 	)
@@ -489,7 +482,7 @@ jupiter_control(
 	u_char sloppyclockflag;
 
 	pp = peer->procptr;
-	instance = (struct instance *)pp->unitptr;
+	instance = pp->unitptr;
 
 	DTOLFP(pp->fudgetime2, &instance->limit);
 	/* Force positive value. */
@@ -534,15 +527,15 @@ jupiter_receive(struct recvbuf *rbufp)
 	l_fp tstamp;
 
 	/* Initialize pointers and read the timecode and timestamp */
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	instance = (struct instance *)pp->unitptr;
+	instance = pp->unitptr;
 
 	bp = (u_char *)rbufp->recv_buffer;
 	bpcnt = rbufp->recv_length;
 
 	/* This shouldn't happen */
-	if (bpcnt > sizeof(instance->sbuf) - instance->ssize)
+	if (bpcnt > (int)sizeof(instance->sbuf) - instance->ssize)
 		bpcnt = sizeof(instance->sbuf) - instance->ssize;
 
 	/* Append to input buffer */
@@ -616,7 +609,7 @@ jupiter_receive(struct recvbuf *rbufp)
 				break;
 
 			/* Add the new sample to a median filter */
-			tstamp.l_ui = JAN_1970 + last_timecode;
+			tstamp.l_ui = JAN_1970 + (u_int32)last_timecode;
 			tstamp.l_uf = 0;
 
 			refclock_process_offset(pp, tstamp, pp->lastrec, pp->fudgetime1);
@@ -877,36 +870,27 @@ jupiter_parse_gpos(struct instance *instance, u_short *sp)
 /*
  * jupiter_debug - print debug messages
  */
-#if defined(__STDC__) || defined(SYS_WINNT)
 static void
-jupiter_debug(struct peer *peer, char *function, char *fmt, ...)
-#else
-static void
-jupiter_debug(peer, function, fmt, va_alist)
-	struct peer *peer;
-	char *function;
-	char *fmt;
-#endif /* __STDC__ */
+jupiter_debug(
+	struct peer *	peer,
+	const char *	function,
+	const char *	fmt,
+	...
+	)
 {
-	char buffer[200];
-	va_list ap;
+	char	buffer[200];
+	va_list	ap;
 
-#if defined(__STDC__) || defined(SYS_WINNT)
 	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif /* __STDC__ */
 	/*
 	 * Print debug message to stdout
 	 * In the future, we may want to get get more creative...
 	 */
-	vsnprintf(buffer, sizeof(buffer), fmt, ap);
-	record_clock_stats(&(peer->srcadr), buffer);
+	mvsnprintf(buffer, sizeof(buffer), fmt, ap);
+	record_clock_stats(&peer->srcadr, buffer);
 #ifdef DEBUG
 	if (debug) {
-		fprintf(stdout, "%s: ", function);
-		fprintf(stdout, buffer);
-		fprintf(stdout, "\n");
+		printf("%s: %s\n", function, buffer);
 		fflush(stdout);
 	}
 #endif
@@ -934,10 +918,10 @@ jupiter_send(struct instance *instance, struct jheader *hp)
 	}
 
 	if ((cc = write(instance->peer->procptr->io.fd, (char *)hp, size)) < 0) {
-		(void)sprintf(errstr, "write: %s", strerror(errno));
+		msnprintf(errstr, sizeof(errstr), "write: %m");
 		return (errstr);
-	} else if (cc != size) {
-		(void)sprintf(errstr, "short write (%d != %d)", cc, size);
+	} else if (cc != (int)size) {
+		snprintf(errstr, sizeof(errstr), "short write (%d != %u)", cc, size);
 		return (errstr);
 	}
 	return (NULL);
