@@ -349,11 +349,7 @@ static	void	dump_buf	P((l_fp *coffs, int from, int to, char *text));
 static	void	ees_report_event P((struct eesunit *ees, int code));
 static	void	ees_receive	P((struct recvbuf *rbufp));
 static	void	ees_process	P((struct eesunit *ees));
-#ifdef QSORT_USES_VOID_P
 static	int	offcompare	P((const void *va, const void *vb));
-#else
-static	int	offcompare	P((const l_fp *a, const l_fp *b));
-#endif /* QSORT_USES_VOID_P */
 
 
 /*
@@ -382,11 +378,15 @@ dump_buf(
 	int i;
 	register char *ptr = buff;
 
-	sprintf(ptr, text);
-	for (i=from; i<to; i++)
-	{	while (*ptr) ptr++;
-	if ((ptr-buff) > DUMP_BUF_SIZE) msyslog(LOG_DEBUG, "D: %s", ptr=buff);
-	sprintf(ptr, " %06d", ((int)coffs[i].l_f) / 4295);
+	snprintf(buff, sizeof(buff), text);
+	for (i = from; i < to; i++) {
+		ptr += strlen(ptr);
+		if ((ptr - buff) > DUMP_BUF_SIZE) {
+			msyslog(LOG_DEBUG, "D: %s", buff);
+			ptr = buff;
+		}
+		snprintf(ptr, sizeof(buff) - (ptr - buff),
+			 " %06d", ((int)coffs[i].l_f) / 4295);
 	}
 	msyslog(LOG_DEBUG, "D: %s", buff);
 }
@@ -450,7 +450,7 @@ msfees_start(
 
 	/* Unit okay, attempt to open the devices.  We do them both at
 	 * once to make sure we can */
-	(void) sprintf(eesdev, EES232, unit);
+	snprintf(eesdev, sizeof(eesdev), EES232, unit);
 
 	fd232 = open(eesdev, O_RDWR, 0777);
 	if (fd232 == -1) {
@@ -533,7 +533,7 @@ msfees_start(
 	ees->timestarted= current_time;
 	ees->ttytype	= 0;
 	ees->io.clock_recv= ees_receive;
-	ees->io.srcclock= (caddr_t)ees;
+	ees->io.srcclock= ees;
 	ees->io.datalen	= 0;
 	ees->io.fd	= fd232;
 
@@ -585,7 +585,7 @@ msfees_start(
 		peer->refid = htonl(EESHSREFID);
 	}
 	unitinuse[unit] = 1;
-	pp->unitptr = (caddr_t) &eesunits[unit];
+	pp->unitptr = &eesunits[unit];
 	pp->clockdesc = EESDESCRIPTION;
 	msyslog(LOG_ERR, "ees clock: %s OK on %d", eesdev, unit);
 	return (1);
@@ -987,17 +987,20 @@ ees_receive(
 
 	/* Dump the deltas each minute */
 	if (dbg & DB_DUMP_DELTAS)
-	{	if (/*0 <= ees->second && */
-		ees->second < ((sizeof deltas) / (sizeof deltas[0]))) deltas[ees->second] = delta_sfsec;
+	{	
+		if (/*0 <= ees->second && */
+		    ees->second < COUNTOF(deltas))
+			deltas[ees->second] = delta_sfsec;
 	/* Dump on second 1, as second 0 sometimes missed */
 	if (ees->second == 1) {
-		char text[16 * ((sizeof deltas) / (sizeof deltas[0]))];
+		char text[16 * COUNTOF(deltas)];
 		char *cptr=text;
 		int i;
-		for (i=0; i<((sizeof deltas) / (sizeof deltas[0])); i++) {
-			sprintf(cptr, " %d.%04d",
-				msec(deltas[i]), subms(deltas[i]));
-			while (*cptr) cptr++;
+		for (i = 0; i < COUNTOF(deltas); i++) {
+			snprintf(cptr, sizeof(text) / COUNTOF(deltas),
+				" %d.%04d", msec(deltas[i]),
+				subms(deltas[i]));
+			cptr += strlen(cptr);
 		}
 		msyslog(LOG_ERR, "Deltas: %d.%04d<->%d.%04d: %s",
 			msec(EES_STEP_F - EES_STEP_F_GRACE), subms(EES_STEP_F - EES_STEP_F_GRACE),
@@ -1220,7 +1223,6 @@ ees_receive(
 
 /* offcompare - auxiliary comparison routine for offset sort */
 
-#ifdef QSORT_USES_VOID_P
 static int
 offcompare(
 	const void *va,
@@ -1231,16 +1233,6 @@ offcompare(
 	const l_fp *b = (const l_fp *)vb;
 	return(L_ISGEQ(a, b) ? (L_ISEQU(a, b) ? 0 : 1) : -1);
 }
-#else
-static int
-offcompare(
-	const l_fp *a,
-	const l_fp *b
-	)
-{
-	return(L_ISGEQ(a, b) ? (L_ISEQU(a, b) ? 0 : 1) : -1);
-}
-#endif /* QSORT_USES_VOID_P */
 
 
 /* ees_process - process a pile of samples from the clock */
@@ -1282,16 +1274,11 @@ ees_process(
 	if (samples < 1) return;
 
 	/* If requested, dump the raw data we have in the buffer */
-	if (ees->dump_vals) dump_buf(coffs, 0, samples, "Raw  data  is:");
+	if (ees->dump_vals)
+		dump_buf(coffs, 0, samples, "Raw  data  is:");
 
 	/* Sort the offsets, trim off the extremes, then choose one. */
-	qsort(
-#ifdef QSORT_USES_VOID_P
-	    (void *)
-#else
-	    (char *)
-#endif
-	    coffs, (size_t)samples, sizeof(l_fp), offcompare);
+	qsort(coffs, (size_t)samples, sizeof(coffs[0]), offcompare);
 
 	noff = samples;
 	i = 0;
