@@ -175,11 +175,7 @@ static	void	mx4200_poll	(int, struct peer *);
 static	char *	mx4200_parse_t	(struct peer *);
 static	char *	mx4200_parse_p	(struct peer *);
 static	char *	mx4200_parse_s	(struct peer *);
-#ifdef QSORT_USES_VOID_P
 int	mx4200_cmpl_fp	(const void *, const void *);
-#else
-int	mx4200_cmpl_fp	(const l_fp *, const l_fp *);
-#endif /* not QSORT_USES_VOID_P */
 static	int	mx4200_config	(struct peer *);
 static	void	mx4200_ref	(struct peer *);
 static	void	mx4200_send	(struct peer *, char *, ...)
@@ -222,31 +218,27 @@ mx4200_start(
 	/*
 	 * Open serial port
 	 */
-	(void)sprintf(gpsdev, DEVICE, unit);
-	if (!(fd = refclock_open(gpsdev, SPEED232, LDISC_PPS))) {
-	    return (0);
-	}
+	snprintf(gpsdev, sizeof(gpsdev), DEVICE, unit);
+	fd = refclock_open(gpsdev, SPEED232, LDISC_PPS);
+	if (fd <= 0)
+		return 0;
 
 	/*
 	 * Allocate unit structure
 	 */
-	if (!(up = (struct mx4200unit *) emalloc(sizeof(struct mx4200unit)))) {
-		perror("emalloc");
-		(void) close(fd);
-		return (0);
-	}
-	memset((char *)up, 0, sizeof(struct mx4200unit));
+	up = emalloc_zero(sizeof(*up));
 	pp = peer->procptr;
 	pp->io.clock_recv = mx4200_receive;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
-		(void) close(fd);
+		close(fd);
+		pp->io.fd = -1;
 		free(up);
 		return (0);
 	}
-	pp->unitptr = (caddr_t)up;
+	pp->unitptr = up;
 
 	/*
 	 * Initialize miscellaneous variables
@@ -273,9 +265,11 @@ mx4200_shutdown(
 	struct refclockproc *pp;
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
-	io_closeclock(&pp->io);
-	free(up);
+	up = pp->unitptr;
+	if (-1 != pp->io.fd)
+		io_closeclock(&pp->io);
+	if (NULL != up)
+		free(up);
 }
 
 
@@ -294,7 +288,7 @@ mx4200_config(
 	int mode;
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Initialize the unit variables
@@ -494,7 +488,7 @@ mx4200_ref(
 	char nsc, ewc;
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/* Should never happen! */
 	if (up->moving) return;
@@ -576,9 +570,9 @@ mx4200_ref(
 	}
 	alt = up->avg_alt;
 	minute = (lat - (double)(int)lat) * 60.0;
-	sprintf(lats,"%02d%02.4f", (int)lat, minute);
+	snprintf(lats, sizeof(lats), "%02d%02.4f", (int)lat, minute);
 	minute = (lon - (double)(int)lon) * 60.0;
-	sprintf(lons,"%03d%02.4f", (int)lon, minute);
+	snprintf(lons, sizeof(lons), "%03d%02.4f", (int)lon, minute);
 
 	mx4200_send(peer, "%s,%03d,,,,,%s,%c,%s,%c,%.2f,%d", pmvxg,
 	    PMVXG_S_INITMODEA,
@@ -612,7 +606,7 @@ mx4200_poll(
 	struct refclockproc *pp;
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * You don't need to poll this clock.  It puts out timecodes
@@ -679,9 +673,9 @@ mx4200_receive(
 	/*
 	 * Initialize pointers and read the timecode and timestamp.
 	 */
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * If operating mode has been changed, then reinitialize the receiver
@@ -845,7 +839,7 @@ mx4200_receive(
 		 * Capture the last PPS signal.
 		 * Precision timestamp is returned in pp->lastrec
 		 */
-		if (mx4200_pps(peer) != NULL) {
+		if (0 != mx4200_pps(peer)) {
 			mx4200_debug(peer, "mx4200_receive: pps failure\n");
 			refclock_report(peer, CEVNT_FAULT);
 			return;
@@ -961,18 +955,18 @@ mx4200_parse_t(
 	char   time_mark_valid, time_sync, op_mode;
 	int    sentence_type, valid;
 	int    year, day_of_year, month, day_of_month;
-	int    hour, minute, second, leapsec;
+	int    hour, minute, second, leapsec_warn;
 	int    oscillator_offset, time_mark_error, time_bias;
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
-	leapsec = 0;  /* Not all receivers output leap second warnings (!) */
+	leapsec_warn = 0;  /* Not all receivers output leap second warnings (!) */
 	sscanf(pp->a_lastcode,
 		"$PMVXG,%d,%c,%d,%d,%d,%d:%d:%d,%c,%c,%d,%d,%d,%d",
 		&sentence_type, &time_mark_valid, &year, &month, &day_of_month,
 		&hour, &minute, &second, &time_sync, &op_mode,
-		&oscillator_offset, &time_mark_error, &time_bias, &leapsec);
+		&oscillator_offset, &time_mark_error, &time_bias, &leapsec_warn);
 
 	if (sentence_type != PMVXG_D_TRECOVOUT)
 		return ("wrong rec-type");
@@ -1005,8 +999,8 @@ mx4200_parse_t(
 		mx4200_debug(peer,
 		    "mx4200_parse_t: bad time %02d:%02d:%02d",
 		    hour, minute, second);
-		if (leapsec != 0)
-			mx4200_debug(peer, " (leap %+d\n)", leapsec);
+		if (leapsec_warn != 0)
+			mx4200_debug(peer, " (leap %+d\n)", leapsec_warn);
 		mx4200_debug(peer, "\n");
 		refclock_report(peer, CEVNT_BADTIME);
 		return ("bad time");
@@ -1068,7 +1062,7 @@ mx4200_parse_t(
 	/*
 	 * Setup leap second indicator
 	 */
-	switch (leapsec) {
+	switch (leapsec_warn) {
 		case 0:
 			pp->leap = LEAP_NOWARNING;
 			break;
@@ -1085,12 +1079,12 @@ mx4200_parse_t(
 	/*
 	 * Any change to the leap second warning status?
 	 */
-	if (leapsec != up->last_leap ) {
+	if (leapsec_warn != up->last_leap ) {
 		msyslog(LOG_DEBUG,
 		    "mx4200: leap second warning: %d to %d (%d)",
-		    up->last_leap, leapsec, pp->leap);
+		    up->last_leap, leapsec_warn, pp->leap);
 	}
-	up->last_leap = leapsec;
+	up->last_leap = leapsec_warn;
 
 	/*
 	 * Copy time data for billboard monitoring.
@@ -1245,7 +1239,7 @@ mx4200_parse_p(
 	char   north_south, east_west;
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/* Should never happen! */
 	if (up->moving) return ("mobile platform - no pos!");
@@ -1464,7 +1458,7 @@ mx4200_parse_s(
 	int sentence_type;
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
         sscanf ( pp->a_lastcode, "$PMVXG,%d", &sentence_type);
 
@@ -1509,7 +1503,7 @@ mx4200_pps(
 	struct timespec timeout;
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Grab the timestamp of the PPS signal.
@@ -1520,14 +1514,14 @@ mx4200_pps(
 	if (time_pps_fetch(up->pps_h, PPS_TSFMT_TSPEC, &(up->pps_i),
 			&timeout) < 0) {
 		mx4200_debug(peer,
-		  "mx4200_pps: time_pps_fetch: serial=%ul, %s\n",
-		     (unsigned long)up->pps_i.assert_sequence, strerror(errno));
+		  "mx4200_pps: time_pps_fetch: serial=%lu, %m\n",
+		     (unsigned long)up->pps_i.assert_sequence);
 		refclock_report(peer, CEVNT_FAULT);
 		return(1);
 	}
 	if (temp_serial == up->pps_i.assert_sequence) {
 		mx4200_debug(peer,
-		   "mx4200_pps: assert_sequence serial not incrementing: %ul\n",
+		   "mx4200_pps: assert_sequence serial not incrementing: %lu\n",
 			(unsigned long)up->pps_i.assert_sequence);
 		refclock_report(peer, CEVNT_FAULT);
 		return(1);
@@ -1540,7 +1534,7 @@ mx4200_pps(
 		if (up->pps_i.assert_sequence == up->lastserial) {
 			mx4200_debug(peer, "mx4200_pps: no new pps event\n");
 		} else {
-			mx4200_debug(peer, "mx4200_pps: missed %ul pps events\n",
+			mx4200_debug(peer, "mx4200_pps: missed %lu pps events\n",
 			    up->pps_i.assert_sequence - up->lastserial - 1UL);
 		}
 		refclock_report(peer, CEVNT_FAULT);
@@ -1562,15 +1556,8 @@ mx4200_pps(
 /*
  * mx4200_debug - print debug messages
  */
-#if defined(__STDC__)
 static void
 mx4200_debug(struct peer *peer, char *fmt, ...)
-#else
-static void
-mx4200_debug(peer, fmt, va_alist)
-     struct peer *peer;
-     char *fmt;
-#endif /* __STDC__ */
 {
 #ifdef DEBUG
 	va_list ap;
@@ -1578,22 +1565,16 @@ mx4200_debug(peer, fmt, va_alist)
 	struct mx4200unit *up;
 
 	if (debug) {
-
-#if defined(__STDC__)
 		va_start(ap, fmt);
-#else
-		va_start(ap);
-#endif /* __STDC__ */
 
 		pp = peer->procptr;
-		up = (struct mx4200unit *)pp->unitptr;
-
+		up = pp->unitptr;
 
 		/*
 		 * Print debug message to stdout
 		 * In the future, we may want to get get more creative...
 		 */
-		vprintf(fmt, ap);
+		mvprintf(fmt, ap);
 
 		va_end(ap);
 	}
@@ -1630,7 +1611,7 @@ mx4200_send(peer, fmt, va_alist)
 #endif /* __STDC__ */
 
 	pp = peer->procptr;
-	up = (struct mx4200unit *)pp->unitptr;
+	up = pp->unitptr;
 
 	cp = buf;
 	*cp++ = '$';
