@@ -3,18 +3,11 @@
  * Refclock_neoclock4x.c
  * - NeoClock4X driver for DCF77 or FIA Timecode
  *
- * Date: 2006-01-11 v1.15
+ * Date: 2009-12-04 v1.16
  *
  * see http://www.linum.com/redir/jump/id=neoclock4x&action=redir
  * for details about the NeoClock4X device
  *
- * Copyright (C) 2002-2004 by Linum Software GmbH <neoclock4x@linum.com>
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -109,7 +102,7 @@
 #define NEOCLOCK4X_OFFSET_ANTENNA2         33
 #define NEOCLOCK4X_OFFSET_CRC              35
 
-#define NEOCLOCK4X_DRIVER_VERSION          "1.15 (2006-01-11)"
+#define NEOCLOCK4X_DRIVER_VERSION          "1.16 (2009-12-04)"
 
 #define NSEC_TO_MILLI                      1000000
 
@@ -138,20 +131,20 @@ struct neoclock4x_unit {
   int   utc_msec;
 };
 
-static	int	neoclock4x_start        (int, struct peer *);
+static	int	neoclock4x_start	(int, struct peer *);
 static	void	neoclock4x_shutdown	(int, struct peer *);
 static	void	neoclock4x_receive	(struct recvbuf *);
 static	void	neoclock4x_poll		(int, struct peer *);
-static	void	neoclock4x_control      (int, struct refclockstat *, struct refclockstat *, struct peer *);
+static	void	neoclock4x_control	(int, const struct refclockstat *, struct refclockstat *, struct peer *);
 
-static int      neol_atoi_len           (const char str[], int *, int);
-static int      neol_hexatoi_len        (const char str[], int *, int);
-static void     neol_jdn_to_ymd         (unsigned long, int *, int *, int *);
-static void     neol_localtime          (unsigned long, int* , int*, int*, int*, int*, int*);
-static unsigned long neol_mktime        (int, int, int, int, int, int);
+static int	neol_atoi_len		(const char str[], int *, int);
+static int	neol_hexatoi_len	(const char str[], int *, int);
+static void	neol_jdn_to_ymd		(unsigned long, int *, int *, int *);
+static void	neol_localtime		(unsigned long, int* , int*, int*, int*, int*, int*);
+static unsigned long neol_mktime	(int, int, int, int, int, int);
 #if !defined(NEOCLOCK4X_FIRMWARE)
-static int      neol_query_firmware     (int, int, char *, int);
-static int      neol_check_firmware     (int, const char*, char *);
+static int	neol_query_firmware	(int, int, char *, int);
+static int	neol_check_firmware	(int, const char*, char *);
 #endif
 
 struct refclock refclock_neoclock4x = {
@@ -324,10 +317,10 @@ neoclock4x_start(int unit,
 
   up->leap_status = 0;
   up->unit = unit;
-  strcpy(up->firmware, "?");
+  strncpy(up->firmware, "?", sizeof(up->firmware));
   up->firmwaretag = '?';
-  strcpy(up->serial, "?");
-  strcpy(up->radiosignal, "?");
+  strncpy(up->serial, "?", sizeof(up->serial));
+  strncpy(up->radiosignal, "?", sizeof(up->radiosignal));
   up->timesource  = '?';
   up->dststatus   = '?';
   up->quarzstatus = '?';
@@ -343,7 +336,8 @@ neoclock4x_start(int unit,
 
 #if defined(NEOCLOCK4X_FIRMWARE)
 #if NEOCLOCK4X_FIRMWARE == NEOCLOCK4X_FIRMWARE_VERSION_A
-  strcpy(up->firmware, "(c) 2002 NEOL S.A. FRANCE / L0.01 NDF:A:* (compile time)");
+  strncpy(up->firmware, "(c) 2002 NEOL S.A. FRANCE / L0.01 NDF:A:* (compile time)",
+	  sizeof(up->firmware));
   up->firmwaretag = 'A';
 #else
   msyslog(LOG_EMERG, "NeoClock4X(%d): unknown firmware defined at compile time for NeoClock4X",
@@ -678,7 +672,7 @@ neoclock4x_poll(int unit,
 
 static void
 neoclock4x_control(int unit,
-		   struct refclockstat *in,
+		   const struct refclockstat *in,
 		   struct refclockstat *out,
 		   struct peer *peer)
 {
@@ -938,13 +932,13 @@ neol_query_firmware(int fd,
 	  if(read_errors > 5)
 	    {
 	      msyslog(LOG_ERR, "NeoClock4X(%d): can't read firmware version (timeout)", unit);
-	      strcpy(tmpbuf, "unknown due to timeout");
+	      strncpy(tmpbuf, "unknown due to timeout", sizeof(tmpbuf));
 	      break;
 	    }
           if(chars_read > 500)
             {
 	      msyslog(LOG_ERR, "NeoClock4X(%d): can't read firmware version (garbage)", unit);
-	      strcpy(tmpbuf, "unknown due to garbage input");
+	      strncpy(tmpbuf, "unknown due to garbage input", sizeof(tmpbuf));
 	      break;
             }
 	  if(-1 == read(fd, &c, 1))
@@ -970,7 +964,7 @@ neol_query_firmware(int fd,
 	      if(0xA9 != c) /* wait for (c) char in input stream */
 		continue;
 
-	      strcpy(tmpbuf, "(c)");
+	      strncpy(tmpbuf, "(c)", sizeof(tmpbuf));
 	      len = 3;
 	      init = 0;
 	      continue;
@@ -1015,7 +1009,7 @@ neol_query_firmware(int fd,
   else
     {
       msyslog(LOG_ERR, "NeoClock4X(%d): can't query firmware version", unit);
-      strcpy(tmpbuf, "unknown error");
+      strncpy(tmpbuf, "unknown error", sizeof(tmpbuf));
     }
   strncpy(firmware, tmpbuf, maxlen);
   firmware[maxlen] = '\0';
@@ -1024,6 +1018,12 @@ neol_query_firmware(int fd,
     {
       NLOG(NLOG_CLOCKINFO)
 	msyslog(LOG_INFO, "NeoClock4X(%d): firmware version: %s", unit, firmware);
+
+      if(strstr(firmware, "/R2"))
+	{
+	  msyslog(LOG_INFO, "NeoClock4X(%d): Your NeoClock4X uses the new R2 firmware release. Please note the changed LED behaviour.", unit);
+	}
+
     }
 
   return (flag);
@@ -1110,4 +1110,16 @@ int refclock_neoclock4x_bs;
  * - remove some unsued #ifdefs
  * - fix nsec calculation, closes #499
  *
+ * 2009/12/04 cjh
+ * Revision 1.16
+ * - change license to ntp COPYRIGHT notice. This should allow Debian
+ *   to add this refclock driver in further releases.
+ * - detect R2 hardware
+ *
  */
+
+
+
+
+
+
