@@ -9,6 +9,11 @@
  *
  * Updated to process 'time1' fudge factor
  *		Venu Gopal May 05, 2008
+ *
+ * Converted to common PPSAPI code, separate PPS fudge time1
+ * from serial timecode fudge time2.
+ *		Dave Hart July 1, 2009
+ *		hart@ntp.org, davehart@davehart.com
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -16,6 +21,7 @@
 
 #if defined(REFCLOCK) && defined(CLOCK_NMEA)
 
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 
@@ -84,14 +90,60 @@ extern int async_write(int, const void *, unsigned int);
 /*
  * Definitions
  */
-#define DEVICE	   "/dev/gps%d"	/* name of radio device */
+#define	DEVICE		"/dev/gps%d"	/* GPS serial device */
+#define	PPSDEV		"/dev/gpspps%d"	/* PPSAPI device override */
 #define	SPEED232	B4800	/* uart speed (4800 bps) */
 #define	PRECISION	(-9)	/* precision assumed (about 2 ms) */
 #define	PPS_PRECISION	(-20)	/* precision assumed (about 1 us) */
 #define	REFID		"GPS\0"	/* reference id */
 #define	DESCRIPTION	"NMEA GPS Clock" /* who we are */
-#define NANOSECOND	1000000000 /* one second (ns) */
-#define RANGEGATE	500000	/* range gate (ns) */
+#ifndef O_NOCTTY
+#define M_NOCTTY	0
+#else
+#define M_NOCTTY	O_NOCTTY
+#endif
+#ifndef O_NONBLOCK
+#define M_NONBLOCK	0
+#else
+#define M_NONBLOCK	O_NONBLOCK
+#endif
+#define PPSOPENMODE	(O_RDWR | M_NOCTTY | M_NONBLOCK)
+
+/* NMEA sentence array indexes for those we use */
+#define NMEA_GPRMC	0	/* recommended min. nav. */
+#define NMEA_GPGGA	1	/* fix and quality */
+#define NMEA_GPGLL	2	/* geo. lat/long */
+#define NMEA_GPZDA	3	/* date/time */
+/*
+ * $GPZDG is a proprietary sentence that violates the spec, by not
+ * using $P and an assigned company identifier to prefix the sentence
+ * identifier.  When used with this driver, the system needs to be
+ * isolated from other NTP networks, as it operates in GPS time, not
+ * UTC as is much more common.  GPS time is >15 seconds different from
+ * UTC due to not respecting leap seconds since 1970 or so.  Other
+ * than the different timebase, $GPZDG is similar to $GPZDA.
+ */
+#define NMEA_GPZDG	4
+#define NMEA_ARRAY_SIZE (NMEA_GPZDG + 1)
+
+/*
+ * Sentence selection mode bits
+ */
+#define USE_ALL			0	/* any/all */
+#define USE_GPRMC		1
+#define USE_GPGGA		2
+#define USE_GPGLL		4
+#define USE_GPZDA_ZDG		8	/* affects both */
+
+/* mapping from sentence index to controlling mode bit */
+u_char sentence_mode[NMEA_ARRAY_SIZE] =
+{
+	USE_GPRMC,
+	USE_GPGGA,
+	USE_GPGLL,
+	USE_GPZDA_ZDG,
+	USE_GPZDA_ZDG
+};
 
 /*
  * Unit control structure
@@ -99,11 +151,16 @@ extern int async_write(int, const void *, unsigned int);
 struct nmeaunit {
 #ifdef HAVE_PPSAPI
 	struct refclock_atom atom; /* PPSAPI structure */
+	int	ppsapi_tried;	/* attempt PPSAPI once */
+	int	ppsapi_lit;	/* time_pps_create() worked */
+	int	ppsapi_fd;	/* fd used with PPSAPI */
 	int	tcount;		/* timecode sample counter */
 	int	pcount;		/* PPS sample counter */
 #endif /* HAVE_PPSAPI */
 	l_fp	tstamp;		/* timestamp of last poll */
 	int	gps_time;	/* 0 UTC, 1 GPS time */
+		/* per sentence checksum seen flag */
+	u_char	cksum_seen[NMEA_ARRAY_SIZE];
 };
 
 /*
@@ -114,7 +171,14 @@ static	void	nmea_shutdown	(int, struct peer *);
 static	void	nmea_receive	(struct recvbuf *);
 static	void	nmea_poll	(int, struct peer *);
 #ifdef HAVE_PPSAPI
+static	void	nmea_control	(int, const struct refclockstat *,
+				 struct refclockstat *, struct peer *);
 static  void	nmea_timer	(int, struct peer *);
+#define		NMEA_CONTROL	nmea_control
+#define		NMEA_TIMER	nmea_timer
+#else
+#define		NMEA_CONTROL	noentry
+#define		NMEA_TIMER	noentry
 #endif /* HAVE_PPSAPI */
 static	void	gps_send	(int, const char *, struct peer *);
 static	char *	field_parse	(char *, int);
@@ -127,14 +191,10 @@ struct	refclock refclock_nmea = {
 	nmea_start,		/* start up driver */
 	nmea_shutdown,		/* shut down driver */
 	nmea_poll,		/* transmit poll message */
-	noentry,		/* fudge control */
+	NMEA_CONTROL,		/* fudge control */
 	noentry,		/* initialize driver */
 	noentry,		/* buginfo */
-#ifdef HAVE_PPSAPI
-	nmea_timer,		/* called once per second */
-#else
-	noentry,		/* timer */
-#endif
+	NMEA_TIMER		/* called once per second */
 };
 
 /*
@@ -153,10 +213,12 @@ nmea_start(
 	int baudrate;
 	char *baudtext;
 
+	pp = peer->procptr;
+
 	/*
 	 * Open serial port. Use CLK line discipline, if available.
 	 */
-	(void)sprintf(device, DEVICE, unit);
+	snprintf(device, sizeof(device), DEVICE, unit);
 	
 	/*
 	 * Opening the serial port with appropriate baudrate
@@ -211,7 +273,7 @@ nmea_start(
 		 * ln -s server:port /dev/gps1
 		 */
 		char buffer[80];
-		char *nmea_host;
+		char *nmea_host, *nmea_tail;
 		int   nmea_port;
 		int   len;
 		struct hostent *he;
@@ -224,13 +286,16 @@ nmea_start(
 
 		if ((nmea_host = strtok(buffer,":")) == NULL)
 			return(0);
+		if ((nmea_tail = strtok(NULL,":")) == NULL)
+			return(0);
 
-		nmea_port = atoi(strtok(NULL,":"));
+		nmea_port = atoi(nmea_tail);
 
 		if ((he = gethostbyname(nmea_host)) == NULL)
 			return(0);
-		if ((p = getprotobyname("ip")) == NULL)
+		if ((p = getprotobyname("tcp")) == NULL)
 			return(0);
+		memset(&so_addr, 0, sizeof(so_addr));
 		so_addr.sin_family = AF_INET;
 		so_addr.sin_port = htons(nmea_port);
 		so_addr.sin_addr = *((struct in_addr *) he->h_addr);
@@ -242,28 +307,25 @@ nmea_start(
 			return (0);
 		}
 #else
+		pp->io.fd = -1;
 		return (0);
 #endif
 	}
 
-	msyslog(LOG_NOTICE, "refclock_nmea: serial %s open at %s bps",
-		device, baudtext);
+	msyslog(LOG_NOTICE, "%s serial %s open at %s bps",
+		refnumtoa(&peer->srcadr), device, baudtext);
 
 	/*
 	 * Allocate and initialize unit structure
 	 */
 	up = emalloc(sizeof(*up));
-	if (NULL == up) {
-		close(fd);
-		return (0);
-	}
 	memset(up, 0, sizeof(*up));
-	pp = peer->procptr;
 	pp->io.clock_recv = nmea_receive;
 	pp->io.srcclock = (caddr_t)peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
+		pp->io.fd = -1;
 		close(fd);
 		free(up);
 		return (0);
@@ -275,22 +337,19 @@ nmea_start(
 	 */
 	peer->precision = PRECISION;
 	pp->clockdesc = DESCRIPTION;
-	memcpy((char *)&pp->refid, REFID, 4);
+	memcpy(&pp->refid, REFID, 4);
 
 	gps_send(fd,"$PMOTG,RMC,0000*1D\r\n", peer);
 
-#ifdef HAVE_PPSAPI
-	/*
-	 * Light up the PPSAPI interface.
-	 */
-	return (refclock_ppsapi(fd, &up->atom));
-#else /* HAVE_PPSAPI */
 	return (1);
-#endif /* HAVE_PPSAPI */
 }
 
+
 /*
  * nmea_shutdown - shut down a GPS clock
+ * 
+ * NOTE this routine is called after nmea_start() returns failure,
+ * as well as during a normal shutdown due to ntpq :config unpeer.
  */
 static void
 nmea_shutdown(
@@ -305,10 +364,92 @@ nmea_shutdown(
 
 	pp = peer->procptr;
 	up = (struct nmeaunit *)pp->unitptr;
-	io_closeclock(&pp->io);
-	free(up);
+	if (up != NULL) {
+#ifdef HAVE_PPSAPI
+		if (up->ppsapi_lit) {
+			time_pps_destroy(up->atom.handle);
+			if (up->ppsapi_fd != pp->io.fd)
+				close(up->ppsapi_fd);
+		}
+#endif
+		free(up);
+	}
+	if (-1 != pp->io.fd)
+		io_closeclock(&pp->io);
 }
 
+/*
+ * nmea_control - configure fudge params
+ */
+#ifdef HAVE_PPSAPI
+static void
+nmea_control(
+	int unit,
+	const struct refclockstat *in_st,
+	struct refclockstat *out_st,
+	struct peer *peer
+	)
+{
+	char device[32];
+	register struct nmeaunit *up;
+	struct refclockproc *pp;
+	int pps_fd;
+	
+	UNUSED_ARG(in_st);
+	UNUSED_ARG(out_st);
+
+	pp = peer->procptr;
+	up = (struct nmeaunit *)pp->unitptr;
+
+	if (!(CLK_FLAG1 & pp->sloppyclockflag)) {
+		if (!up->ppsapi_tried)
+			return;
+		up->ppsapi_tried = 0;
+		if (!up->ppsapi_lit)
+			return;
+		peer->flags &= ~FLAG_PPS;
+		peer->precision = PRECISION;
+		time_pps_destroy(up->atom.handle);
+		if (up->ppsapi_fd != pp->io.fd)
+			close(up->ppsapi_fd);
+		up->atom.handle = 0;
+		up->ppsapi_lit = 0;
+		up->ppsapi_fd = -1;
+		return;
+	}
+
+	if (up->ppsapi_tried)
+		return;
+	/*
+	 * Light up the PPSAPI interface.
+	 */
+	up->ppsapi_tried = 1;
+
+	/*
+	 * if /dev/gpspps$UNIT can be opened that will be used for
+	 * PPSAPI.  Otherwise, the GPS serial device /dev/gps$UNIT
+	 * already opened is used for PPSAPI as well.
+	 */
+	snprintf(device, sizeof(device), PPSDEV, unit);
+
+	pps_fd = open(device, PPSOPENMODE, S_IRUSR | S_IWUSR);
+
+	if (-1 == pps_fd)
+		pps_fd = pp->io.fd;
+	
+	if (refclock_ppsapi(pps_fd, &up->atom)) {
+		up->ppsapi_lit = 1;
+		up->ppsapi_fd = pps_fd;
+		return;
+	}
+
+	NLOG(NLOG_CLOCKINFO)
+		msyslog(LOG_WARNING, "%s flag1 1 but PPSAPI fails",
+			refnumtoa(&peer->srcadr));
+}
+#endif	/* HAVE_PPSAPI */
+
+
 /*
  * nmea_timer - called once per second, fetches PPS
  *		timestamp and stuffs in median filter.
@@ -328,12 +469,11 @@ nmea_timer(
 	pp = peer->procptr;
 	up = (struct nmeaunit *)pp->unitptr;
 
-	if (pp->sloppyclockflag & CLK_FLAG1) {
-		if (refclock_pps(peer, &up->atom, pp->sloppyclockflag) >
-		    0) {
-			up->pcount++,
-			peer->flags |= FLAG_PPS;
-		}
+	if (up->ppsapi_lit &&
+	    refclock_pps(peer, &up->atom, pp->sloppyclockflag) > 0) {
+		up->pcount++,
+		peer->flags |= FLAG_PPS;
+		peer->precision = PPS_PRECISION;
 	}
 }
 #endif	/* HAVE_PPSAPI */
@@ -352,8 +492,7 @@ nmea_receive(
 	struct peer *peer;
 	int month, day;
 	char *cp, *dp, *msg;
-	int cmdtype;
-	int cmdtypezdg = 0;
+	u_char sentence;
 	/* Use these variables to hold data until we decide its worth keeping */
 	char	rd_lastcode[BMAX];
 	l_fp	rd_timestamp;
@@ -373,9 +512,10 @@ nmea_receive(
 			&rd_timestamp);
 
 	/*
-	 * There is a case that a <CR><LF> gives back a "blank" line
+	 * There is a case that a <CR><LF> gives back a "blank" line.
+	 * We can't have a well-formed sentence with less than 8 chars.
 	 */
-	if (rd_lencode == 0)
+	if (rd_lencode < 8)
 		return;
 
 	DPRINTF(1, ("nmea: gpsread %d %s\n", rd_lencode, rd_lastcode));
@@ -412,29 +552,24 @@ nmea_receive(
 	 *	   '1' indicates accuracy of +/-20 ms
 	 *	   '2' indicates accuracy of +/-100 ns
 	 */
-#define GPXXX		0	/* any/all */
-#define GPRMC		1
-#define GPGGA		2
-#define GPGLL		4
-#define GPZDG_ZDA	8
 
 	cp = rd_lastcode;
-	cmdtype=0;
 	if (cp[0] == '$') {
 		/* Allow for GLGGA and GPGGA etc. */
 		msg = cp + 3;
 
 		if (strncmp(msg, "RMC", 3) == 0)
-			cmdtype = GPRMC;
+			sentence = NMEA_GPRMC;
 		else if (strncmp(msg, "GGA", 3) == 0)
-			cmdtype = GPGGA;
+			sentence = NMEA_GPGGA;
 		else if (strncmp(msg, "GLL", 3) == 0)
-			cmdtype = GPGLL;
+			sentence = NMEA_GPGLL;
 		else if (strncmp(msg, "ZD", 2) == 0) {
-			cmdtype = GPZDG_ZDA;
 			if ('G' == msg[2])
-				cmdtypezdg = 1;
-			else if ('A' != msg[2])
+				sentence = NMEA_GPZDG;
+			else if ('A' == msg[2])
+				sentence = NMEA_GPZDA;
+			else
 				return;
 		} else
 			return;
@@ -442,7 +577,7 @@ nmea_receive(
 		return;
 
 	/* See if I want to process this message type */
-	if (peer->ttl && !(cmdtype & (peer->ttl & NMEA_MESSAGE_MASK)))
+	if (peer->ttl && !(peer->ttl & sentence_mode[sentence]))
 		return;
 
 	/* 
@@ -450,27 +585,41 @@ nmea_receive(
 	 * Once have processed a $GPZDG, do not process any further
 	 * UTC sentences (all but $GPZDG currently).
 	 */
-	if (up->gps_time && !cmdtypezdg)
+	if (up->gps_time && NMEA_GPZDG != sentence)
 		return;
 
-	/* make sure it came in clean */
-	if (!nmea_checksum_ok(rd_lastcode)) {
+	/*
+	 * Apparently, older NMEA specifications (which are expensive)
+	 * did not require the checksum for all sentences.  $GPMRC is
+	 * the only one so far identified which has always been required
+	 * to include a checksum.
+	 *
+	 * Today, most NMEA GPS receivers checksum every sentence.  To
+	 * preserve its error-detection capabilities with modern GPSes
+	 * while allowing operation without checksums on all but $GPMRC,
+	 * we keep track of whether we've ever seen a checksum on a
+	 * given sentence, and if so, reject future checksum failures.
+	 */
+	if (nmea_checksum_ok(rd_lastcode)) {
+		up->cksum_seen[sentence] = TRUE;
+	} else if (NMEA_GPRMC == sentence || up->cksum_seen[sentence]) {
 		refclock_report(peer, CEVNT_BADREPLY);
 		return;
 	}
 
-	pp->lencode = (u_short) rd_lencode;
+	pp->lencode = (u_short)rd_lencode;
 	memcpy(pp->a_lastcode, rd_lastcode, pp->lencode + 1);
 	cp = pp->a_lastcode;
 
-	pp->lastrec = up->tstamp = rd_timestamp;
+	up->tstamp = rd_timestamp;
+	pp->lastrec = up->tstamp;
 
 	DPRINTF(1, ("nmea: timecode %d %s\n", pp->lencode, pp->a_lastcode));
 
 	/* Grab field depending on clock string type */
-	switch (cmdtype) {
+	switch (sentence) {
 
-	case GPRMC:
+	case NMEA_GPRMC:
 		/*
 		 * Test for synchronization.  Check for quality byte.
 		 */
@@ -484,7 +633,7 @@ nmea_receive(
 		dp = field_parse(cp, 1);
 		break;
 
-	case GPGGA:
+	case NMEA_GPGGA:
 		/*
 		 * Test for synchronization.  Check for quality byte.
 		 */
@@ -498,7 +647,7 @@ nmea_receive(
 		dp = field_parse(cp, 1);
 		break;
 
-	case GPGLL:
+	case NMEA_GPGLL:
 		/*
 		 * Test for synchronization.  Check for quality byte.
 		 */
@@ -512,17 +661,17 @@ nmea_receive(
 		dp = field_parse(cp, 5);
 		break;
 	
-	case GPZDG_ZDA:
-		/*
-		 * Test for synchronization.  For $GPZDG check for validity of GPS time.
-		 */
-		if (cmdtypezdg) {
-			dp = field_parse(cp, 6);
-			if (dp[0] == '0') 
-				pp->leap = LEAP_NOTINSYNC;
-			else 
-				pp->leap = LEAP_NOWARNING;
-		} else
+	case NMEA_GPZDG:
+		/* For $GPZDG check for validity of GPS time. */
+		dp = field_parse(cp, 6);
+		if (dp[0] == '0') 
+			pp->leap = LEAP_NOTINSYNC;
+		else 
+			pp->leap = LEAP_NOWARNING;
+		/* fall through to NMEA_GPZDA */
+
+	case NMEA_GPZDA:
+		if (NMEA_GPZDA == sentence)
 			pp->leap = LEAP_NOWARNING;
 
 		/* Now point at the time field */
@@ -572,23 +721,6 @@ nmea_receive(
 		}
 	}
 
-	/*
-	 * Manipulating GPS timestamp in GPZDG as the seconds field
-	 * is valid for next PPS tick. Just rolling back the second,
-	 * minute and hour fields appopriately
-	 */
-	if (cmdtypezdg) {
-		if (pp->second == 0) {
-			pp->second = 59;
-			if (pp->minute == 0) {
-				pp->minute = 59;
-				if (pp->hour == 0)
-					pp->hour = 23;
-			}
-		} else
-			pp->second -= 1;
-	}
-
 	if (pp->hour > 23 || pp->minute > 59 || 
 	    pp->second > 59 || pp->nsec > 1000000000) {
 
@@ -598,10 +730,28 @@ nmea_receive(
 		return;
 	}
 
+	/*
+	 * Manipulating GPS timestamp in GPZDG as the seconds field
+	 * is valid for next PPS tick. Just rolling back the second,
+	 * minute and hour fields appopriately
+	 */
+	if (NMEA_GPZDG == sentence) {
+		if (pp->second == 0) {
+			pp->second = 59;
+			if (pp->minute == 0) {
+				pp->minute = 59;
+				if (pp->hour == 0)
+					pp->hour = 23;
+			}
+		} else {
+			pp->second--;
+		}
+	}
+
 	/*
 	 * Convert date and check values.
 	 */
-	if (GPRMC == cmdtype) {
+	if (NMEA_GPRMC == sentence) {
 
 		dp = field_parse(cp,9);
 		day = dp[0] - '0';
@@ -611,7 +761,7 @@ nmea_receive(
 		pp->year = dp[4] - '0';
 		pp->year = (pp->year * 10) + dp[5] - '0';
 
-	} else if (GPZDG_ZDA == cmdtype) {
+	} else if (NMEA_GPZDA == sentence || NMEA_GPZDG == sentence) {
 
 		dp = field_parse(cp, 2);
 		day = 10 * (dp[0] - '0') + (dp[1] - '0');
@@ -669,24 +819,25 @@ nmea_receive(
 		 * Start by pointing cp and dp at the fields with 
 		 * longitude and latitude in the last timecode.
 		 */
-		switch (cmdtype) {
+		switch (sentence) {
 
-		case GPGLL:
+		case NMEA_GPGLL:
 			cp = field_parse(pp->a_lastcode, 1);
 			dp = field_parse(cp, 2);
 			break;
 
-		case GPGGA:
+		case NMEA_GPGGA:
 			cp = field_parse(pp->a_lastcode, 2);
 			dp = field_parse(cp, 2);
 			break;
 
-		case GPRMC:
+		case NMEA_GPRMC:
 			cp = field_parse(pp->a_lastcode, 3);
 			dp = field_parse(cp, 2);
 			break;
 
-		case GPZDG_ZDA:
+		case NMEA_GPZDA:
+		case NMEA_GPZDG:
 		default:
 			cp = dp = NULL;
 		}
@@ -718,7 +869,7 @@ nmea_receive(
 	/*
 	 * Note if we're only using GPS timescale from now on.
 	 */
-	if (cmdtypezdg && !up->gps_time) {
+	if (!up->gps_time && NMEA_GPZDG == sentence) {
 		up->gps_time = 1;
 		NLOG(NLOG_CLOCKINFO)
 			msyslog(LOG_INFO, "%s using only $GPZDG",
@@ -734,7 +885,7 @@ nmea_receive(
 	if (peer->flags & FLAG_PPS)
 		return;
 #endif /* HAVE_PPSAPI */
-	if (!refclock_process(pp))
+	if (!refclock_process_f(pp, pp->fudgetime2))
 		refclock_report(peer, CEVNT_BADTIME);
 }
 
@@ -762,8 +913,10 @@ nmea_poll(
 	 * timeout and keep going.
 	 */
 #ifdef HAVE_PPSAPI
-	if (up->pcount == 0)
+	if (up->pcount == 0) {
 		peer->flags &= ~FLAG_PPS;
+		peer->precision = PRECISION;
+	}
 	if (up->tcount == 0) {
 		pp->coderecv = pp->codeproc;
 		refclock_report(peer, CEVNT_TIMEOUT);
