@@ -56,22 +56,22 @@
  *	     Contact: Fernando Pablo Hauscarriaga
  * 	     E-mail: fernandoph@iar.unlp.edu.ar
  * 	     Home page: www.iar.unlp.edu.ar/~fernandoph
- *	          Instituto Argentino de Radioastronomia
- *	                    www.iar.unlp.edu.ar
+ *		  Instituto Argentino de Radioastronomia
+ *			    www.iar.unlp.edu.ar
  *
  * 14/01/07: Conditinal compilation for Thunderbolt support no longer needed
- *           now we use mode 2 for decode thunderbolt packets.
- *           Fernando P. Hauscarriaga
+ *	     now we use mode 2 for decode thunderbolt packets.
+ *	     Fernando P. Hauscarriaga
  *
  * 30/08/09: Added support for Trimble Acutime Gold Receiver.
- *           Fernando P. Hauscarriaga (fernandoph@iar.unlp.edu.ar)
+ *	     Fernando P. Hauscarriaga (fernandoph@iar.unlp.edu.ar)
  */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
 
-#if defined(REFCLOCK) && (defined(PALISADE) || defined(CLOCK_PALISADE))
+#if defined(REFCLOCK) && defined(CLOCK_PALISADE)
 
 #ifdef SYS_WINNT
 extern int async_write(int, const void *, unsigned int);
@@ -116,8 +116,8 @@ int day_of_year (char *dt);
 #define CLK_TRIMBLE	0	/* Trimble Palisade */
 #define CLK_PRAECIS	1	/* Endrun Technologies Praecis */
 #define CLK_THUNDERBOLT	2	/* Trimble Thunderbolt GPS Receiver */
-#define CLK_ACUTIME     3       /* Trimble Acutime Gold */
-#define CLK_ACUTIMEB    4       /* Trimble Actutime Gold Port B */
+#define CLK_ACUTIME     3	/* Trimble Acutime Gold */
+#define CLK_ACUTIMEB    4	/* Trimble Actutime Gold Port B */
 
 int praecis_msg;
 static void praecis_parse(struct recvbuf *rbufp, struct peer *peer);
@@ -126,6 +126,7 @@ static void praecis_parse(struct recvbuf *rbufp, struct peer *peer);
  * They are taken from Markus Prosch
  */
 
+#ifdef PALISADE_SENDCMD_RESURRECTED
 /*
  * sendcmd - Build data packet for sending
  */
@@ -135,10 +136,11 @@ sendcmd (
 	int c
 	)
 {
-        *buffer->data = DLE;
-        *(buffer->data + 1) = (unsigned char)c;
-        buffer->size = 2;
+	*buffer->data = DLE;
+	*(buffer->data + 1) = (unsigned char)c;
+	buffer->size = 2;
 }
+#endif	/* PALISADE_SENDCMD_RESURRECTED */
 
 /*
  * sendsupercmd - Build super data packet for sending
@@ -150,10 +152,10 @@ sendsupercmd (
 	int c2
 	)
 {
-        *buffer->data = DLE;
-        *(buffer->data + 1) = (unsigned char)c1;
-        *(buffer->data + 2) = (unsigned char)c2;
-        buffer->size = 3;
+	*buffer->data = DLE;
+	*(buffer->data + 1) = (unsigned char)c1;
+	*(buffer->data + 2) = (unsigned char)c2;
+	buffer->size = 3;
 }
 
 /*
@@ -165,9 +167,9 @@ sendbyte (
 	int b
 	)
 {
-        if (b == DLE)
-                *(buffer->data+buffer->size++) = DLE;
-        *(buffer->data+buffer->size++) = (unsigned char)b;
+	if (b == DLE)
+		*(buffer->data+buffer->size++) = DLE;
+	*(buffer->data+buffer->size++) = (unsigned char)b;
 }
 
 /*
@@ -179,8 +181,8 @@ sendint (
 	int a
 	)
 {
-        sendbyte(buffer, (unsigned char)((a>>8) & 0xff));
-        sendbyte(buffer, (unsigned char)(a & 0xff));
+	sendbyte(buffer, (unsigned char)((a>>8) & 0xff));
+	sendbyte(buffer, (unsigned char)(a & 0xff));
 }
 
 /*
@@ -192,40 +194,43 @@ sendetx (
 	int fd
 	)
 {
-        int result;
+	int result;
 	
-        *(buffer->data+buffer->size++) = DLE;
-        *(buffer->data+buffer->size++) = ETX;
-        result = write(fd, buffer->data, (unsigned long)buffer->size);
+	*(buffer->data+buffer->size++) = DLE;
+	*(buffer->data+buffer->size++) = ETX;
+	result = write(fd, buffer->data, (unsigned long)buffer->size);
 	
-        if (result != -1)
-                return(result);
-        else
-                return (-1);
+	if (result != -1)
+		return (result);
+	else
+		return (-1);
 }
 
 /*
  * init_thunderbolt - Prepares Thunderbolt receiver to be used with
- *                    NTP (also taken from Markus Prosch).
+ *		      NTP (also taken from Markus Prosch).
  */
 static void
 init_thunderbolt (
 	int fd
 	)
 {
-	struct packettx buf;
-	
-//	buf.data = (u_char *) malloc(1);
+	struct packettx tx;
 	
+	tx.size = 0;
+	tx.data = (u_char *) malloc(100);
+
 	/* set UTC time */
-	sendsupercmd (&buf, 0x8E, 0xA2);
-	sendbyte     (&buf, 0x3);
-	sendetx      (&buf, fd);
+	sendsupercmd (&tx, 0x8E, 0xA2);
+	sendbyte     (&tx, 0x3);
+	sendetx      (&tx, fd);
 	
 	/* activate packets 0x8F-AB and 0x8F-AC */
-	sendsupercmd (&buf, 0x8F, 0xA5);
-	sendint      (&buf, 0x5);
-	sendetx      (&buf, fd);
+	sendsupercmd (&tx, 0x8F, 0xA5);
+	sendint      (&tx, 0x5);
+	sendetx      (&tx, fd);
+
+	free(tx.data);
 }
 
 /*
@@ -266,13 +271,10 @@ palisade_start (
 	struct refclockproc *pp;
 	int fd;
 	char gpsdev[20];
-
 	struct termios tio;
-#ifdef SYS_WINNT
-	(void) sprintf(gpsdev, "COM%d:", unit);
-#else	
-	(void) sprintf(gpsdev, DEVICE, unit);
-#endif
+
+	snprintf(gpsdev, sizeof(gpsdev), DEVICE, unit);
+
 	/*
 	 * Open serial port. 
 	 */
@@ -287,31 +289,23 @@ palisade_start (
 	msyslog(LOG_NOTICE, "Palisade(%d) fd: %d dev: %s", unit, fd,
 		gpsdev);
 
-        if (tcgetattr(fd, &tio) < 0) {
-                msyslog(LOG_ERR, 
+	if (tcgetattr(fd, &tio) < 0) {
+		msyslog(LOG_ERR, 
 			"Palisade(%d) tcgetattr(fd, &tio): %m",unit);
 #ifdef DEBUG
-                printf("Palisade(%d) tcgetattr(fd, &tio)\n",unit);
+		printf("Palisade(%d) tcgetattr(fd, &tio)\n",unit);
 #endif
-                return (0);
-        }
+		close(fd);
+		return (0);
+	}
 
-        tio.c_cflag |= (PARENB|PARODD);
-        tio.c_iflag &= ~ICRNL;
+	tio.c_cflag |= (PARENB|PARODD);
+	tio.c_iflag &= ~ICRNL;
 
 	/*
 	 * Allocate and initialize unit structure
 	 */
 	up = (struct palisade_unit *) emalloc(sizeof(struct palisade_unit));
-	      
-	if (!(up)) {
-                msyslog(LOG_ERR, "Palisade(%d) emalloc: %m",unit);
-#ifdef DEBUG
-                printf("Palisade(%d) emalloc\n",unit);
-#endif
-		(void) close(fd);
-		return (0);
-	}
 
 	memset((char *)up, 0, sizeof(struct palisade_unit));
 
@@ -338,12 +332,14 @@ palisade_start (
 		break;
 	}
 	if (tcsetattr(fd, TCSANOW, &tio) == -1) {
-                msyslog(LOG_ERR, "Palisade(%d) tcsetattr(fd, &tio): %m",unit);
+		msyslog(LOG_ERR, "Palisade(%d) tcsetattr(fd, &tio): %m",unit);
 #ifdef DEBUG
-                printf("Palisade(%d) tcsetattr(fd, &tio)\n",unit);
+		printf("Palisade(%d) tcsetattr(fd, &tio)\n",unit);
 #endif
-                return 0;
-        }
+		close(fd);
+		free(up);
+		return 0;
+	}
 
 	pp = peer->procptr;
 	pp->io.clock_recv = palisade_io;
@@ -352,9 +348,10 @@ palisade_start (
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
 #ifdef DEBUG
-                printf("Palisade(%d) io_addclock\n",unit);
+		printf("Palisade(%d) io_addclock\n",unit);
 #endif
-		(void) close(fd);
+		close(fd);
+		pp->io.fd = -1;
 		free(up);
 		return (0);
 	}
@@ -374,10 +371,10 @@ palisade_start (
 	up->leap_status = 0;
 	up->unit = (short) unit;
 	up->rpt_status = TSIP_PARSED_EMPTY;
-    	up->rpt_cnt = 0;
+	up->rpt_cnt = 0;
 
-    	if (up->type == CLK_THUNDERBOLT)
-       	    	init_thunderbolt(fd);
+	if (up->type == CLK_THUNDERBOLT)
+		init_thunderbolt(fd);
 	if (up->type == CLK_ACUTIME)
 		init_acutime(fd);
 
@@ -398,8 +395,10 @@ palisade_shutdown (
 	struct refclockproc *pp;
 	pp = peer->procptr;
 	up = (struct palisade_unit *)pp->unitptr;
-	io_closeclock(&pp->io);
-	free(up);
+	if (-1 != pp->io.fd)
+		io_closeclock(&pp->io);
+	if (NULL != up)
+		free(up);
 }
 
 
@@ -652,7 +651,7 @@ TSIP_decode (
 				pp->leap = LEAP_ADDSECOND;  /* we ASSUME addsecond */
 			else 
 				pp->leap = LEAP_NOWARNING;
-                		
+
 #ifdef DEBUG
 			if (debug > 1) 
 				printf("TSIP_decode: unit %d: 0x%02x leap %d\n",
@@ -694,7 +693,7 @@ TSIP_decode (
 
 		    case PACKET_8FAB:
 			/* Thunderbolt Primary Timing Packet */
-	        
+
 			if (up->rpt_cnt != LENCODE_8FAB) /* check length */
 				break;
 
@@ -702,7 +701,7 @@ TSIP_decode (
 				return 0;
 
 			GPS_UTC_Offset = getint((u_char *) &mb(7));
-                
+
 			if (GPS_UTC_Offset == 0){ /* Check UTC Offset */
 #ifdef DEBUG
 				printf("TSIP_decode: UTC Offset Unknown\n");
@@ -770,35 +769,36 @@ TSIP_decode (
 			return 1;
 			break;
 
-		    default: 	
+		    default:
 			/* Ignore Packet */
 			return 0;
 		} /* switch */
-	}/* if 8F packets */	
+	} /* if 8F packets */
 
-        else if (up->rpt_buf[0] == (u_char)0x42) {
+	else if (up->rpt_buf[0] == (u_char)0x42) {
 		printf("0x42\n");
 		return 0;
-        }
-        else if (up->rpt_buf[0] == (u_char)0x43) {
+	}
+	else if (up->rpt_buf[0] == (u_char)0x43) {
 		printf("0x43\n");
 		return 0;
-        }
-        else if ((up->rpt_buf[0] == PACKET_41) & (up->type == CLK_THUNDERBOLT)){
+	}
+	else if ((up->rpt_buf[0] == PACKET_41) & (up->type == CLK_THUNDERBOLT)){
 		printf("Undocumented 0x41 packet on Thunderbolt\n");
 		return 0;
-        }
+	}
 	else if ((up->rpt_buf[0] == PACKET_41A) & (up->type == CLK_ACUTIME)) {
 #ifdef DEBUG
-		printf("GPS TOW: %ld\n", getlong((u_char *) &mb(0)));
+		printf("GPS TOW: %ld\n", (long)getlong((u_char *) &mb(0)));
 		printf("GPS WN: %d\n", getint((u_char *) &mb(4)));
-		printf("GPS UTC-GPS Offser: %ld\n", getlong((u_char *) &mb(6)));
+		printf("GPS UTC-GPS Offser: %ld\n", (long)getlong((u_char *) &mb(6)));
 #endif
 		return 0;
 	}
 
 	/* Health Status for Acutime Receiver */
 	else if ((up->rpt_buf[0] == PACKET_46) & (up->type == CLK_ACUTIME)) {
+#ifdef DEBUG
 		if (debug > 1)
 		/* Status Codes */
 			switch (mb(0)) {
@@ -827,13 +827,14 @@ TSIP_decode (
 				printf("The Chosen satellite is unusable\n");
 				break;
 			}
+#endif
 		/* Error Codes */
 		if (mb(1) != 0)	{
 			
 			refclock_report(peer, CEVNT_BADTIME);
 			up->polled = -1;
-			
-			if (debug > 1){
+#ifdef DEBUG
+			if (debug > 1) {
 				if (mb(1) && 0x01)
 					printf ("Signal Processor Error, reset unit.\n");
 				if (mb(1) && 0x02)
@@ -845,6 +846,7 @@ TSIP_decode (
 				if (mb(1) && 0x05)
 					printf ("Excessive reference frequency error, refer to packet 0x2D and packet 0x4D documentation for further information\n");
 			}
+#endif
 		
 		return 0;
 		}
@@ -852,7 +854,7 @@ TSIP_decode (
 	else if (up->rpt_buf[0] == 0x54)
 		return 0;
 
-        else if (up->rpt_buf[0] == PACKET_6D) {
+	else if (up->rpt_buf[0] == PACKET_6D) {
 #ifdef DEBUG
 		int sats;
 
@@ -871,7 +873,7 @@ TSIP_decode (
 		printf("Tracking %d Satellites\n", sats);
 #endif
 		return 0;
-        } /* else if not super packet */
+	} /* else if not super packet */
 	refclock_report(peer, CEVNT_BADREPLY);
 	up->polled = -1;
 #ifdef DEBUG
@@ -921,8 +923,10 @@ palisade_receive (
 	 * report and process 
 	 */
 
-	(void) sprintf(pp->a_lastcode,"%4d %03d %02d:%02d:%02d.%06ld",
-		       pp->year,pp->day,pp->hour,pp->minute, pp->second,pp->nsec); 
+	snprintf(pp->a_lastcode, sizeof(pp->a_lastcode),
+		 "%4d %03d %02d:%02d:%02d.%06ld",
+		 pp->year, pp->day,
+		 pp->hour,pp->minute, pp->second, pp->nsec); 
 	pp->lencode = 24;
 
 	if (!refclock_process(pp)) {
@@ -1082,11 +1086,11 @@ palisade_io (
 				up->rpt_status = TSIP_PARSED_DATA;
 				mb(up->rpt_cnt++) = 
 				    *c;
-			}       
+			}
 			else if (*c == ETX) 
 				up->rpt_status = TSIP_PARSED_FULL;
 			else 	{
-                        	/* error: start new report packet */
+				/* error: start new report packet */
 				up->rpt_status = TSIP_PARSED_DLE_1;
 				up->rpt_buf[0] = *c;
 			}
@@ -1095,11 +1099,11 @@ palisade_io (
 		    case TSIP_PARSED_FULL:
 		    case TSIP_PARSED_EMPTY:
 		    default:
-		        if ( *c != DLE)
+			if ( *c != DLE)
 				up->rpt_status = TSIP_PARSED_EMPTY;
 			else 
 				up->rpt_status = TSIP_PARSED_DLE_1;
-                        break;
+			break;
 		}
 		
 		c++;
@@ -1107,11 +1111,11 @@ palisade_io (
 		if (up->rpt_status == TSIP_PARSED_DLE_1) {
 			up->rpt_cnt = 0;
 			if (pp->sloppyclockflag & CLK_FLAG2) 
-                		/* stamp it */
+				/* stamp it */
 				get_systime(&pp->lastrec);
 		}
 		else if (up->rpt_status == TSIP_PARSED_EMPTY)
-		    	up->rpt_cnt = 0;
+			up->rpt_cnt = 0;
 
 		else if (up->rpt_cnt > BMAX) 
 			up->rpt_status =TSIP_PARSED_EMPTY;
@@ -1186,86 +1190,62 @@ HW_poll (
 	return 0;
 }
 
-#if 0 /* unused */
 /*
- * this 'casts' a character array into a float
+ * copy/swap a big-endian palisade double into a host double
  */
-float
-getfloat (
-	u_char *bp
-	)
-{
-	float sval;
-#ifdef WORDS_BIGENDIAN 
-	((char *) &sval)[0] = *bp++;
-	((char *) &sval)[1] = *bp++;
-	((char *) &sval)[2] = *bp++;
-	((char *) &sval)[3] = *bp++;
-#else
-	((char *) &sval)[3] = *bp++;
-	((char *) &sval)[2] = *bp++;
-	((char *) &sval)[1] = *bp++;
-	((char *) &sval)[0] = *bp;
-#endif  /* ! XNTP_BIG_ENDIAN */ 
-	return sval;
-}
-#endif
-
-/*
- * this 'casts' a character array into a double
- */
-double
+static double
 getdbl (
 	u_char *bp
 	)
 {
-	double dval;
-#ifdef WORDS_BIGENDIAN 
-	((char *) &dval)[0] = *bp++;
-	((char *) &dval)[1] = *bp++;
-	((char *) &dval)[2] = *bp++;
-	((char *) &dval)[3] = *bp++;
-	((char *) &dval)[4] = *bp++;
-	((char *) &dval)[5] = *bp++;
-	((char *) &dval)[6] = *bp++;
-	((char *) &dval)[7] = *bp;
+#ifdef WORDS_BIGENDIAN
+	double out;
+
+	memcpy(&out, bp, sizeof(out));
+	return out;
 #else
-	((char *) &dval)[7] = *bp++;
-	((char *) &dval)[6] = *bp++;
-	((char *) &dval)[5] = *bp++;
-	((char *) &dval)[4] = *bp++;
-	((char *) &dval)[3] = *bp++;
-	((char *) &dval)[2] = *bp++;
-	((char *) &dval)[1] = *bp++;
-	((char *) &dval)[0] = *bp;
-#endif  /* ! XNTP_BIG_ENDIAN */ 
-	return dval;
+	union {
+		u_char ch[8];
+		u_int32 u32[2];
+	} ui;
+		
+	union {
+		double out;
+		u_int32 u32[2];
+	} uo;
+
+	memcpy(ui.ch, bp, sizeof(ui.ch));
+	/* least-significant 32 bits of double from swapped bp[4] to bp[7] */
+	uo.u32[0] = ntohl(ui.u32[1]);
+	/* most-significant 32 bits from swapped bp[0] to bp[3] */
+	uo.u32[1] = ntohl(ui.u32[0]);
+
+	return uo.out;
+#endif
 }
 
 /*
- * cast a 16 bit character array into a short (16 bit) int
+ * copy/swap a big-endian palisade short into a host short
  */
-short
+static short
 getint (
 	u_char *bp
 	)
 {
-	return (short) (bp[1] + (bp[0] << 8));
+	return (short)ntohs(*(u_short *)bp);
 }
 
 /*
- * cast a 32 bit character array into a long (32 bit) int
+ * copy/swap a big-endian palisade 32-bit int into a host 32-bit int
  */
-long
+static int32
 getlong(
 	u_char *bp
 	)
 {
-	return (long) (bp[0] << 24) | 
-	    (bp[1] << 16) |
-	    (bp[2] << 8) |
-	    bp[3];
+	return (int32)(u_int32)ntohl(*(u_int32 *)bp);
 }
 
-int refclock_palisade_bs;
-#endif /* REFCLOCK */
+#else	/* REFCLOCK && CLOCK_PALISADE*/
+int refclock_palisade_c_notempty;
+#endif
