@@ -63,7 +63,7 @@
  *
  * Fudge factors
  *
- * Fudge flag4 causes the dubugging output described above to be
+ * Fudge flag4 causes the debugging output described above to be
  * recorded in the clockstats file. Fudge flag2 selects the audio input
  * port, where 0 is the mike port (default) and 1 is the line-in port.
  * It does not seem useful to select the compact disc player port. Fudge
@@ -137,7 +137,7 @@
 #define FGATE		0x0010	/* frequency gate */
 #define DGATE		0x0020	/* data pulse amplitude error */
 #define BGATE		0x0040	/* data pulse width error */
-#define	METRIC		0x0080	/*one or more stations heard */
+#define	METRIC		0x0080	/* one or more stations heard */
 #define LEPSEC		0x1000	/* leap minute */
 
 /*
@@ -155,10 +155,10 @@
  * These bits indicate various alarm conditions, which are decoded to
  * form the quality character included in the timecode.
  */
-#define CMPERR		1	/* digit or misc bit compare error */
-#define LOWERR		2	/* low bit or digit amplitude or SNR */
-#define NINERR		4	/* less than nine digits in minute */
-#define SYNERR		8	/* not tracking second sync */
+#define CMPERR		0x1	/* digit or misc bit compare error */
+#define LOWERR		0x2	/* low bit or digit amplitude or SNR */
+#define NINERR		0x4	/* less than nine digits in minute */
+#define SYNERR		0x8	/* not tracking second sync */
 
 /*
  * Watchcat timeouts (watch)
@@ -465,7 +465,6 @@ char dstcod[] = {
 struct decvec {
 	int radix;		/* radix (3, 4, 6, 10) */
 	int digit;		/* current clock digit */
-	int mldigit;		/* maximum-likelihood digit */
 	int count;		/* match count */
 	double digprb;		/* max digit probability */
 	double digsnr;		/* likelihood function (dB) */
@@ -597,7 +596,7 @@ static	void	wwv_corr4	(struct peer *, struct decvec *,
 				    double [], double [][4]);
 static	void	wwv_gain	(struct peer *);
 static	void	wwv_tsec	(struct peer *);
-static	int	timecode	(struct wwvunit *, char *);
+static	int	timecode	(struct wwvunit *, char *, size_t);
 static	double	wwv_snr		(double, double);
 static	int	carry		(struct decvec *);
 static	int	wwv_newchan	(struct peer *);
@@ -660,15 +659,10 @@ wwv_start(
 	/*
 	 * Allocate and initialize unit structure
 	 */
-	if (!(up = (struct wwvunit *)emalloc(sizeof(struct wwvunit)))) {
-		close(fd);
-		return (0);
-	}
-	memset(up, 0, sizeof(struct wwvunit));
+	up = emalloc_zero(sizeof(*up));
 	pp = peer->procptr;
-	pp->unitptr = (caddr_t)up;
 	pp->io.clock_recv = wwv_receive;
-	pp->io.srcclock = (caddr_t)peer;
+	pp->io.srcclock = peer;
 	pp->io.datalen = 0;
 	pp->io.fd = fd;
 	if (!io_addclock(&pp->io)) {
@@ -676,6 +670,7 @@ wwv_start(
 		free(up);
 		return (0);
 	}
+	pp->unitptr = up;
 
 	/*
 	 * Initialize miscellaneous variables
@@ -694,8 +689,8 @@ wwv_start(
 	for (i = 3; i < OFFSET; i++) {
 		up->comp[i] = up->comp[i - 1] + step;
 		up->comp[OFFSET + i] = -up->comp[i];
-                if (i % 16 == 0)
-		    step *= 2.;
+		if (i % 16 == 0)
+			step *= 2.;
 	}
 	DTOLFP(1. / SECOND, &up->tick);
 
@@ -766,7 +761,7 @@ wwv_shutdown(
 	struct wwvunit *up;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (up == NULL)
 		return;
 
@@ -804,9 +799,9 @@ wwv_receive(
 	int	bufcnt;		/* buffer counter */
 	l_fp	ltemp;
 
-	peer = (struct peer *)rbufp->recv_srcclock;
+	peer = rbufp->recv_peer;
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Main loop - read until there ain't no more. Note codec
@@ -885,7 +880,7 @@ wwv_poll(
 	struct wwvunit *up;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (up->errflg)
 		refclock_report(peer, up->errflg);
 	up->errflg = 0;
@@ -979,7 +974,7 @@ wwv_rf(
 	int	i;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	if (!iniflg) {
 		iniflg = 1;
@@ -1255,7 +1250,6 @@ wwv_rf(
 
 		epomax = dtemp;
 		epopos = epoch;
-		up->timestamp = up->timestamp;
 		j = epoch - 6 * MS;
 		if (j < 0)
 			j += SECOND;
@@ -1310,7 +1304,7 @@ wwv_qrz(
 	long	epoch;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Find the sample with peak amplitude, which defines the minute
@@ -1411,10 +1405,10 @@ wwv_endpoc(
 	int tmp2;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (!iniflg) {
 		iniflg = 1;
-		memset((char *)epoch_mf, 0, sizeof(epoch_mf));
+		ZERO(epoch_mf);
 	}
 
 	/*
@@ -1627,7 +1621,7 @@ wwv_epoch(
 	static double sigmin, sigzer, sigone, engmax, engmin;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Find the maximum minute sync pulse energy for both the
@@ -1765,10 +1759,10 @@ wwv_rsec(
 	int	sw, arg, nsec;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (!iniflg) {
 		iniflg = 1;
-		memset((char *)bitvec, 0, sizeof(bitvec));
+		ZERO(bitvec);
 	}
 
 	/*
@@ -2061,7 +2055,7 @@ wwv_clock(
 	l_fp	offset;		/* offset in NTP seconds */
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (!(up->status & SSYNC))
 		up->alarm |= SYNERR;
 	if (up->digcnt < 9)
@@ -2097,7 +2091,8 @@ wwv_clock(
 			refclock_receive(peer);
 		}
 	}
-	pp->lencode = timecode(up, pp->a_lastcode);
+	pp->lencode = timecode(up, pp->a_lastcode,
+			       sizeof(pp->a_lastcode));
 	record_clock_stats(&peer->srcadr, pp->a_lastcode);
 #ifdef DEBUG
 	if (debug)
@@ -2134,7 +2129,7 @@ wwv_corr4(
 	int	i, j;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Correlate digit vector with each BCD coefficient vector. If
@@ -2174,26 +2169,23 @@ wwv_corr4(
 	 * not considered correct until all nine clock digits have
 	 * reached threshold. This is intended as eye candy, but avoids
 	 * mistakes when the signal is low and the SNR is very marginal.
-	 * once correctly set, the maximum-likelihood digit is ignored
-	 * on the assumption the clock will always be correct unless for
-	 * some reason it drifts to a different second.
 	 */
-	vp->mldigit = mldigit;
 	if (vp->digprb < BTHR || vp->digsnr < BSNR) {
-		vp->count = 0;
 		up->status |= BGATE;
 	} else {
-		up->status |= DSYNC;
 		if (vp->digit != mldigit) {
-			vp->count = 0;
 			up->alarm |= CMPERR;
-			if (!(up->status & INSYNC))
+			if (vp->count > 0)
+				vp->count--;
+			if (vp->count == 0)
 				vp->digit = mldigit;
 		} else {
 			if (vp->count < BCMP)
 				vp->count++;
-			if (vp->count == BCMP)
+			if (vp->count == BCMP) {
+				up->status |= DSYNC;
 				up->digcnt++;
+			}
 		}
 	}
 	if ((pp->sloppyclockflag & CLK_FLAG4) && !(up->status &
@@ -2201,7 +2193,7 @@ wwv_corr4(
 		sprintf(tbuf,
 		    "wwv4 %2d %04x %3d %4d %5.0f %2d %d %d %d %5.0f %5.1f",
 		    up->rsec - 1, up->status, up->gain, up->yepoch,
-		    up->epomax, vp->radix, vp->digit, vp->mldigit,
+		    up->epomax, vp->radix, vp->digit, mldigit,
 		    vp->count, vp->digprb, vp->digsnr);
 		record_clock_stats(&peer->srcadr, tbuf);
 #ifdef DEBUG
@@ -2230,7 +2222,7 @@ wwv_tsec(
 	int temp;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Advance minute unit of the day. Don't propagate carries until
@@ -2409,7 +2401,7 @@ wwv_newchan(
 	int i, j, rval;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Search all five station pairs looking for the channel with
@@ -2443,9 +2435,9 @@ wwv_newchan(
 	 * greater than the threshold, tune to that frequency and
 	 * transmitter QTH.
 	 */
+	up->status &= ~(SELV | SELH);
 	if (rank < MTHR) {
 		up->dchan = (up->dchan + 1) % NCHAN;
-		up->status &= ~(SELV | SELH);
 		if (up->status & METRIC) {
 			up->status &= ~METRIC;
 			refclock_report(peer, CEVNT_PROP);
@@ -2453,18 +2445,20 @@ wwv_newchan(
 		rval = FALSE;
 	} else {
 		up->dchan = j;
-		up->status |= SELV | SELH;
 		up->sptr = sp;
 		memcpy(&pp->refid, sp->refid, 4);
 		peer->refid = pp->refid;
-		if (sp->select & SELV)
-			up->pdelay = pp->fudgetime1;
-		else if (sp->select & SELH)
-			up->pdelay = pp->fudgetime2;
-		else
-			up->pdelay = 0;
-		rval = TRUE;
 		up->status |= METRIC;
+		if (sp->select & SELV) {
+			up->status |= SELV;
+			up->pdelay = pp->fudgetime1;
+		} else if (sp->select & SELH) {
+			up->status |= SELH;
+			up->pdelay = pp->fudgetime2;
+		} else {
+			up->pdelay = 0;
+		}
+		rval = TRUE;
 	}
 #ifdef ICOM
 	if (up->fd_icom > 0)
@@ -2499,7 +2493,7 @@ wwv_newgame(
 	int i;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Initialize strategic values. Note we set the leap bits
@@ -2576,7 +2570,7 @@ wwv_qsy(
 	struct wwvunit *up;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 	if (up->fd_icom > 0) {
 		up->mitig[up->achan].gain = up->gain;
 		rval = icom_freq(up->fd_icom, peer->ttl & 0x7f,
@@ -2617,7 +2611,8 @@ wwv_qsy(
 static int
 timecode(
 	struct wwvunit *up,	/* driver structure pointer */
-	char *ptr		/* target string */
+	char *		tc,	/* target string */
+	size_t		tcsiz	/* target max chars */
 	)
 {
 	struct sync *sp;
@@ -2642,20 +2637,23 @@ timecode(
 	dut = up->misc & 0x7;
 	if (!(up->misc & DUTS))
 		dut = -dut;
-	sprintf(ptr, "%c%1X", synchar, up->alarm);
-	sprintf(cptr, " %4d %03d %02d:%02d:%02d %c%c %+d",
-	    year, day, hour, minute, second, leapchar, dst, dut);
-	strcat(ptr, cptr);
+	snprintf(tc, tcsiz, "%c%1X", synchar, up->alarm);
+	snprintf(cptr, sizeof(cptr),
+		 " %4d %03d %02d:%02d:%02d %c%c %+d",
+		 year, day, hour, minute, second, leapchar, dst, dut);
+	strlcat(tc, cptr, tcsiz);
 
 	/*
 	 * Specific variable-format fields
 	 */
 	sp = up->sptr;
-	sprintf(cptr, " %d %d %s %.0f %d %.1f %d", up->watch,
-	    up->mitig[up->dchan].gain, sp->refid, sp->metric,
-	    up->errcnt, up->freq / SECOND * 1e6, up->avgint);
-	strcat(ptr, cptr);
-	return (strlen(ptr));
+	snprintf(cptr, sizeof(cptr), " %d %d %s %.0f %d %.1f %d",
+		 up->watch, up->mitig[up->dchan].gain, sp->refid,
+		 sp->metric, up->errcnt, up->freq / SECOND * 1e6,
+		 up->avgint);
+	strlcat(tc, cptr, tcsiz);
+
+	return strlen(tc);
 }
 
 
@@ -2679,7 +2677,7 @@ wwv_gain(
 	struct wwvunit *up;
 
 	pp = peer->procptr;
-	up = (struct wwvunit *)pp->unitptr;
+	up = pp->unitptr;
 
 	/*
 	 * Apparently, the codec uses only the high order bits of the
