@@ -20,9 +20,8 @@
 #include "ntp_syslog.h"
 #include "ntp_select.h"
 #include "ntp_stdlib.h"
+#include <ssl_applink.c>
 
-/* Don't include ISC's version of IPv6 variables and structures */
-#define ISC_IPV6_H 1
 #include "isc/net.h"
 #include "isc/result.h"
 #include "isc/sockaddr.h"
@@ -37,19 +36,15 @@
 #ifdef HAVE_POLL_H
 # include <poll.h>
 #endif
-#ifndef SYS_WINNT
-# ifdef HAVE_SYS_SIGNAL_H
-#  include <sys/signal.h>
-# else
-#  include <signal.h>
-# endif
-# ifdef HAVE_SYS_IOCTL_H
-#  include <sys/ioctl.h>
-# endif
-#endif /* SYS_WINNT */
+#ifdef HAVE_SYS_SIGNAL_H
+# include <sys/signal.h>
+#endif
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif
 #ifdef HAVE_SYS_RESOURCE_H
 # include <sys/resource.h>
-#endif /* HAVE_SYS_RESOURCE_H */
+#endif
 
 #include <arpa/inet.h>
 
@@ -63,7 +58,7 @@ struct timeval timeout = {0,0};
 #elif defined(SYS_WINNT)
 /*
  * Windows does not abort a select select call if SIGALRM goes off
- * so a 200 ms timeout is needed
+ * so a 200 ms timeout is needed (TIMER_HZ is 5).
  */
 struct sock_timeval timeout = {0,1000000/TIMER_HZ};
 #else
@@ -77,9 +72,6 @@ struct timeval timeout = {60,0};
 #include "recvbuff.h"
 
 #ifdef SYS_WINNT
-#define EPROTONOSUPPORT WSAEPROTONOSUPPORT
-#define EAFNOSUPPORT	WSAEAFNOSUPPORT
-#define EPFNOSUPPORT	WSAEPFNOSUPPORT
 #define TARGET_RESOLUTION 1  /* Try for 1-millisecond accuracy
 				on Windows NT timers. */
 #pragma comment(lib, "winmm")
@@ -96,7 +88,7 @@ UINT wTimerRes;
 # define	NTPDATE_PRIO	(100)
 #endif
 
-#if defined(HAVE_TIMER_SETTIME) || defined (HAVE_TIMER_CREATE)
+#ifdef HAVE_TIMER_CREATE
 /* POSIX TIMERS - vxWorks doesn't have itimer - casey */
 static timer_t ntpdate_timerid;
 #endif
@@ -220,14 +212,14 @@ static	void	clock_filter	(struct server *);
 static	struct server *clock_select (void);
 static	int clock_adjust	(void);
 static	void	addserver	(char *);
-static	struct server *findserver (struct sockaddr_storage *);
+static	struct server *findserver (sockaddr_u *);
 		void	timer		(void);
 static	void	init_alarm	(void);
 #ifndef SYS_WINNT
 static	RETSIGTYPE alarming (int);
 #endif /* SYS_WINNT */
 static	void	init_io 	(void);
-static	void	sendpkt 	(struct sockaddr_storage *, struct pkt *, int);
+static	void	sendpkt 	(sockaddr_u *, struct pkt *, int);
 void	input_handler	(void);
 
 static	int l_adj_systime	(l_fp *);
@@ -334,31 +326,23 @@ ntpdatemain (
 	ni_namelist *netinfoservers;
 #endif
 #ifdef SYS_WINNT
-	HANDLE process_handle;
-
-	wVersionRequested = MAKEWORD(1,1);
-	if (WSAStartup(wVersionRequested, &wsaData)) {
-		netsyslog(LOG_ERR, "No useable winsock.dll: %m");
-		exit(1);
-	}
-
 	key_file = key_file_storage;
 
 	if (!ExpandEnvironmentStrings(KEYFILE, key_file, MAX_PATH))
-	{
 		msyslog(LOG_ERR, "ExpandEnvironmentStrings(KEYFILE) failed: %m\n");
-	}
+
+	ssl_applink();
 #endif /* SYS_WINNT */
 
 #ifdef NO_MAIN_ALLOWED
 	clear_globals();
 #endif
 
+	init_lib();	/* sets up ipv4_works, ipv6_works */
 
-	/* Check to see if we have IPv6. Otherwise force the -4 flag */
-	if (isc_net_probeipv6() != ISC_R_SUCCESS) {
+	/* Check to see if we have IPv6. Otherwise default to IPv4 */
+	if (!ipv6_works)
 		ai_fam_templ = AF_INET;
-	}
 
 	errflg = 0;
 	progname = argv[0];
@@ -435,8 +419,7 @@ ntpdatemain (
 			} else {
 				sys_timeout = ((LFPTOFP(&tmp) * TIMER_HZ)
 					   + 0x8000) >> 16;
-				if (sys_timeout == 0)
-				sys_timeout = 1;
+				sys_timeout = max(sys_timeout, MINTIMEOUT);
 			}
 			break;
 		case 'v':
@@ -595,7 +578,7 @@ ntpdatemain (
 #else
 				if (WSAGetLastError() != WSAEINTR)
 #endif
-					netsyslog(LOG_ERR,
+					msyslog(LOG_ERR,
 #ifdef HAVE_POLL_H
 						"poll() error: %m"
 #else
@@ -604,7 +587,7 @@ ntpdatemain (
 						);
 			} else if (errno != 0) {
 #ifndef SYS_VXWORKS
-				netsyslog(LOG_DEBUG,
+				msyslog(LOG_DEBUG,
 #ifdef HAVE_POLL_H
 					"poll(): nfound = %d, error: %m",
 #else
@@ -674,7 +657,7 @@ transmit(
 	struct pkt xpkt;
 
 	if (debug)
-		printf("transmit(%s)\n", stoa(&(server->srcadr)));
+		printf("transmit(%s)\n", stoa(&server->srcadr));
 
 	if (server->filter_nextpt < server->xmtcnt) {
 		l_fp ts;
@@ -725,18 +708,18 @@ transmit(
 		L_ADDUF(&server->xmt, sys_authdelay);
 		HTONL_FP(&server->xmt, &xpkt.xmt);
 		len = authencrypt(sys_authkey, (u_int32 *)&xpkt, LEN_PKT_NOMAC);
-		sendpkt(&(server->srcadr), &xpkt, (int)(LEN_PKT_NOMAC + len));
+		sendpkt(&server->srcadr, &xpkt, (int)(LEN_PKT_NOMAC + len));
 
 		if (debug > 1)
 			printf("transmit auth to %s\n",
-			   stoa(&(server->srcadr)));
+			   stoa(&server->srcadr));
 	} else {
 		get_systime(&(server->xmt));
 		HTONL_FP(&server->xmt, &xpkt.xmt);
-		sendpkt(&(server->srcadr), &xpkt, LEN_PKT_NOMAC);
+		sendpkt(&server->srcadr, &xpkt, LEN_PKT_NOMAC);
 
 		if (debug > 1)
-			printf("transmit to %s\n", stoa(&(server->srcadr)));
+			printf("transmit to %s\n", stoa(&server->srcadr));
 	}
 
 	/*
@@ -844,6 +827,19 @@ receive(
 	if (!is_authentic)
 		server->trust |= 1;
 
+	/*
+	 * Check for a KoD (rate limiting) response, cease and decist.
+	 */
+	if (LEAP_NOTINSYNC == PKT_LEAP(rpkt->li_vn_mode) &&
+	    STRATUM_PKT_UNSPEC == rpkt->stratum &&
+	    !memcmp("RATE", &rpkt->refid, 4)) {
+		msyslog(LOG_ERR, "%s rate limit response from server.\n",
+			stoa(&rbufp->recv_srcadr));
+		server->event_time = 0;
+		complete_servers++;
+		return;
+	}
+
 	/*
 	 * Looks good.	Record info from the packet.
 	 */
@@ -862,7 +858,7 @@ receive(
 	 * again.
 	 */
 	if (L_ISZERO(&rec) || !L_ISHIS(&server->org, &rec)) {
-		transmit(server);
+		server->event_time = current_time + sys_timeout;
 		return;
 	}
 
@@ -911,10 +907,21 @@ receive(
 	}
 
 	/*
-	 * Shift this data in, then transmit again.
+	 * Shift this data in, then schedule another transmit.
 	 */
 	server_data(server, (s_fp) di, &ci, 0);
-	transmit(server);
+
+	if ((int)server->filter_nextpt >= sys_samples) {
+		/*
+		 * Got all the data we need.  Mark this guy
+		 * completed and return.
+		 */
+		server->event_time = 0;
+		complete_servers++;
+		return;
+	}
+
+	server->event_time = current_time + sys_timeout;
 }
 
 
@@ -1022,12 +1029,14 @@ clock_filter(
 static struct server *
 clock_select(void)
 {
-	register struct server *server;
-	register int i;
-	register int nlist;
-	register s_fp d;
-	register int j;
-	register int n;
+	struct server *server;
+	u_int nlist;
+	s_fp d;
+	u_int count;
+	u_int i;
+	u_int j;
+	u_int k;
+	int n;
 	s_fp local_threshold;
 	struct server *server_list[NTP_MAXCLOCK];
 	u_fp server_badness[NTP_MAXCLOCK];
@@ -1123,12 +1132,14 @@ clock_select(void)
 	 * Got the five-or-less best.	 Cut the list where the number of
 	 * strata exceeds two.
 	 */
-	j = 0;
+	count = 0;
 	for (i = 1; i < nlist; i++)
-		if (server_list[i]->stratum > server_list[i-1]->stratum)
-		if (++j == 2) {
-			nlist = i;
-			break;
+		if (server_list[i]->stratum > server_list[i-1]->stratum) {
+			count++;
+			if (2 == count) {
+				nlist = i;
+				break;
+			}
 		}
 
 	/*
@@ -1138,9 +1149,9 @@ clock_select(void)
 	 * detection.
 	 */
 
-	if (nlist == 0)
-		sys_server = 0;
-	else if (nlist == 1) {
+	if (0 == nlist)
+		sys_server = NULL;
+	else if (1 == nlist) {
 		sys_server = server_list[0];
 	} else {
 		/*
@@ -1149,12 +1160,13 @@ clock_select(void)
 		 */
 		for (i = 0; i < nlist-1; i++)
 			for (j = i+1; j < nlist; j++) {
-				if (server_list[i]->stratum
-				< server_list[j]->stratum)
-				break;	/* already sorted by stratum */
-				if (server_list[i]->delay
-				< server_list[j]->delay)
-				continue;
+				if (server_list[i]->stratum <
+				    server_list[j]->stratum)
+					/* already sorted by stratum */
+					break;
+				if (server_list[i]->delay <
+				    server_list[j]->delay)
+					continue;
 				server = server_list[i];
 				server_list[i] = server_list[j];
 				server_list[j] = server;
@@ -1170,14 +1182,14 @@ clock_select(void)
 		 * Now drop samples until we're down to one.
 		 */
 		while (nlist > 1) {
-			for (n = 0; n < nlist; n++) {
-				server_badness[n] = 0;
+			for (k = 0; k < nlist; k++) {
+				server_badness[k] = 0;
 				for (j = 0; j < nlist; j++) {
-					if (j == n) /* with self? */
+					if (j == k) /* with self? */
 						continue;
-					d = server_list[j]->soffset
-						- server_list[n]->soffset;
-					if (d < 0)	/* absolute value */
+					d = server_list[j]->soffset -
+					    server_list[k]->soffset;
+					if (d < 0)	/* abs value */
 						d = -d;
 					/*
 					 * XXX This code *knows* that
@@ -1185,7 +1197,7 @@ clock_select(void)
 					 */
 					for (i = 0; i < j; i++)
 						d = (d>>1) + (d>>2);
-					server_badness[n] += d;
+					server_badness[k] += d;
 				}
 			}
 
@@ -1276,7 +1288,7 @@ clock_adjust(void)
 				lfptoa(&server->offset, 6));
 		}
 	} else {
-#if !defined SYS_WINNT && !defined SYS_CYGWIN32
+#ifndef SYS_WINNT
 		if (simple_query || l_adj_systime(&server->offset)) {
 			msyslog(LOG_NOTICE, "adjust time server %s offset %s sec",
 				stoa(&server->srcadr),
@@ -1305,16 +1317,16 @@ clock_adjust(void)
  *		    (non-blocking).
  */
 static int
-is_reachable (struct sockaddr_storage *dst)
+is_reachable (sockaddr_u *dst)
 {
 	SOCKET sockfd;
 
-	sockfd = socket(dst->ss_family, SOCK_DGRAM, 0);
+	sockfd = socket(AF(dst), SOCK_DGRAM, 0);
 	if (sockfd == -1) {
 		return 0;
 	}
 
-	if(connect(sockfd, (struct sockaddr *)dst, SOCKLEN(dst))) {
+	if (connect(sockfd, &dst->sa, SOCKLEN(dst))) {
 		closesocket(sockfd);
 		return 0;
 	}
@@ -1343,10 +1355,12 @@ addserver(
 	int error;
 	/* Service name */
 	char service[5];
-	strcpy(service, "ntp");
+	sockaddr_u addr;
+
+	strncpy(service, "ntp", sizeof(service));
 
 	/* Get host address. Looking for UDP datagram connection. */
-	memset(&hints, 0, sizeof(hints));
+	ZERO(hints);
 	hints.ai_family = ai_fam_templ;
 	hints.ai_socktype = SOCK_DGRAM;
 
@@ -1362,28 +1376,35 @@ addserver(
 			/* Name server is unusable. Exit after failing on the
 			   first server, in order to shorten the timeout caused
 			   by waiting for resolution of several servers */
-			fprintf(stderr, "Name server cannot be used, exiting");
-			msyslog(LOG_ERR, "name server cannot be used, reason: %s\n", gai_strerror(error));
+			fprintf(stderr, "Exiting, name server cannot be used: %s (%d)",
+				gai_strerror(error), error);
+			msyslog(LOG_ERR, "name server cannot be used: %s (%d)\n",
+				gai_strerror(error), error);
 			exit(1);
 		}
-		fprintf(stderr, "Error : %s\n", gai_strerror(error));
-		msyslog(LOG_ERR, "can't find host %s\n", serv);
+		fprintf(stderr, "Error resolving %s: %s (%d)\n", serv,
+			gai_strerror(error), error);
+		msyslog(LOG_ERR, "Can't find host %s: %s (%d)\n", serv,
+			gai_strerror(error), error);
 		return;
 	}
 #ifdef DEBUG
-	else if (debug) {
-		fprintf(stderr, "host found : %s\n", stohost((struct sockaddr_storage*)addrResult->ai_addr));
+	if (debug) {
+		ZERO(addr);
+		INSIST(addrResult->ai_addrlen <= sizeof(addr));
+		memcpy(&addr, addrResult->ai_addr, addrResult->ai_addrlen);
+		fprintf(stderr, "host found : %s\n", stohost(&addr));
 	}
 #endif
 
 	/* We must get all returned server in case the first one fails */
 	for (ptr = addrResult; ptr != NULL; ptr = ptr->ai_next) {
-		if (is_reachable ((struct sockaddr_storage *)ptr->ai_addr)) {
-			server = (struct server *)emalloc(sizeof(struct server));
-			memset((char *)server, 0, sizeof(struct server));
-
-			memset(&(server->srcadr), 0, sizeof(struct sockaddr_storage));
-			memcpy(&(server->srcadr), ptr->ai_addr, ptr->ai_addrlen);
+		ZERO(addr);
+		INSIST(ptr->ai_addrlen <= sizeof(addr));
+		memcpy(&addr, ptr->ai_addr, ptr->ai_addrlen);
+		if (is_reachable(&addr)) {
+			server = emalloc_zero(sizeof(*server));
+			memcpy(&server->srcadr, ptr->ai_addr, ptr->ai_addrlen);
 			server->event_time = ++sys_numservers;
 			if (sys_servers == NULL)
 				sys_servers = server;
@@ -1391,7 +1412,8 @@ addserver(
 				struct server *sp;
 
 				for (sp = sys_servers; sp->next_server != NULL;
-				     sp = sp->next_server) ;
+				     sp = sp->next_server)
+					/* empty */;
 				sp->next_server = server;
 			}
 		}
@@ -1407,40 +1429,23 @@ addserver(
  */
 static struct server *
 findserver(
-	struct sockaddr_storage *addr
+	sockaddr_u *addr
 	)
 {
 	struct server *server;
 	struct server *mc_server;
-	isc_sockaddr_t laddr;
-	isc_sockaddr_t saddr;
-
-	if(addr->ss_family == AF_INET) {
-		isc_sockaddr_fromin( &laddr, &((struct sockaddr_in*)addr)->sin_addr, 0);
-	}
-	else {
-		isc_sockaddr_fromin6(&laddr, &((struct sockaddr_in6*)addr)->sin6_addr, 0);
-	}
-
 
 	mc_server = NULL;
-	if (htons(((struct sockaddr_in*)addr)->sin_port) != NTP_PORT)
+	if (SRCPORT(addr) != NTP_PORT)
 		return 0;
 
 	for (server = sys_servers; server != NULL; 
 	     server = server->next_server) {
-		
-		if(server->srcadr.ss_family == AF_INET) {
-			isc_sockaddr_fromin(&saddr, &((struct sockaddr_in*)&server->srcadr)->sin_addr, 0);
-		}
-		else {
-			isc_sockaddr_fromin6(&saddr, &((struct sockaddr_in6*)&server->srcadr)->sin6_addr, 0);
-		}
-		if (isc_sockaddr_eqaddr(&laddr, &saddr) == ISC_TRUE)
+		if (SOCK_EQ(addr, &server->srcadr))
 			return server;
 
-		if(addr->ss_family == server->srcadr.ss_family) {
-			if (isc_sockaddr_ismulticast(&saddr) == ISC_TRUE)
+		if (AF(addr) == AF(&server->srcadr)) {
+			if (IS_MCAST(&server->srcadr))
 				mc_server = server;
 		}
 	}
@@ -1454,15 +1459,15 @@ findserver(
 			complete_servers++;
 		}
 
-		server = (struct server *)emalloc(sizeof(struct server));
-		memset((char *)server, 0, sizeof(struct server));
+		server = emalloc_zero(sizeof(*server));
 
-		memcpy(&server->srcadr, addr, sizeof(struct sockaddr_storage));
+		server->srcadr = *addr;
 
 		server->event_time = ++sys_numservers;
 
 		for (sp = sys_servers; sp->next_server != NULL;
-		     sp = sp->next_server) ;
+		     sp = sp->next_server)
+			/* empty */;
 		sp->next_server = server;
 		transmit(server);
 	}
@@ -1513,7 +1518,7 @@ alarming(
 {
 	alarm_flag++;
 }
-#else
+#else	/* SYS_WINNT follows */
 void CALLBACK 
 alarming(UINT uTimerID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
 {
@@ -1539,16 +1544,14 @@ static void
 init_alarm(void)
 {
 #ifndef SYS_WINNT
-# ifndef HAVE_TIMER_SETTIME
-	struct itimerval itimer;
+# ifdef HAVE_TIMER_CREATE
+	struct itimerspec its;
 # else
-	struct itimerspec ntpdate_itimer;
+	struct itimerval itv;
 # endif
-#else
+#else	/* SYS_WINNT follows */
 	TIMECAPS tc;
 	UINT wTimerID;
-# endif /* SYS_WINNT */
-#if defined SYS_CYGWIN32 || defined SYS_WINNT
 	HANDLE hToken;
 	TOKEN_PRIVILEGES tkp;
 	DWORD dwUser = 0;
@@ -1557,7 +1560,7 @@ init_alarm(void)
 	alarm_flag = 0;
 
 #ifndef SYS_WINNT
-# if defined(HAVE_TIMER_CREATE) && defined(HAVE_TIMER_SETTIME)
+# ifdef HAVE_TIMER_CREATE
 	alarm_flag = 0;
 	/* this code was put in as setitimer() is non existant this us the
 	 * POSIX "equivalents" setup - casey
@@ -1579,44 +1582,26 @@ init_alarm(void)
 	 * Set up the alarm interrupt.	The first comes 1/(2*TIMER_HZ)
 	 * seconds from now and they continue on every 1/TIMER_HZ seconds.
 	 */
-	(void) signal_no_reset(SIGALRM, alarming);
-	ntpdate_itimer.it_interval.tv_sec = ntpdate_itimer.it_value.tv_sec = 0;
-	ntpdate_itimer.it_interval.tv_nsec = 1000000000/TIMER_HZ;
-	ntpdate_itimer.it_value.tv_nsec = 1000000000/(TIMER_HZ<<1);
-	timer_settime(ntpdate_timerid, 0 /* !TIMER_ABSTIME */, &ntpdate_itimer, NULL);
-# else
+	signal_no_reset(SIGALRM, alarming);
+	its.it_interval.tv_sec = 0;
+	its.it_value.tv_sec = 0;
+	its.it_interval.tv_nsec = 1000000000/TIMER_HZ;
+	its.it_value.tv_nsec = 1000000000/(TIMER_HZ<<1);
+	timer_settime(ntpdate_timerid, 0 /* !TIMER_ABSTIME */, &its, NULL);
+# else	/* !HAVE_TIMER_CREATE follows */
 	/*
 	 * Set up the alarm interrupt.	The first comes 1/(2*TIMER_HZ)
 	 * seconds from now and they continue on every 1/TIMER_HZ seconds.
 	 */
-	(void) signal_no_reset(SIGALRM, alarming);
-	itimer.it_interval.tv_sec = itimer.it_value.tv_sec = 0;
-	itimer.it_interval.tv_usec = 1000000/TIMER_HZ;
-	itimer.it_value.tv_usec = 1000000/(TIMER_HZ<<1);
+	signal_no_reset(SIGALRM, alarming);
+	itv.it_interval.tv_sec = 0;
+	itv.it_value.tv_sec = 0;
+	itv.it_interval.tv_usec = 1000000/TIMER_HZ;
+	itv.it_value.tv_usec = 1000000/(TIMER_HZ<<1);
 
-	setitimer(ITIMER_REAL, &itimer, (struct itimerval *)0);
-# endif
-#if defined SYS_CYGWIN32
-	/*
-	 * Get privileges needed for fiddling with the clock
-	 */
-
-	/* get the current process token handle */
-	if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) {
-		msyslog(LOG_ERR, "OpenProcessToken failed: %m");
-		exit(1);
-	}
-	/* get the LUID for system-time privilege. */
-	LookupPrivilegeValue(NULL, SE_SYSTEMTIME_NAME, &tkp.Privileges[0].Luid);
-	tkp.PrivilegeCount = 1;		/* one privilege to set */
-	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
-	/* get set-time privilege for this process. */
-	AdjustTokenPrivileges(hToken, FALSE, &tkp, 0,(PTOKEN_PRIVILEGES) NULL, 0);
-	/* cannot test return value of AdjustTokenPrivileges. */
-	if (GetLastError() != ERROR_SUCCESS)
-		msyslog(LOG_ERR, "AdjustTokenPrivileges failed: %m");
-#endif
-#else	/* SYS_WINNT */
+	setitimer(ITIMER_REAL, &itv, NULL);
+# endif	/* !HAVE_TIMER_CREATE */
+#else	/* SYS_WINNT follows */
 	_tzset();
 
 	/*
@@ -1694,7 +1679,9 @@ init_io(void)
 {
 	struct addrinfo *res, *ressave;
 	struct addrinfo hints;
+	sockaddr_u addr;
 	char service[5];
+	int rc;
 	int optval = 1;
 	int check_ntp_port_in_use = !debug && !simple_query && !unpriv_port;
 
@@ -1707,25 +1694,25 @@ init_io(void)
 	 * Open the socket
 	 */
 
-	strcpy(service, "ntp");
+	strncpy(service, "ntp", sizeof(service));
 
 	/*
 	 * Init hints addrinfo structure
 	 */
-	memset(&hints, 0, sizeof(hints));
+	ZERO(hints);
 	hints.ai_family = ai_fam_templ;
 	hints.ai_flags = AI_PASSIVE;
 	hints.ai_socktype = SOCK_DGRAM;
 
-	if(getaddrinfo(NULL, service, &hints, &res) != 0) {
-	       msyslog(LOG_ERR, "getaddrinfo() failed: %m");
-	       exit(1);
-	       /*NOTREACHED*/
+	if (getaddrinfo(NULL, service, &hints, &res) != 0) {
+		msyslog(LOG_ERR, "getaddrinfo() failed: %m");
+		exit(1);
+		/*NOTREACHED*/
 	}
 
 #ifdef SYS_WINNT
 	if (check_ntp_port_in_use && ntp_port_inuse(AF_INET, NTP_PORT)){
-		netsyslog(LOG_ERR, "the NTP socket is in use, exiting: %m");
+		msyslog(LOG_ERR, "the NTP socket is in use, exiting: %m");
 		exit(1);
 	}
 #endif
@@ -1749,13 +1736,13 @@ init_io(void)
 		    err == WSAEPFNOSUPPORT)
 #endif
 			continue;
-		netsyslog(LOG_ERR, "socket() failed: %m");
+		msyslog(LOG_ERR, "socket() failed: %m");
 		exit(1);
 		/*NOTREACHED*/
 		}
 		/* set socket to reuse address */
 		if (setsockopt(fd[nbsock], SOL_SOCKET, SO_REUSEADDR, (void*) &optval, sizeof(optval)) < 0) {
-				netsyslog(LOG_ERR, "setsockopt() SO_REUSEADDR failed: %m");
+				msyslog(LOG_ERR, "setsockopt() SO_REUSEADDR failed: %m");
 				exit(1);
 				/*NOTREACHED*/
 		}
@@ -1763,7 +1750,7 @@ init_io(void)
 		/* Restricts AF_INET6 socket to IPv6 communications (see RFC 2553bis-03) */
 		if (res->ai_family == AF_INET6)
 			if (setsockopt(fd[nbsock], IPPROTO_IPV6, IPV6_V6ONLY, (void*) &optval, sizeof(optval)) < 0) {
-				   netsyslog(LOG_ERR, "setsockopt() IPV6_V6ONLY failed: %m");
+				   msyslog(LOG_ERR, "setsockopt() IPV6_V6ONLY failed: %m");
 					exit(1);
 					/*NOTREACHED*/
 		}
@@ -1776,15 +1763,15 @@ init_io(void)
 		 * bind the socket to the NTP port
 		 */
 		if (check_ntp_port_in_use) {
-			if (bind(fd[nbsock], res->ai_addr, SOCKLEN(res->ai_addr)) < 0) {
-#ifndef SYS_WINNT
-				if (errno == EADDRINUSE)
-#else
-				if (WSAGetLastError() == WSAEADDRINUSE)
-#endif /* SYS_WINNT */
-					netsyslog(LOG_ERR, "the NTP socket is in use, exiting");
+			ZERO(addr);
+			INSIST(res->ai_addrlen <= sizeof(addr));
+			memcpy(&addr, res->ai_addr, res->ai_addrlen);
+			rc = bind(fd[nbsock], &addr.sa, SOCKLEN(&addr));
+			if (rc < 0) {
+				if (EADDRINUSE == socket_errno())
+					msyslog(LOG_ERR, "the NTP socket is in use, exiting");
 				else
-					netsyslog(LOG_ERR, "bind() fails: %m");
+					msyslog(LOG_ERR, "bind() fails: %m");
 				exit(1);
 			}
 		}
@@ -1805,24 +1792,24 @@ init_io(void)
 #ifndef SYS_WINNT
 # ifdef SYS_VXWORKS
 		{
-		int on = TRUE;
+			int on = TRUE;
 
-		if (ioctl(fd[nbsock],FIONBIO, &on) == ERROR) {
-		  netsyslog(LOG_ERR, "ioctl(FIONBIO) fails: %m");
-			exit(1);
-		}
+			if (ioctl(fd[nbsock],FIONBIO, &on) == ERROR) {
+				msyslog(LOG_ERR, "ioctl(FIONBIO) fails: %m");
+				exit(1);
+			}
 		}
 # else /* not SYS_VXWORKS */
 #  if defined(O_NONBLOCK)
 		if (fcntl(fd[nbsock], F_SETFL, O_NONBLOCK) < 0) {
-			netsyslog(LOG_ERR, "fcntl(FNDELAY|FASYNC) fails: %m");
+			msyslog(LOG_ERR, "fcntl(FNDELAY|FASYNC) fails: %m");
 			exit(1);
 			/*NOTREACHED*/
 		}
 #  else /* not O_NONBLOCK */
 #	if defined(FNDELAY)
 		if (fcntl(fd[nbsock], F_SETFL, FNDELAY) < 0) {
-			netsyslog(LOG_ERR, "fcntl(FNDELAY|FASYNC) fails: %m");
+			msyslog(LOG_ERR, "fcntl(FNDELAY|FASYNC) fails: %m");
 			exit(1);
 			/*NOTREACHED*/
 		}
@@ -1833,7 +1820,7 @@ init_io(void)
 # endif /* SYS_VXWORKS */
 #else /* SYS_WINNT */
 		if (ioctlsocket(fd[nbsock], FIONBIO, (u_long *) &on) == SOCKET_ERROR) {
-			netsyslog(LOG_ERR, "ioctlsocket(FIONBIO) fails: %m");
+			msyslog(LOG_ERR, "ioctlsocket(FIONBIO) fails: %m");
 			exit(1);
 		}
 #endif /* SYS_WINNT */
@@ -1847,7 +1834,7 @@ init_io(void)
  */
 static void
 sendpkt(
-	struct sockaddr_storage *dest,
+	sockaddr_u *dest,
 	struct pkt *pkt,
 	int len
 	)
@@ -1862,14 +1849,14 @@ sendpkt(
 
 	/* Find a local family compatible socket to send ntp packet to ntp server */
 	for(i = 0; (i < MAX_AF); i++) {
-		if(dest->ss_family == fd_family[i]) {
+		if(AF(dest) == fd_family[i]) {
 			sock = fd[i];
 		break;
 		}
 	}
 
-	if ( sock == INVALID_SOCKET ) {
-		netsyslog(LOG_ERR, "cannot find family compatible socket to send ntp packet");
+	if (INVALID_SOCKET == sock) {
+		msyslog(LOG_ERR, "cannot find family compatible socket to send ntp packet");
 		exit(1);
 		/*NOTREACHED*/
 	}
@@ -1877,14 +1864,14 @@ sendpkt(
 	cc = sendto(sock, (char *)pkt, len, 0, (struct sockaddr *)dest,
 			SOCKLEN(dest));
 
-	if (cc == SOCKET_ERROR) {
+	if (SOCKET_ERROR == cc) {
 #ifndef SYS_WINNT
 		if (errno != EWOULDBLOCK && errno != ENOBUFS)
 #else
 		err = WSAGetLastError();
 		if (err != WSAEWOULDBLOCK && err != WSAENOBUFS)
 #endif /* SYS_WINNT */
-			netsyslog(LOG_ERR, "sendto(%s): %m", stohost(dest));
+			msyslog(LOG_ERR, "sendto(%s): %m", stohost(dest));
 	}
 }
 
@@ -1898,7 +1885,7 @@ input_handler(void)
 	register int n;
 	register struct recvbuf *rb;
 	struct sock_timeval tvzero;
-	int fromlen;
+	GETSOCKNAME_SOCKLEN_TYPE fromlen;
 	l_fp ts;
 	int i;
 #ifdef HAVE_POLL_H
@@ -1954,7 +1941,7 @@ input_handler(void)
 			return;
 		else if (n == -1) {
 			if (errno != EINTR)
-				netsyslog(LOG_ERR,
+				msyslog(LOG_ERR,
 #ifdef HAVE_POLL_H
 					"poll() error: %m"
 #else
@@ -1989,7 +1976,7 @@ input_handler(void)
 
 		rb = get_free_recv_buffer();
 
-		fromlen = sizeof(struct sockaddr_storage);
+		fromlen = sizeof(rb->recv_srcadr);
 		rb->recv_length = recvfrom(fdc, (char *)&rb->recv_pkt,
 		   sizeof(rb->recv_pkt), 0,
 		   (struct sockaddr *)&rb->recv_srcadr, &fromlen);
@@ -2137,7 +2124,7 @@ printserver(
 {
 	register int i;
 	char junk[5];
-	char *str;
+	const char *str;
 
 	if (!debug) {
 		(void) fprintf(fp, "server %s, stratum %d, offset %s, delay %s\n",
@@ -2259,7 +2246,7 @@ getnetinfoservers(void)
 	ni_status status;
 	void *domain;
 	ni_id confdir;
-	ni_namelist *namelist = (ni_namelist*)malloc(sizeof(ni_namelist));
+	ni_namelist *namelist = emalloc(sizeof(ni_namelist));
 
 	/* Find a time server in NetInfo */
 	if ((status = ni_open(NULL, ".", &domain)) != NI_OK) return NULL;
