@@ -4,6 +4,7 @@
 #include "ntp_fp.h"
 #include "ntp.h"
 #include "clockstuff.h"
+#include "ntp_worker.h"
 
 # if defined(HAVE_IO_COMPLETION_PORT)
 
@@ -14,14 +15,12 @@ extern	int	io_completion_port_add_socket(SOCKET fd, struct interface *);
 
 struct refclockio; /* in ntp_refclock.h but inclusion here triggers problems */
 extern	int	io_completion_port_add_clock_io(struct refclockio *rio);
-
-extern	int	io_completion_port_sendto(struct interface *, struct pkt *, int, sockaddr_u *);
-
-extern	HANDLE	get_io_event(void);
-extern	HANDLE	get_exit_event(void);
+extern	int	io_completion_port_sendto(int, void *, size_t, sockaddr_u *);
 
 extern	int	GetReceivedBuffers(void);
 
+extern	HANDLE	WaitableExitEventHandle;
+
 # endif
 
 #endif
