@@ -2,9 +2,16 @@
  *
  * Created by Sven Dietrich  sven@inter-yacht.com
  *
- * New interpolation scheme by Dave Hart <davehart@davehart.com> February 2009
- * overcomes 500us-1ms inherent jitter with the older scheme, first identified
- * by Peter Rosin (nee Ekberg) <peda@lysator.liu.se> in 2003 [Bug 216].
+ * New interpolation scheme by Dave Hart <davehart@davehart.com> in
+ * February 2009 overcomes 500us-1ms inherent jitter with the older
+ * scheme, first identified by Peter Rosin (nee Ekberg)
+ * <peda@lysator.liu.se> in 2003 [Bug 216].
+ *
+ * Note:  The Windows port of ntpd uses the C99-snprintf replacement for
+ * (v)snprintf(), also used by msyslog(), which does not understand the
+ * printf format specifier %I64d, only the more common %lld.  With the
+ * minimum supported compiler raised to Visual C++ 2005 in ntp-dev in
+ * August 2011, all MS C runtime routines also understand %lld and %llu.
  */
 
 
@@ -32,10 +39,11 @@
 #include "ntp_stdlib.h"
 #include "ntp_unixtime.h"
 #include "ntp_timer.h"
+#include "ntp_assert.h"
 #include "clockstuff.h"
 #include "ntservice.h"
 #include "ntpd.h"
-#include "../../../ntpd/ntpd-opts.h"
+#include "ntpd-opts.h"
 
 extern double sys_residual;	/* residual from previous adjustment */
 
@@ -77,7 +85,6 @@ static LONGLONG ls_elapsed;
 
 static BOOL winnt_time_initialized = FALSE;
 static BOOL winnt_use_interpolation = FALSE;
-static ULONGLONG last_interp_time;
 static unsigned clock_thread_id;
 
 
@@ -86,7 +93,7 @@ static void StartClockThread(void);
 static void tune_ctr_freq(LONGLONG, LONGLONG);
 void StopClockThread(void);
 void atexit_revert_mm_timer(void);
-void time_stepped(void);
+void win_time_stepped(void);
 
 static HANDLE clock_thread = NULL;
 static HANDLE TimerThreadExitRequest = NULL;
@@ -118,8 +125,9 @@ static volatile int	newest_baseline_gen = 0;
 static ULONGLONG	baseline_counts[BASELINES_TOT] = {0};
 static LONGLONG		baseline_times[BASELINES_TOT] = {0};
 
+#define CLOCK_BACK_THRESHOLD	100	/* < 10us unremarkable */
+static ULONGLONG	clock_backward_max = CLOCK_BACK_THRESHOLD;
 static int		clock_backward_count;
-static ULONGLONG	clock_backward_max;
 
 
 /*
@@ -168,17 +176,31 @@ volatile ULONGLONG PerfCtrFreq = 0;
 static LONGLONG QPC_offset = 0;
 
 /*
- * use_pcc could be a local static in perf_ctr but
- * lock_thread_to_processor cares about it.
+ * Substitute RDTSC for QueryPerformanceCounter()?
  */
 static int use_pcc = -1;
 
+/*
+ * Restrict threads that call QPC/RDTSC to one CPU?
+ */
+static int lock_interp_threads = -1;
+
 /*
  * ppm_per_adjust_unit is parts per million effect on the OS
  * clock per slewing adjustment unit per second.  Per haps.
  */
 static DOUBLE ppm_per_adjust_unit = 0.0;
 
+/*
+ * wintickadj emulates the functionality provided by unix tickadj,
+ * providing a baseline clock correction if needed to get the
+ * clock within a few hundred PPM of correct frequency.
+ */
+static long wintickadj;
+
+static void	choose_interp_counter(void);
+static int	is_qpc_built_on_pcc(void);
+
 /*
  * performance counter frequency observations
  */
@@ -201,26 +223,14 @@ do {	\
 	(t) -= HECTONANOSECONDS / 2;	\
 } while (0)
 
-/*
- * workaround for VC6 inability to convert unsigned __int64 to double
- */
-#define	LL_HNS	((LONGLONG)HECTONANOSECONDS)
-
 /*
  * NT native time format is 100's of nanoseconds since 1601-01-01.
  * Helpers for converting between "hectonanoseconds" and the 
  * performance counter scale from which interpolated time is
  * derived.
- *
- * Once support for VC6 is dropped, the cast of PerfCtrFreq to
- * LONGLONG can come out of PERF2HNS().  It avoids the VC6 error
- * message:
- * 
- * conversion from unsigned __int64 to double not implemented, use
- * signed __int64
  */
-#define HNS2PERF(hns)	((hns) * PerfCtrFreq / LL_HNS)
-#define PERF2HNS(ctr)	((ctr) * LL_HNS / (LONGLONG)PerfCtrFreq)
+#define HNS2PERF(hns)	((hns) * PerfCtrFreq / HECTONANOSECONDS)
+#define PERF2HNS(ctr)	((ctr) * HECTONANOSECONDS / PerfCtrFreq)
 
 
 #if defined(_MSC_VER) && _MSC_VER >= 1400	/* VS 2005 */
@@ -239,125 +249,151 @@ get_pcc(void)
 }
 #endif
 
+
 /*
  * perf_ctr() returns the current performance counter value, 
  * from QueryPerformanceCounter or RDTSC.
  */
-
 ULONGLONG WINAPI 
 perf_ctr(void)
 {
-	FT_ULL ft1;
+	FT_ULL ft;
 
-	if (1 == use_pcc) {
+	if (use_pcc)
 		return get_pcc();
-	} else if (0 == use_pcc) {
-		QueryPerformanceCounter(&ft1.li);
-		return ft1.ull;
-	} else {
-		FT_ULL ft2;
-		FT_ULL ft3;
-		FT_ULL ft4;
-		FT_ULL ft5;
-		LONGLONG offset;
-		const char *ntpd_pcc_freq_text;
-
-		/* one-time initialization */
-
-		/*
-		 * It's time to make some more permanent knobs,
-		 * but for right now the RDTSC aka PCC dance on x86 is:
-		 *
-		 * 1.  With none of these variables defined, only QPC
-		 *     is used because there is no reliable way to
-		 *     detect counter frequency variation after ntpd
-		 *     startup implemented.
-		 * 2.  We need a better knob, but for now if you know
-		 *     your RDTSC / CPU frequency is invariant, set
-		 *     NTPD_PCC and assuming your QPC is based on the
-		 *     PCC as well, RDTSC will be substituted.
-		 * 3.  More forcefully, you can jam in a desired exact
-		 *     processor frequency, expressed in cycles per
-		 *     second by setting NTPD_PCC_FREQ=398125000, for
-		 *     example, if yor actual known CPU frequency is
-		 *     398.125 MHz, and NTPD_PCC doesn't work because
-		 *     QueryPerformanceCounter is implemented using
-		 *     another counter.  It is very easy to make ntpd
-		 *     fall down if the NTPD_PCC_FREQ value isn't very
-		 *     close to the observed RDTSC units per second.
-		 *
-		 * Items 2 and 3 could probably best be combined into one
-		 * new windows-specific command line switch such as
-		 *   ntpd --pcc
-		 * or
-		 *   ntpd --pcc=398125000
-		 *
-		 * They are currently tied to Windows because that is
-		 * the only ntpd port with its own interpolation, and
-		 * to x86/x64 because no one has ported the Windows
-		 * ntpd port to the sole remaining alternative, Intel
-		 * Itanium.
-		 */
-		if (HAVE_OPT(PCCFREQ))
-			ntpd_pcc_freq_text = OPT_ARG(PCCFREQ);
-		else
-			ntpd_pcc_freq_text = getenv("NTPD_PCC_FREQ");
-
-		if (!HAVE_OPT(USEPCC)
-		    && NULL == ntpd_pcc_freq_text
-		    && NULL == getenv("NTPD_PCC")) {
-			use_pcc = 0;
-			QueryPerformanceCounter(&ft1.li);
-			return ft1.ull;
-		}
-
-#ifdef DEBUG
-		if (!NomPerfCtrFreq) {
-			msyslog(LOG_ERR, "NomPerfCtrFreq not initialized before first perf_ctr() call");
-			exit(-1);
-		}
-#endif
-
-		QueryPerformanceCounter(&ft1.li);
-		ft2.ull = get_pcc();
-		Sleep(1);
-		QueryPerformanceCounter(&ft3.li);
-		Sleep(1);
-		ft4.ull = get_pcc();
-		Sleep(1);
-		QueryPerformanceCounter(&ft5.li);
-
-		offset = ft2.ull - ft1.ull;
-		ft3.ull += offset;
-		ft5.ull += offset;
-
-		if (ntpd_pcc_freq_text  ||
-			(ft2.ull <= ft3.ull &&
-			 ft3.ull <= ft4.ull &&
-			 ft4.ull <= ft5.ull)) {
-
-			use_pcc = 1;
-			ft1.ull = ft2.ull;
-			QPC_offset = offset;
-
-			if (ntpd_pcc_freq_text)
-				sscanf(ntpd_pcc_freq_text, 
-				       "%I64u", 
-				       &NomPerfCtrFreq);
-
-			NLOG(NLOG_CLOCKINFO)
-				msyslog(LOG_INFO, 
-					"using processor cycle counter "
-					"%.3f MHz", 
-					(LONGLONG)NomPerfCtrFreq / 1e6);
-		} else {
-			use_pcc = 0;
-		}
-
-		return ft1.ull;
+	else {
+		QueryPerformanceCounter(&ft.li);
+		return ft.ull;
 	}
 }
 
+
+/*
+ * choose_interp_counter - select between QueryPerformanceCounter and
+ *			   the x86 processor cycle counter (TSC).
+ */
+static void
+choose_interp_counter(void)
+{
+	const char *	ntpd_pcc_freq_text;
+	int		qpc_built_on_pcc;
+
+	/*
+	 * Regardless of whether we actually use RDTSC, first determine
+	 * if QueryPerformanceCounter is built on it, so that we can
+	 * decide whether it's prudent to lock QPC-consuming threads to
+	 * a particular CPU.
+	 */
+	qpc_built_on_pcc = is_qpc_built_on_pcc();
+	lock_interp_threads = qpc_built_on_pcc;
+
+	/*
+	 * It's time to make some more permanent knobs,
+	 * but for right now the RDTSC aka PCC dance on x86 is:
+	 *
+	 * 1.  With none of these variables defined, only QPC
+	 *     is used because there is no reliable way to
+	 *     detect counter frequency variation after ntpd
+	 *     startup implemented.
+	 * 2.  We need a better knob, but for now if you know
+	 *     your RDTSC / CPU frequency is invariant, set
+	 *     NTPD_PCC and assuming your QPC is based on the
+	 *     PCC as well, RDTSC will be substituted.
+	 * 3.  More forcefully, you can jam in a desired exact
+	 *     processor frequency, expressed in cycles per
+	 *     second by setting NTPD_PCC_FREQ=398125000, for
+	 *     example, if yor actual known CPU frequency is
+	 *     398.125 MHz, and NTPD_PCC doesn't work because
+	 *     QueryPerformanceCounter is implemented using
+	 *     another counter.  It is very easy to make ntpd
+	 *     fall down if the NTPD_PCC_FREQ value isn't very
+	 *     close to the observed RDTSC units per second.
+	 *
+	 * Items 2 and 3 could probably best be combined into one
+	 * new windows-specific command line switch such as
+	 *   ntpd --pcc
+	 * or
+	 *   ntpd --pcc=398125000
+	 *
+	 * They are currently tied to Windows because that is
+	 * the only ntpd port with its own interpolation, and
+	 * to x86/x64 because no one has ported the Windows
+	 * ntpd port to the sole remaining alternative, Intel
+	 * Itanium.
+	 */
+	if (HAVE_OPT(PCCFREQ))
+		ntpd_pcc_freq_text = OPT_ARG(PCCFREQ);
+	else
+		ntpd_pcc_freq_text = getenv("NTPD_PCC_FREQ");
+
+	if (!HAVE_OPT(USEPCC)
+	    && NULL == ntpd_pcc_freq_text
+	    && NULL == getenv("NTPD_PCC")) {
+		use_pcc = 0;
+		return;
+	}
+
+	if (!qpc_built_on_pcc && NULL == ntpd_pcc_freq_text) {
+		use_pcc = 0;
+		return;
+	}
+
+	use_pcc = 1;
+	if (ntpd_pcc_freq_text != NULL)
+		sscanf(ntpd_pcc_freq_text, 
+		       "%llu", 
+		       &NomPerfCtrFreq);
+
+	NLOG(NLOG_CLOCKINFO)
+		msyslog(LOG_INFO, 
+			"using processor cycle counter "
+			"%.3f MHz", 
+			NomPerfCtrFreq / 1e6);
+	return;
+}
+
+
+/*
+ * is_qpc_built_on_pcc - test if QueryPerformanceCounter runs at the
+ *			 same rate as the processor cycle counter (TSC).
+ */
+static int
+is_qpc_built_on_pcc(void)
+{
+	LONGLONG	offset;
+	FT_ULL		ft1;
+	FT_ULL		ft2;
+	FT_ULL		ft3;
+	FT_ULL		ft4;
+	FT_ULL		ft5;
+
+	NTP_REQUIRE(NomPerfCtrFreq != 0);
+
+	QueryPerformanceCounter(&ft1.li);
+	ft2.ull = get_pcc();
+	Sleep(1);
+	QueryPerformanceCounter(&ft3.li);
+	Sleep(1);
+	ft4.ull = get_pcc();
+	Sleep(1);
+	QueryPerformanceCounter(&ft5.li);
+
+	offset = ft2.ull - ft1.ull;
+	ft3.ull += offset;
+	ft5.ull += offset;
+
+	if (ft2.ull <= ft3.ull &&
+	    ft3.ull <= ft4.ull &&
+	    ft4.ull <= ft5.ull) {
+
+		QPC_offset = offset;
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 /*
  * Request Multimedia Timer
  */
@@ -370,7 +406,14 @@ set_mm_timer(
 }
 
 /*
- * adj_systime - called once every second to make system time adjustments.
+ * adj_systime - called once every second to discipline system clock.
+ * Normally, the offset passed in (parameter now) is in the range
+ * [-NTP_MAXFREQ, NTP_MAXFREQ].  However, at EVNT_NSET, a much larger
+ * slew is requested if the initial offset is less than the step
+ * threshold, in the range [-step, step] where step is the step
+ * threshold, 128 msec by default.  For the remainder of the frequency
+ * training interval, adj_systime is called with 0 offset each second
+ * and slew the large offset at 500 PPM (500 usec/sec).
  * Returns 1 if okay, 0 if trouble.
  */
 int
@@ -378,31 +421,41 @@ adj_systime(
 	double now
 	)
 {
-	double dtemp;
-	u_char isneg = 0;
-	int rc;
-	long TimeAdjustment;
+	static double	adjtime_carry;
+	double		dtemp;
+	u_char		isneg;
+	BOOL		rc;
+	long		TimeAdjustment;
+	SYSTEMTIME	st;
+	ULONGLONG	this_perf_count;
+	FT_ULL		curr_ft;
 
 	/*
 	 * Add the residual from the previous adjustment to the new
 	 * adjustment, bound and round.
 	 */
-	dtemp = sys_residual + now;
-	sys_residual = 0;
-	if (dtemp < 0)
-	{
-		isneg = 1;
+	dtemp = adjtime_carry + sys_residual + now;
+	adjtime_carry = 0.;
+	sys_residual = 0.;
+	if (dtemp < 0) {
+		isneg = TRUE;
 		dtemp = -dtemp;
+	} else {
+		isneg = FALSE;
 	}
 
-	if (dtemp > NTP_MAXFREQ)
+	if (dtemp > NTP_MAXFREQ) {
+		adjtime_carry = dtemp - NTP_MAXFREQ;
 		dtemp = NTP_MAXFREQ;
+	}
+
+	if (isneg) {
+		dtemp = -dtemp;
+		adjtime_carry = -adjtime_carry;
+	}
 
 	dtemp = dtemp * 1e6;
 
-	if (isneg)
-		dtemp = -dtemp;
-
 	/* 
 	 * dtemp is in micro seconds. NT uses 100 ns units,
 	 * so a unit change in TimeAdjustment corresponds
@@ -411,34 +464,32 @@ adj_systime(
 	 * frequency as per suggestion from Harry Pyle,
 	 * and leave the remainder in dtemp
 	 */
-	TimeAdjustment = (long) (dtemp / ppm_per_adjust_unit + (isneg ? -0.5 : 0.5));
-	dtemp -= (double) TimeAdjustment * ppm_per_adjust_unit;	
+	TimeAdjustment = (long)(dtemp / ppm_per_adjust_unit +
+				((isneg)
+				     ? -0.5
+				     : 0.5));
+	dtemp -= TimeAdjustment * ppm_per_adjust_unit;	
 
 
 	/*
-	 * If a leap second is pending then determine the UTC time stamp 
-	 * of when the insertion must take place 
+	 * If a leap second is pending for the end of the month,
+	 * determine the UTC time stamp when the insertion must take
+	 * place 
 	 */
-	if (leap_sec > 0)
-	{
-		if ( ls_ft.ull == 0 )  /* time stamp has not yet been computed */
-		{
-			SYSTEMTIME st;
-
+	if (0 < leapsec && leapsec < 31 * DAY) {
+		if (0 == ls_ft.ull) {  /* time stamp has not yet been computed */
 			GetSystemTime(&st);
 
  			/*
 			 * Accept leap announcement only 1 month in advance,
 			 * for end of March, June, September, or December.
 			 */
-			if ( ( st.wMonth % 3 ) == 0 )
-			{
+			if (0 == (st.wMonth % 3)) {
 				/*
 				 * The comparison time stamp is computed according 
 				 * to 0:00h UTC of the following day 
 				 */
-				if ( ++st.wMonth > 12 )
-				{
+				if (++st.wMonth > 12) {
 					st.wMonth -= 12;
 					st.wYear++;
 				}
@@ -457,42 +508,43 @@ adj_systime(
 					st.wHour, st.wMinute, st.wSecond);
 			}
 		}
+	} else {
+		if (ls_ft.ull != 0) {  /* Leap second has been armed before */
+			/*
+			 * Disarm leap second only if the leap second
+			 * is not already in progress.
+			 */
+			if (0 == ls_time_adjustment) {
+				ls_ft.ull = 0;
+				msyslog(LOG_NOTICE, "Leap second announcement disarmed");
+			}
+		}
 	}
 
- 
+
 	/*
 	 * If the time stamp for the next leap second has been set
 	 * then check if the leap second must be handled
 	 */
-	if ( ls_ft.ull )
-	{
-		ULONGLONG this_perf_count;
-
+	if (ls_ft.ull != 0) {
 		this_perf_count = perf_ctr();
 
-		if ( ls_time_adjustment == 0 ) /* has not yet been scheduled */
-		{
-			FT_ULL curr_ft;
+		if (0 == ls_time_adjustment) { /* has not yet been scheduled */
 
-	 		GetSystemTimeAsFileTime(&curr_ft.ft);   
-			if ( curr_ft.ull >= ls_ft.ull )
-			{
+	 		GetSystemTimeAsFileTime(&curr_ft.ft);
+			if (curr_ft.ull >= ls_ft.ull) {
 				ls_time_adjustment = clockperiod / LS_CORR_INTV_SECS;
 				ls_ref_perf_cnt = this_perf_count;
 				ls_elapsed = 0;
 				msyslog(LOG_NOTICE, "Inserting positive leap second.");
 			}
-		}
-		else  /* leap sec adjustment has been scheduled previously */
-		{
-			ls_elapsed = ( this_perf_count - ls_ref_perf_cnt ) 
-				       * HECTONANOSECONDS / PerfCtrFreq;
+		} else {  /* leap sec adjustment has been scheduled previously */
+			ls_elapsed = (this_perf_count - ls_ref_perf_cnt) 
+				     * HECTONANOSECONDS / PerfCtrFreq;
 		}
 
-		if ( ls_time_adjustment )  /* leap second adjustment is currently active */
-		{
-			if ( ls_elapsed > ( LS_CORR_INTV - LS_CORR_LIMIT ) )
-			{
+		if (ls_time_adjustment != 0) {  /* leap second adjustment is currently active */
+			if (ls_elapsed > (LS_CORR_INTV - LS_CORR_LIMIT)) {
 				ls_time_adjustment = 0;  /* leap second adjustment done */
 				ls_ft.ull = 0;
 			}
@@ -501,38 +553,41 @@ adj_systime(
 			 * NOTE: While the system time is slewed during the leap second 
 			 * the interpolation function which is based on the performance 
 			 * counter does not account for the slew.
-			*/
+			 */
 			TimeAdjustment -= ls_time_adjustment;
 		}
 	}
 
 
+	sys_residual = dtemp / 1e6;
+	DPRINTF(3, ("adj_systime: %.9f -> %.9f residual %.9f", 
+		    now, 1e-6 * (TimeAdjustment * ppm_per_adjust_unit),
+		    sys_residual));
+	if (0. == adjtime_carry)
+		DPRINTF(3, ("\n"));
+	else
+		DPRINTF(3, (" adjtime %.9f\n", adjtime_carry));
+
 	/* only adjust the clock if adjustment changes */
+	TimeAdjustment += wintickadj;
 	if (last_Adj != TimeAdjustment) {
 		last_Adj = TimeAdjustment;
-		DPRINTF(1, ("SetSystemTimeAdjustment(%+ld)\n", TimeAdjustment));
-		rc = !SetSystemTimeAdjustment(clockperiod + TimeAdjustment, FALSE);
-	}
-	else rc = 0;
-	if (rc)
-	{
-		msyslog(LOG_ERR, "Can't adjust time: %m");
-		return 0;
-	}
-	else {
-		sys_residual = dtemp / 1e6;
+		DPRINTF(2, ("SetSystemTimeAdjustment(%+ld)\n", TimeAdjustment));
+		rc = SetSystemTimeAdjustment(clockperiod + TimeAdjustment, FALSE);
+		if (!rc)
+			msyslog(LOG_ERR, "Can't adjust time: %m");
+	} else {
+		rc = TRUE;
 	}
 
-	DPRINTF(6, ("adj_systime: adj %.9f -> remaining residual %.9f\n", 
-		    now, sys_residual));
-
-	return 1;
+	return rc;
 }
 
 
 void 
 init_winnt_time(void)
 {
+	static const char settod[] = "settimeofday=\"SetSystemTime\"";
 	char szMsgPath[MAX_PATH+1];
 	HANDLE hToken = INVALID_HANDLE_VALUE;
 	TOKEN_PRIVILEGES tkp;
@@ -542,6 +597,9 @@ init_winnt_time(void)
 	LARGE_INTEGER Freq;
 	FT_ULL initial_hectonanosecs;
 	FT_ULL next_hectonanosecs;
+	double adjppm;
+	double rawadj;
+	char * pch;
 
 	if (winnt_time_initialized)
 		return;
@@ -553,14 +611,12 @@ init_winnt_time(void)
 	ntservice_init();
 
 	/* Set up the Console Handler */
-	if (!SetConsoleCtrlHandler(OnConsoleEvent, TRUE))
-	{
+	if (!SetConsoleCtrlHandler(OnConsoleEvent, TRUE)) {
 		msyslog(LOG_ERR, "Can't set console control handler: %m");
 	}
 
 	/* Set the Event-ID message-file name. */
-	if (!GetModuleFileName(NULL, szMsgPath, sizeof(szMsgPath))) 
-	{
+	if (!GetModuleFileName(NULL, szMsgPath, sizeof(szMsgPath))) {
 		msyslog(LOG_ERR, "GetModuleFileName(PGM_EXE_FILE) failed: %m\n");
 		exit(1);
 	}
@@ -576,7 +632,6 @@ init_winnt_time(void)
 	if (SIZEOF_TIME_T != sizeof(time_t)
 	    || SIZEOF_INT != sizeof(int)
 	    || SIZEOF_SIGNED_CHAR != sizeof(char)) {
-	
 		msyslog(LOG_ERR, "config.h SIZEOF_* macros wrong, fatal");
 		exit(1);
 	}
@@ -589,9 +644,10 @@ init_winnt_time(void)
 	 */
 
 	/* get the current process token handle */
-	if (!OpenProcessToken(GetCurrentProcess(),
-	     TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) 
-	{
+	if (!OpenProcessToken(
+		GetCurrentProcess(),
+		TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY,
+		&hToken)) {
 		msyslog(LOG_ERR, "OpenProcessToken failed: %m");
 		exit(-1);
 	}
@@ -606,8 +662,7 @@ init_winnt_time(void)
 
 	/* cannot use return value of AdjustTokenPrivileges. */
 	/* (success does not indicate all privileges were set) */
-	if (GetLastError() != ERROR_SUCCESS) 
-	{
+	if (GetLastError() != ERROR_SUCCESS) {
 		msyslog(LOG_ERR, "AdjustTokenPrivileges failed: %m");
 	 	/* later set time call will probably fail */
 	}
@@ -615,6 +670,11 @@ init_winnt_time(void)
 	CloseHandle(hToken);
 	hToken = INVALID_HANDLE_VALUE;
 
+	/*
+	 * Say how we're setting the time of day
+	 */
+	set_sys_var(settod, sizeof(settod), RO);
+
 	/*
 	 * ntpd on Windows has always raised its priority, without
 	 * requiring -N as on Unix.  Since Windows ntpd doesn't share
@@ -629,51 +689,36 @@ init_winnt_time(void)
 	if (-1 == setpriority(PRIO_PROCESS, 0, NTP_PRIO))
 		exit(-1);
 
-	/*
-	 * register with libntp ntp_set_tod() to call us back
-	 * when time is stepped.
-	 */
-	step_callback = time_stepped;
-
 	/* 
 	 * before we start looking at clock period, do any multimedia
 	 * timer manipulation requested via -M option.
 	 */
 	if (modify_mm_timer) {
-
 		if (timeGetDevCaps(&tc, sizeof(tc)) == TIMERR_NOERROR) {
-
 			wTimerRes = min(max(tc.wPeriodMin, MM_TIMER_INTV), tc.wPeriodMax);
 			timeBeginPeriod(wTimerRes);
 			atexit(atexit_revert_mm_timer);
 			
 			msyslog(LOG_INFO, "MM timer resolution: %u..%u msec, set to %u msec",
 				tc.wPeriodMin, tc.wPeriodMax, wTimerRes );
-		} else
+		} else {
 			msyslog(LOG_ERR, "Multimedia timer unavailable");
+		}
 	}
 	
 	/* get the performance counter ticks per second */
-	if (!QueryPerformanceFrequency(&Freq) || !Freq.QuadPart)
-	{
+	if (!QueryPerformanceFrequency(&Freq) || !Freq.QuadPart) {
 		msyslog(LOG_ERR, "QueryPerformanceFrequency failed: %m\n");
 		exit(-1);
 	}
 
 	NomPerfCtrFreq = PerfCtrFreq = Freq.QuadPart;
-	/*
-	 * the cast to LONGLONG is for VC6 compatibility:
-	 * nt_clockstuff.c(586) : error C2520: conversion from
-	 * unsigned __int64 to double not implemented, use signed 
-	 * __int64
-	 */
 	msyslog(LOG_INFO, 
-		"Performance counter frequency %.3f MHz", 
-		(LONGLONG)PerfCtrFreq / 1e6);
+		"Performance counter frequency %.3f MHz",
+		PerfCtrFreq / 1e6);
 
 	/* Determine the existing system time slewing */
-	if (!GetSystemTimeAdjustment(&adjclockperiod, &clockperiod, &noslew))
-	{
+	if (!GetSystemTimeAdjustment(&adjclockperiod, &clockperiod, &noslew)) {
 		msyslog(LOG_ERR, "GetSystemTimeAdjustment failed: %m\n");
 		exit(-1);
 	}
@@ -717,12 +762,27 @@ init_winnt_time(void)
 
 	msyslog(LOG_INFO,
 		"Windows clock precision %.3f msec, min. slew %.3f ppm/s",
-		(LONGLONG)os_clock_precision / 1e4, 
-		ppm_per_adjust_unit);
+		os_clock_precision / 1e4, ppm_per_adjust_unit);
+
+	pch = getenv("NTPD_TICKADJ_PPM");
+	if (pch != NULL && 1 == sscanf(pch, "%lf", &adjppm)) {
+		rawadj = adjppm / ppm_per_adjust_unit;
+		rawadj += (rawadj < 0)
+			      ? -0.5
+			      : 0.5;
+		wintickadj = (long)rawadj;
+		msyslog(LOG_INFO,
+			"Using NTPD_TICKADJ_PPM %+g ppm (%+ld)",
+			adjppm, wintickadj);
+	}
 
 	winnt_time_initialized = TRUE;
 
-	if (os_clock_precision < 4 * 10000 && !getenv("NTPD_USE_INTERP_DANGEROUS")) {
+	choose_interp_counter();
+
+	if (getenv("NTPD_USE_SYSTEM_CLOCK") ||
+	    (os_clock_precision < 4 * 10000 &&
+	     !getenv("NTPD_USE_INTERP_DANGEROUS"))) {
 		msyslog(LOG_INFO, "using Windows clock directly");
 	} else {
 		winnt_use_interpolation = TRUE;
@@ -757,7 +817,7 @@ reset_winnt_time(void)
 	 * Verify this will not call SetSystemTimeAdjustment if
 	 * ntpd is running in ntpdate mode.
 	 */
-	if (sys_leap == LEAP_NOTINSYNC || ls_time_adjustment)
+	if (sys_leap == LEAP_NOTINSYNC || ls_time_adjustment != 0)
 		SetSystemTimeAdjustment(0, TRUE);	 
 
 	/*
@@ -784,26 +844,31 @@ GetInterpTimeAsFileTime(
 	LPFILETIME pft
 	)
 {
+	static ULONGLONG last_interp_time;
 	FT_ULL now_time;
 	FT_ULL now_count;
+	ULONGLONG clock_backward;
 
-	/*  Mark a mark ASAP. The latency to here should
-	 *  be reasonably deterministic
-	*/
+	/*
+	 * Mark a mark ASAP.  The latency to here should be reasonably
+	 * deterministic
+	 */
 
 	now_count.ull = perf_ctr();
 	now_time.ull = interp_time(now_count.ull, TRUE);
 
-	if (last_interp_time > now_time.ull) {
-
-		clock_backward_count++;
-		if (last_interp_time - now_time.ull > clock_backward_max)
-			clock_backward_max = last_interp_time - now_time.ull;
-		now_time.ull = last_interp_time;
-	} else
+	if (last_interp_time <= now_time.ull) {
 		last_interp_time = now_time.ull;
-
+	} else {
+		clock_backward = last_interp_time - now_time.ull;
+		if (clock_backward > clock_backward_max) {
+			clock_backward_max = clock_backward;
+			clock_backward_count++;
+		}
+		now_time.ull = last_interp_time;
+	}
 	*pft = now_time.ft;
+
 	return;
 }
 
@@ -843,7 +908,6 @@ TimerApcFunction(
 	 */
 	if (INVALID_HANDLE_VALUE == ctr_freq_timer &&
 	    LEAP_NOTINSYNC != sys_leap)
-
 		start_ctr_freq_timer(now_time);
 }
 
@@ -873,7 +937,7 @@ ClockThread(
 		timer_period_msec = 43;
 	} else if (HZ > 98 && HZ < 102) {
 		timer_period_msec = 27;
-		if (!ntpd_int_int_text)
+		if (NULL == ntpd_int_int_text)
 			msyslog(LOG_WARNING, 
 				"%.3f Hz system clock may benefit from "
 				"custom NTPD_INT_INT env var timer interval "
@@ -881,7 +945,7 @@ ClockThread(
 				HZ);
 	} else {
 		timer_period_msec = (DWORD)(0.5 + (2.752 * clockperiod / 10000));
-		if (!ntpd_int_int_text)
+		if (NULL == ntpd_int_int_text)
 			msyslog(LOG_WARNING, 
 				"unfamiliar %.3f Hz system clock may benefit "
 				"from custom NTPD_INT_INT env var timer "
@@ -890,7 +954,7 @@ ClockThread(
 				HZ);
 	}
 
-	if (ntpd_int_int_text) {
+	if (ntpd_int_int_text != NULL) {
 		timer_period_msec = atoi(ntpd_int_int_text);
 		timer_period_msec = max(9, timer_period_msec);
 		msyslog(LOG_NOTICE, 
@@ -1028,21 +1092,21 @@ lock_thread_to_processor(HANDLE thread)
 		exit(-1);
 	}
 
-	if ( ! winnt_use_interpolation)
+	if (!winnt_use_interpolation)
 		return;
 	
-	if (-1 == use_pcc) {
-		DPRINTF(1, ("perf_ctr is not called before affinity, "
-			    "change lock_thread_to_processor to call it\n"));
+	if (-1 == lock_interp_threads) {
+		DPRINTF(1, ("choose_interp_counter() is not called "
+			    "before lock_thread_to_processor()\n"));
 		exit(-1);
-	} else if (0 == use_pcc)
+	} else if (!lock_interp_threads)
 		return;
 
 	/*
 	 * Calculate the ThreadAffinityMask we'll use once on the
 	 * first invocation.
 	 */
-	if ( ! ProcessAffinityMask) {
+	if (!ProcessAffinityMask) {
 
 		/*
 		 * Choose which processor to nail the main and clock threads to.
@@ -1100,7 +1164,6 @@ lock_thread_to_processor(HANDLE thread)
 
 	if (ThreadAffinityMask && 
 	    !SetThreadAffinityMask(thread, ThreadAffinityMask))
-
 		msyslog(LOG_ERR, 
 			"Unable to wire thread to mask %x: %m\n", 
 			ThreadAffinityMask);
@@ -1142,20 +1205,29 @@ ntp_timestamp_from_counter(
 	ULONGLONG	InterpTimestamp;
 
 	if (winnt_use_interpolation) {
+		if (0 == Counterstamp) {
+			DPRINTF(1, ("ntp_timestamp_from_counter rejecting 0 counter.\n"));
+			ZERO(*result);
+			return;
+		}
+
 		InterpTimestamp = interp_time(Counterstamp + QPC_offset, FALSE);
 
 #ifdef DEBUG
 		/* sanity check timestamp is within 1 minute of now */
 		GetSystemTimeAsFileTime(&Now.ft);
 		Now.ll -= InterpTimestamp;
-		if (Now.ll > 60 * HECTONANOSECONDS || 
-		    Now.ll < -60 * (LONGLONG) HECTONANOSECONDS) {
-			DPRINTF(1, ("ntp_timestamp_from_counter interpolated "
-				    "time %.6fs from current\n",
-					Now.ll / (double)LL_HNS));
-			DPRINTF(1, ("interpol time %llx from  %llx\n",
-					InterpTimestamp,
-					Counterstamp));
+		if (debug &&
+		    Now.ll > 60 * HECTONANOSECONDS || 
+		    Now.ll < -60 * HECTONANOSECONDS) {
+			DPRINTF(1, ("ntp_timestamp_from_counter interpolated time %.6fs from current\n",
+					Now.ll / (double)HECTONANOSECONDS));
+			DPRINTF(1, ("interpol time %llx from  %llx + %llx\n",
+					InterpTimestamp, Counterstamp,
+					QPC_offset));
+			msyslog(LOG_ERR,
+				"ntp_timestamp_from_counter interpolated time %.6fs from current\n",
+				Now.ll / (double)HECTONANOSECONDS);
 			exit(-1);
 		}
 #endif
@@ -1166,11 +1238,14 @@ ntp_timestamp_from_counter(
 		/* sanity check timestamp is within 1 minute of now */
 		GetSystemTimeAsFileTime(&Now.ft);
 		Now.ll -= InterpTimestamp;
-		if (Now.ll > 60 * HECTONANOSECONDS || 
-		    Now.ll < -60 * (LONGLONG) HECTONANOSECONDS) {
-			DPRINTF(1, ("ntp_timestamp_from_counter serial driver system "
-				    "time %.6fs from current\n",
-				    Now.ll / (double)LL_HNS));
+		if (debug &&
+		    Now.ll > 60 * HECTONANOSECONDS || 
+		    Now.ll < -60 * HECTONANOSECONDS) {
+			DPRINTF(1, ("ntp_timestamp_from_counter serial driver system time %.6fs from current\n",
+				    Now.ll / (double)HECTONANOSECONDS));
+			msyslog(LOG_ERR,
+				"ntp_timestamp_from_counter serial driver system time %.6fs from current\n",
+				Now.ll / (double)HECTONANOSECONDS);
 			exit(-1);
 		}
 #endif
@@ -1181,13 +1256,13 @@ ntp_timestamp_from_counter(
 	InterpTimestamp -= FILETIME_1970;
 	result->l_ui = JAN_1970 + (u_int32)(InterpTimestamp / HECTONANOSECONDS);
 	result->l_uf = (u_int32)((InterpTimestamp % HECTONANOSECONDS) *
-				 (FRAC / LL_HNS));
+				 (ULONGLONG)FRAC / HECTONANOSECONDS);
 }
 #endif  /* HAVE_PPSAPI */
 
 
-void 
-time_stepped(void)
+void
+win_time_stepped(void)
 {
 	/*
 	 * called back by ntp_set_tod after the system
@@ -1213,13 +1288,11 @@ time_stepped(void)
 	 */
 	newest_baseline_gen++;
 
-	last_interp_time = 
-		clock_backward_max = 
-		clock_backward_count = 
-		newest_baseline = 0;
-
-	memset(baseline_counts, 0, sizeof(baseline_counts));
-	memset(baseline_times, 0, sizeof(baseline_times));
+	clock_backward_max = CLOCK_BACK_THRESHOLD;
+	clock_backward_count = 0;
+	newest_baseline = 0;
+	ZERO(baseline_counts);
+	ZERO(baseline_times);
 
 	StartClockThread();
 }
@@ -1304,12 +1377,10 @@ ctr_freq_timer_fired(
 	now_time.ft.dwHighDateTime = dwTimeHigh;
 
 	if (now_time.ull >= next_period_time) {
-
 		now_count.ull = perf_ctr();
 		tune_ctr_freq(
 			now_count.ull - begin_count.ull,
 			now_time.ull - begin_time.ull);
-
 		next_period_time += (ULONGLONG)tune_ctr_period * HECTONANOSECONDS;
 		begin_count.ull = now_count.ull;
 		begin_time.ull = now_time.ull;
@@ -1318,25 +1389,21 @@ ctr_freq_timer_fired(
 	/* 
 	 * Log clock backward events no more often than 5 minutes.
 	 */
-	if (!report_systemtime) 
-
+	if (!report_systemtime) {
 		report_systemtime = now_time.ull + five_minutes;
-
-	else if (report_systemtime <= now_time.ull) {
-
+	} else if (report_systemtime <= now_time.ull) {
 		report_systemtime +=  five_minutes;
-
 		if (clock_backward_count) {
 			msyslog(LOG_WARNING, 
 				"clock would have gone backward %d times, "
 				"max %.1f usec",
 				clock_backward_count, 
-				(LONGLONG)clock_backward_max / 10.);
+				clock_backward_max / 10.);
 
-			clock_backward_max = clock_backward_count = 0;
+			clock_backward_max = CLOCK_BACK_THRESHOLD;
+			clock_backward_count = 0;
 		}
 	}
-
 	reset_ctr_freq_timer(next_period_time, now_time.ull);
 }
 
@@ -1349,7 +1416,6 @@ reset_ctr_freq_timer_abs(
 	FT_ULL	fire_time;
 
 	fire_time.ull = when; 
-
 	SetWaitableTimer(
 		ctr_freq_timer,
 		&fire_time.li,		/* first fire */
@@ -1368,7 +1434,6 @@ reset_ctr_freq_timer(
 {
 	if (when - now > 
 	    (tune_ctr_freq_max_interval * HECTONANOSECONDS + HECTONANOSECONDS))
-
 		when = now + tune_ctr_freq_max_interval * HECTONANOSECONDS;
 
 	reset_ctr_freq_timer_abs(when);
@@ -1383,7 +1448,6 @@ start_ctr_freq_timer(
 	ULONGLONG when;
 
 	ctr_freq_timer = CreateWaitableTimer(NULL, FALSE, NULL);
-
 	when = now_time;
 	ROUND_TO_NEXT_SEC_BOTTOM(when);
 
@@ -1409,18 +1473,18 @@ tune_ctr_freq(
 	static double nom_freq = 0;
 	static LONGLONG diffs[TUNE_CTR_DEPTH] = {0};
 	static LONGLONG sum = 0;
-
+	char ctr_freq_eq[64];
 	LONGLONG delta;
 	LONGLONG deltadiff;
 	ULONGLONG ObsPerfCtrFreq;
-	double obs_freq;
+	double freq;
 	double this_freq;
-	int isneg;
+	BOOL isneg;
 
 	/* one-time initialization */
 	if (!report_at_count) {
 		report_at_count = 24 * 60 * 60 / tune_ctr_period;
-		nom_freq = (LONGLONG)NomPerfCtrFreq / 1e6;
+		nom_freq = NomPerfCtrFreq / 1e6;
 	}
 
 	/* delta is the per-second observed frequency this time */
@@ -1430,10 +1494,11 @@ tune_ctr_freq(
 	/* disbelieve any delta more than +/- 976 PPM from nominal */
 	deltadiff = delta - NomPerfCtrFreq;
 	if (0 > deltadiff) {
-		isneg = 1;
+		isneg = TRUE;
 		deltadiff = -deltadiff;
-	} else
-		isneg = 0;
+	} else {
+		isneg = FALSE;
+	}
 
 	if ((ULONGLONG)deltadiff > (NomPerfCtrFreq / 1024)) {
 		disbelieved++;
@@ -1469,12 +1534,21 @@ tune_ctr_freq(
 	/* get rid of ObsPerfCtrFreq when removing the #ifdef */
 	PerfCtrFreq = ObsPerfCtrFreq;
 #endif
-	obs_freq = (LONGLONG)ObsPerfCtrFreq / 1e6;
+	freq = PerfCtrFreq / 1e6;
+
+	/*
+	 * make the performance counter's frequency error from its
+	 * nominal rate, expressed in PPM, available via ntpq as
+	 * system variable "ctr_frequency".  This is consistent with
+	 * "frequency" which is the system clock drift in PPM.
+	 */
+	snprintf(ctr_freq_eq, sizeof(ctr_freq_eq), "ctr_frequency=%.2f", 
+		 1e6 * (freq - nom_freq) / nom_freq);
+	set_sys_var(ctr_freq_eq, strlen(ctr_freq_eq) + 1, RO | DEF);
 
 	/* 
 	 * report observed ctr freq each time the estimate used during
-	 * startup moves toward the observed freq from the nominal, 
-	 * and once a day afterward.
+	 * startup moves toward the observed freq from the nominal.
 	 */
 
 	if (count > COUNTOF(diffs) &&
@@ -1486,23 +1560,21 @@ tune_ctr_freq(
 		if (count <= COUNTOF(diffs))
 			/* moving to observed freq. from nominal (startup) */
 			msyslog(LOG_INFO,
-				(obs_freq > 100)
-				 ? "ctr %.3f MHz %+6.2f PPM using "
-				       "%.3f MHz %+6.2f PPM"
-				 : "ctr %.6f MHz %+6.2f PPM using "
-				       "%.6f MHz %+6.2f PPM",
+				(freq > 100)
+				   ? "ctr %.3f MHz %+6.2f PPM using %.3f MHz %+6.2f PPM"
+				   : "ctr %.6f MHz %+6.2f PPM using %.6f MHz %+6.2f PPM",
 				this_freq,
 				1e6 * (this_freq - nom_freq) / nom_freq,
-				obs_freq, 
-				1e6 * (obs_freq - nom_freq) / nom_freq);
+				freq, 
+				1e6 * (freq - nom_freq) / nom_freq);
 		else
 			/* steady state */
 			msyslog(LOG_INFO,
-				(obs_freq > 100)
-				 ? "ctr %.3f MHz %+.2f PPM"
-				 : "ctr %.6f MHz %+.2f PPM",
-				obs_freq, 
-				1e6 * (obs_freq - nom_freq) / nom_freq);
+				(freq > 100)
+				   ? "ctr %.3f MHz %+.2f PPM"
+				   : "ctr %.6f MHz %+.2f PPM",
+				freq, 
+				1e6 * (freq - nom_freq) / nom_freq);
 
 	if (disbelieved) {
 		msyslog(LOG_ERR, 
