@@ -15,51 +15,55 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: ntservice.c,v 1.3.2.1.10.3 2004/03/08 04:04:22 marka Exp $ */
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
 #include <stdio.h>
 
-#include <ntp_cmdargs.h>
 #include <ntp_stdlib.h>
 #include "syslog.h"
+#include "ntpd.h"
 #include "ntservice.h"
 #include "clockstuff.h"
 #include "ntp_iocompletionport.h"
+#include "ntpd-opts.h"
 #include "isc/win32os.h"
+#include <ssl_applink.c>
 
 
-/* Handle to SCM for updating service status */
+/*
+ * Globals
+ */
 static SERVICE_STATUS_HANDLE hServiceStatus = 0;
 static BOOL foreground = FALSE;
 static BOOL computer_shutting_down = FALSE;
 static int glb_argc;
 static char **glb_argv;
 HANDLE hServDoneEvent = NULL;
-int accept_wildcard_if_for_winnt;
-extern volatile int debug;
+extern int accept_wildcard_if_for_winnt;
 
-void uninit_io_completion_port();
-int ntpdmain(int argc, char *argv[]);
-/*
- * libisc\nt_strerror.c
- */
-#ifdef DEBUG
-void FormatErrorFreeMem(void);
-#endif
 /*
  * Forward declarations
  */
+void uninit_io_completion_port();
+int ntpdmain(int argc, char *argv[]);
 void WINAPI ServiceControl(DWORD dwCtrlCode);
 void ntservice_exit(void);
 
-void WINAPI service_main( DWORD argc, LPTSTR *argv )
+#ifdef WRAP_DBG_MALLOC
+void *wrap_dbg_malloc(size_t s, const char *f, int l);
+void *wrap_dbg_realloc(void *p, size_t s, const char *f, int l);
+void wrap_dbg_free(void *p);
+#endif
+
+void WINAPI
+service_main(
+	DWORD argc,
+	LPTSTR *argv
+	)
 {
-	if ( argc > 1 )
-	{
+	if (argc > 1) {
 		/*
 		 * Let command line parameters from the Windows SCM GUI
 		 * override the standard parameters from the ImagePath registry key.
@@ -68,7 +72,7 @@ void WINAPI service_main( DWORD argc, LPTSTR *argv )
 		glb_argv = argv;
 	}
 
-	ntpdmain( glb_argc, glb_argv );
+	ntpdmain(glb_argc, glb_argv);
 }
 
 
@@ -77,10 +81,16 @@ void WINAPI service_main( DWORD argc, LPTSTR *argv )
  * We can call ntpdmain() explicitly or via StartServiceCtrlDispatcher()
  * as we need to.
  */
-int main( int argc, char *argv[] )
+int main(
+	int	argc,
+	char **	argv
+	)
 {
-	int rc;
-	int i = 1;
+	int	rc;
+	int	argc_after_opts;
+	char **	argv_after_opts;
+
+	ssl_applink();
 
 	/* Save the command line parameters */
 	glb_argc = argc;
@@ -88,25 +98,25 @@ int main( int argc, char *argv[] )
 
 	/* Under original Windows NT we must not discard the wildcard */
 	/* socket to workaround a bug in NT's getsockname(). */
-	if ( isc_win32os_majorversion() <= 4 )
-		accept_wildcard_if_for_winnt = 1;
+	if (isc_win32os_majorversion() <= 4)
+		accept_wildcard_if_for_winnt = TRUE;
 
-	/* Command line users should put -n in the options */
-	while (argv[i]) {
-		if (!_strnicmp(argv[i], "-d", 2) ||
-			!strcmp(argv[i], "-q") ||
-			!strcmp(argv[i], "--help") ||
-			!strcmp(argv[i], "-n")) {
-			foreground = TRUE;
-			break;
-		}
-		i++;
-	}
+	argc_after_opts = argc;
+	argv_after_opts = argv;
+	parse_cmdline_opts(&argc_after_opts, &argv_after_opts);
 
-	if (foreground) {
-		/* run in console window */
+	if (HAVE_OPT(QUIT)
+	    || HAVE_OPT(SAVECONFIGQUIT)
+	    || HAVE_OPT(HELP)
+#ifdef DEBUG
+	    || DESC(DEBUG_LEVEL).optOccCt != 0
+#endif
+	    || HAVE_OPT(NOFORK))
+		foreground = TRUE;
+
+	if (foreground)			/* run in console window */
 		rc = ntpdmain(argc, argv);
-	} else {
+	else {
 		/* Start up as service */
 
 		SERVICE_TABLE_ENTRY dispatchTable[] = {
@@ -119,15 +129,19 @@ int main( int argc, char *argv[] )
 			rc = 0; 
 		else {
 			rc = GetLastError();
-#ifdef DEBUG
-			fprintf(stderr, "%s: unable to start as service, rc: %i\n\n", argv[0], rc);
-#endif
-			fprintf(stderr, "\nUse -d, -q, --help or -n to run from the command line.\n");
+			fprintf(stderr,
+				"%s: unable to start as service:\n"
+				"%s\n"
+				"Use -d, -q, -n, -?, --help or "
+				"--saveconfigquit to run "
+				"interactive.\n",
+				argv[0], ntp_strerror(rc));
 		}
 	}
 	return rc;
 }
 
+
 /*
  * Initialize the Service by registering it.
  */
@@ -146,8 +160,9 @@ ntservice_init() {
 		}
 		UpdateSCM(SERVICE_RUNNING);
 	} else {
-		strcpy(ConsoleTitle, "NTP Version ");
-		strcat(ConsoleTitle, Version);
+		snprintf(ConsoleTitle, sizeof(ConsoleTitle),
+			 "NTP Version %s", Version);
+		ConsoleTitle[sizeof(ConsoleTitle) - 1] = '\0';
 		SetConsoleTitle(ConsoleTitle);
 	}
 
@@ -176,13 +191,6 @@ ntservice_init() {
 	atexit( ntservice_exit );
 }
 
-/*
- * Routine to check if this is a service or a foreground program
- */
-BOOL
-ntservice_isservice() {
-	return(!foreground);
-}
 
 /*
  * Routine to check if the service is stopping
@@ -203,10 +211,6 @@ ntservice_exit( void )
 
 	msyslog(LOG_INFO, "ntservice: The Network Time Protocol Service is stopping.");
 
-#ifdef DEBUG
-	FormatErrorFreeMem();
-#endif 
-
 	if (!foreground) {
 		/* service mode, need to have the service_main routine
 		 * register with the service control manager that the 
@@ -221,21 +225,21 @@ ntservice_exit( void )
  * to the service.
  */
 void WINAPI
-ServiceControl(DWORD dwCtrlCode) {
-	/* Handle the requested control code */
-	HANDLE exitEvent = get_exit_event();
-
-	switch(dwCtrlCode) {
+ServiceControl(
+	DWORD dwCtrlCode
+	) 
+{
+	switch (dwCtrlCode) {
 
 	case SERVICE_CONTROL_SHUTDOWN:
 		computer_shutting_down = TRUE;
 		/* fall through to stop case */
 
 	case SERVICE_CONTROL_STOP:
-		if (exitEvent != NULL) {
-			SetEvent(exitEvent);
+		if (WaitableExitEventHandle != NULL) {
+			SetEvent(WaitableExitEventHandle);
 			UpdateSCM(SERVICE_STOP_PENDING);
-			Sleep( 100 );  //##++
+			Sleep(100);  //##++
 		}
 		return;
 
@@ -259,7 +263,7 @@ void UpdateSCM(DWORD state) {
 		if (state)
 			dwState = state;
 
-		memset(&ss, 0, sizeof(SERVICE_STATUS));
+		ZERO(ss);
 		ss.dwServiceType |= SERVICE_WIN32_OWN_PROCESS;
 		ss.dwCurrentState = dwState;
 		ss.dwControlsAccepted = SERVICE_ACCEPT_STOP |
@@ -278,8 +282,6 @@ OnConsoleEvent(
 	DWORD dwCtrlType
 	)
 {
-	HANDLE exitEvent = get_exit_event();
-
 	switch (dwCtrlType) {
 #ifdef DEBUG
 		case CTRL_BREAK_EVENT:
@@ -302,9 +304,9 @@ OnConsoleEvent(
 		case CTRL_C_EVENT:
 		case CTRL_CLOSE_EVENT:
 		case CTRL_SHUTDOWN_EVENT:
-			if (exitEvent != NULL) {
-				SetEvent(exitEvent);
-				Sleep( 100 );  //##++
+			if (WaitableExitEventHandle != NULL) {
+				SetEvent(WaitableExitEventHandle);
+				Sleep(100);  //##++
 			}
 			break;
 
