@@ -1,7 +1,7 @@
 
-/*
- *  $Id: load.c,v 4.29 2009/08/01 17:44:36 bkorb Exp $
- *  Time-stamp:      "2008-12-06 10:16:05 bkorb"
+/**
+ *  \file load.c
+ *  Time-stamp:      "2012-08-11 08:20:09 bkorb"
  *
  *  This file contains the routines that deal with processing text strings
  *  for options, either from a NUL-terminated string passed in or from an
@@ -9,7 +9,7 @@
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2012 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -28,26 +28,16 @@
  *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
-tOptionLoadMode option_load_mode = OPTION_LOAD_UNCOOKED;
-
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
-static ag_bool
-insertProgramPath(
-    char*   pzBuf,
-    int     bufSize,
-    tCC*    pzName,
-    tCC*    pzProgPath );
+static bool
+add_prog_path(char * pzBuf, int bufSize, char const * pzName,
+              char const * pzProgPath);
 
-static ag_bool
-insertEnvVal(
-    char*   pzBuf,
-    int     bufSize,
-    tCC*    pzName,
-    tCC*    pzProgPath );
+static bool
+add_env_val(char * buf, int buf_sz, char const * name);
 
-static char*
-assembleArgValue( char* pzTxt, tOptionLoadMode mode );
+static char *
+assemble_arg_val(char * txt, tOptionLoadMode mode);
 /* = = = END-STATIC-FORWARD = = = */
 
 /*=export_func  optionMakePath
@@ -59,8 +49,8 @@ assembleArgValue( char* pzTxt, tOptionLoadMode mode );
  * arg:   + char const* + pzName     + The input name +
  * arg:   + char const* + pzProgPath + The full path of the current program +
  *
- * ret-type: ag_bool
- * ret-desc: AG_TRUE if the name was handled, otherwise AG_FALSE.
+ * ret-type: bool
+ * ret-desc: true if the name was handled, otherwise false.
  *           If the name does not start with ``$'', then it is handled
  *           simply by copying the input name to the output buffer and
  *           resolving the name with either
@@ -68,8 +58,8 @@ assembleArgValue( char* pzTxt, tOptionLoadMode mode );
  *
  * doc:
  *
- *  This routine will copy the @code{pzName} input name into the @code{pzBuf}
- *  output buffer, carefully not exceeding @code{bufSize} bytes.  If the
+ *  This routine will copy the @code{pzName} input name into the
+ *  @code{pzBuf} output buffer, not exceeding @code{bufSize} bytes.  If the
  *  first character of the input name is a @code{'$'} character, then there
  *  is special handling:
  *  @*
@@ -86,7 +76,7 @@ assembleArgValue( char* pzTxt, tOptionLoadMode mode );
  *     @code{pzName} string and must either be the entire string or be followed
  *     by the @code{'/'} (backslash on windows) character.
  *
- * err:  @code{AG_FALSE} is returned if:
+ * err:  @code{false} is returned if:
  *       @*
  *       @bullet{} The input name exceeds @code{bufSize} bytes.
  *       @*
@@ -101,31 +91,20 @@ assembleArgValue( char* pzTxt, tOptionLoadMode mode );
  *       @bullet{} @code{canonicalize_file_name} or @code{realpath} return
  *                 errors (cannot resolve the resulting path).
 =*/
-ag_bool
-optionMakePath(
-    char*   pzBuf,
-    int     bufSize,
-    tCC*    pzName,
-    tCC*    pzProgPath )
+bool
+optionMakePath(char * pzBuf, int bufSize, char const * pzName,
+               char const * pzProgPath)
 {
-    size_t  name_len = strlen( pzName );
+    size_t name_len = strlen(pzName);
 
-#   ifndef PKGDATADIR
-#     define PKGDATADIR ""
-#   endif
-
-    tSCC    pkgdatadir[] = PKGDATADIR;
-
-    ag_bool res = AG_TRUE;
-
-    if (bufSize <= name_len)
-        return AG_FALSE;
+    if (((size_t)bufSize <= name_len) || (name_len == 0))
+        return false;
 
     /*
      *  IF not an environment variable, just copy the data
      */
     if (*pzName != '$') {
-        tCC*  pzS = pzName;
+        char const*  pzS = pzName;
         char* pzD = pzBuf;
         int   ct  = bufSize;
 
@@ -133,7 +112,7 @@ optionMakePath(
             if ( (*(pzD++) = *(pzS++)) == NUL)
                 break;
             if (--ct <= 0)
-                return AG_FALSE;
+                return false;
         }
     }
 
@@ -144,65 +123,67 @@ optionMakePath(
      */
     else switch (pzName[1]) {
     case NUL:
-        return AG_FALSE;
+        return false;
 
     case '$':
-        res = insertProgramPath( pzBuf, bufSize, pzName, pzProgPath );
+        if (! add_prog_path(pzBuf, bufSize, pzName, pzProgPath))
+            return false;
         break;
 
     case '@':
-        if (pkgdatadir[0] == NUL)
-            return AG_FALSE;
+        if (program_pkgdatadir[0] == NUL)
+            return false;
 
-        if (name_len + sizeof (pkgdatadir) > bufSize)
-            return AG_FALSE;
-
-        strcpy(pzBuf, pkgdatadir);
-        strcpy(pzBuf + sizeof(pkgdatadir) - 1, pzName + 2);
+        if (snprintf(pzBuf, bufSize, "%s%s", program_pkgdatadir, pzName + 2)
+            >= bufSize)
+            return false;
         break;
 
     default:
-        res = insertEnvVal( pzBuf, bufSize, pzName, pzProgPath );
+        if (! add_env_val(pzBuf, bufSize, pzName))
+            return false;
     }
 
-    if (! res)
-        return AG_FALSE;
-
 #if defined(HAVE_CANONICALIZE_FILE_NAME)
     {
-        char* pz = canonicalize_file_name(pzBuf);
+        char * pz = canonicalize_file_name(pzBuf);
         if (pz == NULL)
-            return AG_FALSE;
-        if (strlen(pz) < bufSize)
-            strcpy(pzBuf, pz);
+            return false;
+
+        name_len = strlen(pz);
+        if (name_len >= (size_t)bufSize) {
+            free(pz);
+            return false;
+        }
+
+        memcpy(pzBuf, pz, name_len + 1);
         free(pz);
     }
 
 #elif defined(HAVE_REALPATH)
     {
-        char z[ PATH_MAX+1 ];
+        char z[PATH_MAX+1];
 
-        if (realpath( pzBuf, z ) == NULL)
-            return AG_FALSE;
+        if (realpath(pzBuf, z) == NULL)
+            return false;
 
-        if (strlen(z) < bufSize)
-            strcpy( pzBuf, z );
+        name_len = strlen(z);
+        if (name_len >= bufSize)
+            return false;
+
+        memcpy(pzBuf, z, name_len + 1);
     }
 #endif
 
-    return AG_TRUE;
+    return true;
 }
 
-
-static ag_bool
-insertProgramPath(
-    char*   pzBuf,
-    int     bufSize,
-    tCC*    pzName,
-    tCC*    pzProgPath )
+static bool
+add_prog_path(char * pzBuf, int bufSize, char const * pzName,
+              char const * pzProgPath)
 {
-    tCC*    pzPath;
-    tCC*    pz;
+    char const*    pzPath;
+    char const*    pz;
     int     skip = 2;
 
     switch (pzName[2]) {
@@ -211,7 +192,7 @@ insertProgramPath(
     case NUL:
         break;
     default:
-        return AG_FALSE;
+        return false;
     }
 
     /*
@@ -219,23 +200,23 @@ insertProgramPath(
      *  If it is, we're done.  Otherwise, we have to hunt
      *  for the program using "pathfind".
      */
-    if (strchr( pzProgPath, DIRCH ) != NULL)
+    if (strchr(pzProgPath, DIRCH) != NULL)
         pzPath = pzProgPath;
     else {
-        pzPath = pathfind( getenv( "PATH" ), (char*)pzProgPath, "rx" );
+        pzPath = pathfind(getenv("PATH"), (char*)pzProgPath, "rx");
 
         if (pzPath == NULL)
-            return AG_FALSE;
+            return false;
     }
 
-    pz = strrchr( pzPath, DIRCH );
+    pz = strrchr(pzPath, DIRCH);
 
     /*
      *  IF we cannot find a directory name separator,
      *  THEN we do not have a path name to our executable file.
      */
     if (pz == NULL)
-        return AG_FALSE;
+        return false;
 
     pzName += skip;
 
@@ -243,11 +224,11 @@ insertProgramPath(
      *  Concatenate the file name to the end of the executable path.
      *  The result may be either a file or a directory.
      */
-    if ((pz - pzPath)+1 + strlen(pzName) >= bufSize)
-        return AG_FALSE;
+    if ((pz - pzPath)+1 + strlen(pzName) >= (unsigned)bufSize)
+        return false;
 
-    memcpy( pzBuf, pzPath, (size_t)((pz - pzPath)+1) );
-    strcpy( pzBuf + (pz - pzPath) + 1, pzName );
+    memcpy(pzBuf, pzPath, (size_t)((pz - pzPath)+1));
+    strcpy(pzBuf + (pz - pzPath) + 1, pzName);
 
     /*
      *  If the "pzPath" path was gotten from "pathfind()", then it was
@@ -255,71 +236,65 @@ insertProgramPath(
      */
     if (pzPath != pzProgPath)
         AGFREE(pzPath);
-    return AG_TRUE;
+    return true;
 }
 
-
-static ag_bool
-insertEnvVal(
-    char*   pzBuf,
-    int     bufSize,
-    tCC*    pzName,
-    tCC*    pzProgPath )
+static bool
+add_env_val(char * buf, int buf_sz, char const * name)
 {
-    char* pzDir = pzBuf;
+    char * dir_part = buf;
 
     for (;;) {
-        int ch = (int)*++pzName;
+        int ch = (int)*++name;
         if (! IS_VALUE_NAME_CHAR(ch))
             break;
-        *(pzDir++) = (char)ch;
+        *(dir_part++) = (char)ch;
     }
 
-    if (pzDir == pzBuf)
-        return AG_FALSE;
+    if (dir_part == buf)
+        return false;
 
-    *pzDir = NUL;
+    *dir_part = NUL;
 
-    pzDir = getenv( pzBuf );
+    dir_part = getenv(buf);
 
     /*
      *  Environment value not found -- skip the home list entry
      */
-    if (pzDir == NULL)
-        return AG_FALSE;
+    if (dir_part == NULL)
+        return false;
 
-    if (strlen( pzDir ) + 1 + strlen( pzName ) >= bufSize)
-        return AG_FALSE;
+    if (strlen(dir_part) + 1 + strlen(name) >= (unsigned)buf_sz)
+        return false;
 
-    sprintf( pzBuf, "%s%s", pzDir, pzName );
-    return AG_TRUE;
+    sprintf(buf, "%s%s", dir_part, name);
+    return true;
 }
 
-
 LOCAL void
-mungeString( char* pzTxt, tOptionLoadMode mode )
+mungeString(char * txt, tOptionLoadMode mode)
 {
-    char* pzE;
+    char * pzE;
 
     if (mode == OPTION_LOAD_KEEP)
         return;
 
-    if (IS_WHITESPACE_CHAR(*pzTxt)) {
-        char* pzS = pzTxt;
-        char* pzD = pzTxt;
-        while (IS_WHITESPACE_CHAR(*++pzS))  ;
-        while ((*(pzD++) = *(pzS++)) != NUL)   ;
-        pzE = pzD-1;
-    } else
-        pzE = pzTxt + strlen( pzTxt );
+    if (IS_WHITESPACE_CHAR(*txt)) {
+        char * pzS = SPN_WHITESPACE_CHARS(txt+1);
+        size_t l   = strlen(pzS) + 1;
+        memmove(txt, pzS, l);
+        pzE = txt + l - 1;
 
-    while ((pzE > pzTxt) && IS_WHITESPACE_CHAR(pzE[-1]))  pzE--;
+    } else
+        pzE = txt + strlen(txt);
+
+    pzE  = SPN_WHITESPACE_BACK(txt, pzE);
     *pzE = NUL;
 
     if (mode == OPTION_LOAD_UNCOOKED)
         return;
 
-    switch (*pzTxt) {
+    switch (*txt) {
     default: return;
     case '"':
     case '\'': break;
@@ -331,22 +306,20 @@ mungeString( char* pzTxt, tOptionLoadMode mode )
     case '\'': break;
     }
 
-    (void)ao_string_cook( pzTxt, NULL );
+    (void)ao_string_cook(txt, NULL);
 }
 
-
-static char*
-assembleArgValue( char* pzTxt, tOptionLoadMode mode )
+static char *
+assemble_arg_val(char * txt, tOptionLoadMode mode)
 {
-    tSCC zBrk[] = " \t\n:=";
-    char* pzEnd = strpbrk( pzTxt, zBrk );
+    char* pzEnd = strpbrk(txt, ARG_BREAK_STR);
     int   space_break;
 
     /*
      *  Not having an argument to a configurable name is okay.
      */
     if (pzEnd == NULL)
-        return pzTxt + strlen(pzTxt);
+        return txt + strlen(txt);
 
     /*
      *  If we are keeping all whitespace, then the  modevalue starts with the
@@ -365,41 +338,49 @@ assembleArgValue( char* pzTxt, tOptionLoadMode mode )
      */
     space_break = IS_WHITESPACE_CHAR(*pzEnd);
     *(pzEnd++) = NUL;
-    while (IS_WHITESPACE_CHAR(*pzEnd))  pzEnd++;
+
+    pzEnd = SPN_WHITESPACE_CHARS(pzEnd);
     if (space_break && ((*pzEnd == ':') || (*pzEnd == '=')))
-        while (IS_WHITESPACE_CHAR(*++pzEnd))  ;
+        pzEnd = SPN_WHITESPACE_CHARS(pzEnd+1);
 
     return pzEnd;
 }
 
-
-/*
+/**
  *  Load an option from a block of text.  The text must start with the
  *  configurable/option name and be followed by its associated value.
  *  That value may be processed in any of several ways.  See "tOptionLoadMode"
  *  in autoopts.h.
+ *
+ * @param[in,out] opts       program options descriptor
+ * @param[in,out] opt_state  option processing state
+ * @param[in,out] line       source line with long option name in it
+ * @param[in]     direction  current processing direction (preset or not)
+ * @param[in]     load_mode  option loading mode (OPTION_LOAD_*)
  */
 LOCAL void
 loadOptionLine(
-    tOptions*   pOpts,
-    tOptState*  pOS,
-    char*       pzLine,
+    tOptions *  opts,
+    tOptState * opt_state,
+    char *      line,
     tDirection  direction,
     tOptionLoadMode   load_mode )
 {
-    while (IS_WHITESPACE_CHAR(*pzLine))  pzLine++;
+    line = SPN_LOAD_LINE_SKIP_CHARS(line);
 
     {
-        char* pzArg = assembleArgValue( pzLine, load_mode );
+        char * arg = assemble_arg_val(line, load_mode);
 
-        if (! SUCCESSFUL( longOptionFind( pOpts, pzLine, pOS )))
+        if (! SUCCESSFUL(opt_find_long(opts, line, opt_state)))
             return;
-        if (pOS->flags & OPTST_NO_INIT)
+
+        if (opt_state->flags & OPTST_NO_INIT)
             return;
-        pOS->pzOptArg = pzArg;
+
+        opt_state->pzOptArg = arg;
     }
 
-    switch (pOS->flags & (OPTST_IMM|OPTST_DISABLE_IMM)) {
+    switch (opt_state->flags & (OPTST_IMM|OPTST_DISABLE_IMM)) {
     case 0:
         /*
          *  The selected option has no immediate action.
@@ -417,7 +398,7 @@ loadOptionLine(
              *  immediately for enablement, but normally for disablement.
              *  Therefore, skip if disabled.
              */
-            if ((pOS->flags & OPTST_DISABLED) == 0)
+            if ((opt_state->flags & OPTST_DISABLED) == 0)
                 return;
         } else {
             /*
@@ -425,7 +406,7 @@ loadOptionLine(
              *  immediately for enablement, but normally for disablement.
              *  Therefore, skip if NOT disabled.
              */
-            if ((pOS->flags & OPTST_DISABLED) != 0)
+            if ((opt_state->flags & OPTST_DISABLED) != 0)
                 return;
         }
         break;
@@ -437,7 +418,7 @@ loadOptionLine(
              *  immediately for disablement, but normally for disablement.
              *  Therefore, skip if NOT disabled.
              */
-            if ((pOS->flags & OPTST_DISABLED) != 0)
+            if ((opt_state->flags & OPTST_DISABLED) != 0)
                 return;
         } else {
             /*
@@ -445,7 +426,7 @@ loadOptionLine(
              *  immediately for disablement, but normally for disablement.
              *  Therefore, skip if disabled.
              */
-            if ((pOS->flags & OPTST_DISABLED) == 0)
+            if ((opt_state->flags & OPTST_DISABLED) == 0)
                 return;
         }
         break;
@@ -464,43 +445,42 @@ loadOptionLine(
     /*
      *  Fix up the args.
      */
-    if (OPTST_GET_ARGTYPE(pOS->pOD->fOptState) == OPARG_TYPE_NONE) {
-        if (*pOS->pzOptArg != NUL)
+    if (OPTST_GET_ARGTYPE(opt_state->pOD->fOptState) == OPARG_TYPE_NONE) {
+        if (*opt_state->pzOptArg != NUL)
             return;
-        pOS->pzOptArg = NULL;
+        opt_state->pzOptArg = NULL;
 
-    } else if (pOS->pOD->fOptState & OPTST_ARG_OPTIONAL) {
-        if (*pOS->pzOptArg == NUL)
-             pOS->pzOptArg = NULL;
+    } else if (opt_state->pOD->fOptState & OPTST_ARG_OPTIONAL) {
+        if (*opt_state->pzOptArg == NUL)
+             opt_state->pzOptArg = NULL;
         else {
-            AGDUPSTR( pOS->pzOptArg, pOS->pzOptArg, "option argument" );
-            pOS->flags |= OPTST_ALLOC_ARG;
+            AGDUPSTR(opt_state->pzOptArg, opt_state->pzOptArg, "opt arg");
+            opt_state->flags |= OPTST_ALLOC_ARG;
         }
 
     } else {
-        if (*pOS->pzOptArg == NUL)
-             pOS->pzOptArg = zNil;
+        if (*opt_state->pzOptArg == NUL)
+             opt_state->pzOptArg = zNil;
         else {
-            AGDUPSTR( pOS->pzOptArg, pOS->pzOptArg, "option argument" );
-            pOS->flags |= OPTST_ALLOC_ARG;
+            AGDUPSTR(opt_state->pzOptArg, opt_state->pzOptArg, "opt arg");
+            opt_state->flags |= OPTST_ALLOC_ARG;
         }
     }
 
     {
         tOptionLoadMode sv = option_load_mode;
         option_load_mode = load_mode;
-        handleOption( pOpts, pOS );
+        handle_opt(opts, opt_state);
         option_load_mode = sv;
     }
 }
 
-
 /*=export_func  optionLoadLine
  *
  * what:  process a string for an option name and value
  *
- * arg:   tOptions*,   pOpts,  program options descriptor
- * arg:   char const*, pzLine, NUL-terminated text
+ * arg:   tOptions*,   opts,  program options descriptor
+ * arg:   char const*, line,  NUL-terminated text
  *
  * doc:
  *
@@ -511,7 +491,8 @@ loadOptionLine(
  *  When passed a pointer to the option struct and a string, it will find
  *  the option named by the first token on the string and set the option
  *  argument to the remainder of the string.  The caller must NUL terminate
- *  the string.  Any embedded new lines will be included in the option
+ *  the string.  The caller need not skip over any introductory hyphens.
+ *  Any embedded new lines will be included in the option
  *  argument.  If the input looks like one or more quoted strings, then the
  *  input will be "cooked".  The "cooking" is identical to the string
  *  formation used in AutoGen definition files (@pxref{basic expression}),
@@ -521,15 +502,13 @@ loadOptionLine(
  *        will cause a warning to print, but the function should return.
 =*/
 void
-optionLoadLine(
-    tOptions*  pOpts,
-    tCC*       pzLine )
+optionLoadLine(tOptions * opts, char const * line)
 {
     tOptState st = OPTSTATE_INITIALIZER(SET);
     char* pz;
-    AGDUPSTR( pz, pzLine, "user option line" );
-    loadOptionLine( pOpts, &st, pz, DIRECTION_PROCESS, OPTION_LOAD_COOKED );
-    AGFREE( pz );
+    AGDUPSTR(pz, line, "user option line");
+    loadOptionLine(opts, &st, pz, DIRECTION_PROCESS, OPTION_LOAD_COOKED);
+    AGFREE(pz);
 }
 /*
  * Local Variables:
