@@ -1,7 +1,8 @@
 
-/*
- *  $Id: pgusage.c,v 4.17 2009/01/01 16:49:26 bkorb Exp $
- * Time-stamp:      "2008-07-27 21:08:42 bkorb"
+/**
+ * \file pgusage.c
+ *
+ * Time-stamp:      "2010-07-17 10:42:53 bkorb"
  *
  *   Automated Options Paged Usage module.
  *
@@ -10,7 +11,7 @@
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2011 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -24,8 +25,8 @@
  *
  *  These files have the following md5sums:
  *
- *  239588c55c22c60ffe159946a760a33e pkg/libopts/COPYING.gplv3
- *  fa82ca978890795162346e661b47161a pkg/libopts/COPYING.lgplv3
+ *  43b91e8ca915626ed3818ffb1b71248b pkg/libopts/COPYING.gplv3
+ *  06a1a2e4760c90ea5e1dad8dfaac4d39 pkg/libopts/COPYING.lgplv3
  *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
@@ -41,15 +42,16 @@ tePagerState pagerState = PAGER_STATE_INITIAL;
  * doc:
  *  Run the usage output through a pager.
  *  This is very handy if it is very long.
+ *  This is disabled on platforms without a working fork() function.
 =*/
 void
-optionPagedUsage( tOptions* pOptions, tOptDesc* pOD )
+optionPagedUsage(tOptions* pOptions, tOptDesc* pOD)
 {
-#if defined(__windows__) && !defined(__CYGWIN__)
+#if ! defined(HAVE_WORKING_FORK)
     if ((pOD->fOptState & OPTST_RESET) != 0)
         return;
 
-    (*pOptions->pUsageProc)( pOptions, EXIT_SUCCESS );
+    (*pOptions->pUsageProc)(pOptions, EXIT_SUCCESS);
 #else
     static pid_t     my_pid;
     char zPageUsage[ 1024 ];
@@ -69,38 +71,38 @@ optionPagedUsage( tOptions* pOptions, tOptDesc* pOD )
 #ifdef HAVE_SNPRINTF
         snprintf(zPageUsage, sizeof(zPageUsage), "/tmp/use.%lu", (tAoUL)my_pid);
 #else
-        sprintf( zPageUsage, "/tmp/use.%lu", (tAoUL)my_pid );
+        sprintf(zPageUsage, "/tmp/use.%lu", (tAoUL)my_pid);
 #endif
-        unlink( zPageUsage );
+        unlink(zPageUsage);
 
         /*
          *  Set usage output to this temporary file
          */
-        option_usage_fp = fopen( zPageUsage, "w" FOPEN_BINARY_FLAG );
+        option_usage_fp = fopen(zPageUsage, "w" FOPEN_BINARY_FLAG);
         if (option_usage_fp == NULL)
-            _exit( EXIT_FAILURE );
+            _exit(EXIT_FAILURE);
 
         pagerState = PAGER_STATE_READY;
 
         /*
          *  Set up so this routine gets called during the exit logic
          */
-        atexit( (void(*)(void))optionPagedUsage );
+        atexit((void(*)(void))optionPagedUsage);
 
         /*
          *  The usage procedure will now put the usage information into
          *  the temporary file we created above.
          */
-        (*pOptions->pUsageProc)( pOptions, EXIT_SUCCESS );
+        (*pOptions->pUsageProc)(pOptions, EXIT_SUCCESS);
 
-        /*NOTREACHED*/
-        _exit( EXIT_FAILURE );
+        /* NOTREACHED */
+        _exit(EXIT_FAILURE);
     }
 
     case PAGER_STATE_READY:
     {
         tSCC zPage[]  = "%1$s /tmp/use.%2$lu ; rm -f /tmp/use.%2$lu";
-        tCC* pzPager  = (tCC*)getenv( "PAGER" );
+        tCC* pzPager  = (tCC*)getenv("PAGER");
 
         /*
          *  Use the "more(1)" program if "PAGER" has not been defined
@@ -114,12 +116,12 @@ optionPagedUsage( tOptions* pOptions, tOptDesc* pOD )
 #ifdef HAVE_SNPRINTF
         snprintf(zPageUsage, sizeof(zPageUsage), zPage, pzPager, (tAoUL)my_pid);
 #else
-        sprintf( zPageUsage, zPage, pzPager, (tAoUL)my_pid );
+        sprintf(zPageUsage, zPage, pzPager, (tAoUL)my_pid);
 #endif
-        fclose( stderr );
-        dup2( STDOUT_FILENO, STDERR_FILENO );
+        fclose(stderr);
+        dup2(STDOUT_FILENO, STDERR_FILENO);
 
-        (void)system( zPageUsage );
+        (void)system(zPageUsage);
     }
 
     case PAGER_STATE_CHILD:
