@@ -87,22 +87,15 @@
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/time.h>
-#if HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
-#include "ntp_types.h"
+#include <sys/types.h>
+
 #include "ntp_random.h"
-#include "l_stdlib.h"
+#include "ntp_stdlib.h"
 #include "ntp_assert.h"
-
+#include "ntp_libopts.h"
+#include "ntp_unixtime.h"
 #include "ntp-keygen-opts.h"
 
-#ifdef SYS_WINNT
-extern	int	ntp_getopt	(int, char **, const char *);
-#define getopt ntp_getopt
-#define optarg ntp_optarg
-#endif /* SYS_WINNT */
-
 #ifdef OPENSSL
 #include "openssl/bn.h"
 #include "openssl/evp.h"
@@ -111,17 +104,19 @@ extern	int	ntp_getopt	(int, char **, const char *);
 #include "openssl/pem.h"
 #include "openssl/x509v3.h"
 #include <openssl/objects.h>
-#endif /* OPENSSL */
+#endif	/* OPENSSL */
+#include <ssl_applink.c>
 
 /*
  * Cryptodefines
  */
-#define	MD5KEYS		16	/* number of MD5 keys generated */
-#define	JAN_1970	2208988800UL /* NTP seconds */
-#define YEAR		((long)60*60*24*365) /* one year in seconds */
+#define	MD5KEYS		10	/* number of keys generated of each type */
+#define	MD5SIZE		20	/* maximum key size */
+#define DAY		((long)60*60*24) /* one day in seconds */
+#define	YEAR		((long)365) /* one year in days */
 #define MAXFILENAME	256	/* max file name length */
 #define MAXHOSTNAME	256	/* max host name length */
-#ifdef OPENSSL
+#ifdef AUTOKEY
 #define	PLEN		512	/* default prime modulus size (bits) */
 #define	ILEN		256	/* default identity modulus size (bits) */
 #define	MVMAX		100	/* max MV parameters */
@@ -133,14 +128,15 @@ extern	int	ntp_getopt	(int, char **, const char *);
 #define BASIC_CONSTRAINTS	"critical,CA:TRUE"
 #define EXT_KEY_PRIVATE		"private"
 #define EXT_KEY_TRUST		"trustRoot"
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 /*
  * Prototypes
  */
 FILE	*fheader	(const char *, const char *, const char *);
 int	gen_md5		(char *);
-#ifdef OPENSSL
+void	followlink	(char *, size_t);
+#ifdef AUTOKEY
 EVP_PKEY *gen_rsa	(char *);
 EVP_PKEY *gen_dsa	(char *);
 EVP_PKEY *gen_iffkey	(char *);
@@ -154,40 +150,39 @@ EVP_PKEY *genkey	(char *, char *);
 EVP_PKEY *readkey	(char *, char *, u_int *, EVP_PKEY **);
 void	writekey	(char *, char *, u_int *, EVP_PKEY **);
 u_long	asn2ntp		(ASN1_TIME *);
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 /*
  * Program variables
  */
 extern char *optarg;		/* command line argument */
 char	*progname;
-int	debug = 0;		/* debug, not de bug */
-#ifdef OPENSSL
-u_int	modulus = PLEN;		/* prime modulus size (bits) */
-u_int	modulus2 = ILEN;	/* identity modulus size (bits) */
-#endif
+u_int	lifetime = YEAR;	/* certificate lifetime (days) */
 int	nkeys;			/* MV keys */
 time_t	epoch;			/* Unix epoch (seconds) since 1970 */
 u_int	fstamp;			/* NTP filestamp */
-char	*hostname = NULL;	/* host name (subject name) */
-char	*groupname = NULL;	/* trusted host name (issuer name) */
-char	filename[MAXFILENAME + 1]; /* file name */
+char	*hostname = NULL;	/* host name */
+char	*groupname = NULL;	/* group name */
+char	*certname = NULL;	/* certificate subjetc/issuer name */
 char	*passwd1 = NULL;	/* input private key password */
 char	*passwd2 = NULL;	/* output private key password */
-#ifdef OPENSSL
+char	filename[MAXFILENAME + 1]; /* file name */
+#ifdef AUTOKEY
+u_int	modulus = PLEN;		/* prime modulus size (bits) */
+u_int	modulus2 = ILEN;	/* identity modulus size (bits) */
 long	d0, d1, d2, d3;		/* callback counters */
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 #ifdef SYS_WINNT
 BOOL init_randfile();
 
 /*
- * Don't try to follow symbolic links
+ * Don't try to follow symbolic links.  Assumes link == file.
  */
 int
 readlink(char *link, char *file, int len)
 {
-	return (-1);
+	return strlen(file);
 }
 
 /*
@@ -200,6 +195,7 @@ symlink(char *filename, char *linkname) {
 	MoveFile(filename, linkname);
 	return (0);
 }
+
 void
 InitWin32Sockets() {
 	WORD wVersionRequested;
@@ -213,6 +209,29 @@ InitWin32Sockets() {
 }
 #endif /* SYS_WINNT */
 
+
+/*
+ * followlink() - replace filename with its target if symlink.
+ *
+ * Some readlink() implementations do not null-terminate the result.
+ */
+void
+followlink(
+	char *	fname,
+	size_t	bufsiz
+	)
+{
+	int len;
+
+	len = readlink(fname, fname, (int)bufsiz);
+	if (len < 0 || bufsiz < 1)
+		return;
+	if (len > (int)bufsiz - 1)
+		len = (int)bufsiz - 1;
+	fname[len] = '\0';
+}
+
+
 /*
  * Main program
  */
@@ -224,7 +243,7 @@ main(
 {
 	struct timeval tv;	/* initialization vector */
 	int	md5key = 0;	/* generate MD5 keys */
-#ifdef OPENSSL
+#ifdef AUTOKEY
 	X509	*cert = NULL;	/* X509 certificate */
 	X509_EXTENSION *ext;	/* X509v3 extension */
 	EVP_PKEY *pkey_host = NULL; /* host key */
@@ -242,73 +261,73 @@ main(
 	EVP_PKEY *pkey = NULL;	/* temp key */
 	const EVP_MD *ectx;	/* EVP digest */
 	char	pathbuf[MAXFILENAME + 1];
+	char	str[MAXFILENAME + 1];
 	const char *scheme = NULL; /* digest/signature scheme */
 	char	*exten = NULL;	/* private extension */
 	char	*grpkey = NULL;	/* identity extension */
 	int	nid;		/* X509 digest/signature scheme */
 	FILE	*fstr = NULL;	/* file handle */
-#define iffsw   HAVE_OPT(ID_KEY)
-#endif /* OPENSSL */
-	char	hostbuf[MAXHOSTNAME + 1];
 	char	groupbuf[MAXHOSTNAME + 1];
+#define iffsw   HAVE_OPT(ID_KEY)
+#endif	/* AUTOKEY */
+	char	hostbuf[MAXHOSTNAME + 1];
 
 	progname = argv[0];
 
 #ifdef SYS_WINNT
 	/* Initialize before OpenSSL checks */
 	InitWin32Sockets();
-	if(!init_randfile())
+	if (!init_randfile())
 		fprintf(stderr, "Unable to initialize .rnd file\n");
+	ssl_applink();
 #endif
 
 #ifdef OPENSSL
-	/*
-	 * OpenSSL version numbers: MNNFFPPS: major minor fix patch
-	 * status We match major, minor, fix and status (not patch)
-	 */
-	if ((SSLeay() ^ OPENSSL_VERSION_NUMBER) & ~0xff0L) {
-		fprintf(stderr,
-		    "OpenSSL version mismatch. Built against %lx, you have %lx\n",
-		    OPENSSL_VERSION_NUMBER, SSLeay());
-		exit (-1);
-
-	} else {
-		fprintf(stderr,
-		    "Using OpenSSL version %lx\n", SSLeay());
-	}
-#endif /* OPENSSL */
+	ssl_check_version();
+#endif	/* OPENSSL */
 
 	/*
 	 * Process options, initialize host name and timestamp.
 	 */
 	gethostname(hostbuf, MAXHOSTNAME);
-	hostname = hostbuf;
+	hostname = groupname = certname = passwd1 = hostbuf;
+	passwd2 = NULL;
 	gettimeofday(&tv, 0);
-
 	epoch = tv.tv_sec;
+	fstamp = (u_int)(epoch + JAN_1970);
 
 	{
-		int optct = optionProcess(&ntp_keygenOptions, argc, argv);
+		int optct = ntpOptionProcess(&ntp_keygenOptions,
+					     argc, argv);
 		argc -= optct;
 		argv += optct;
 	}
+
+#ifdef OPENSSL
+	if (SSLeay() == SSLEAY_VERSION_NUMBER)
+		fprintf(stderr, "Using OpenSSL version %s\n",
+			SSLeay_version(SSLEAY_VERSION));
+	else
+		fprintf(stderr, "Built against OpenSSL %s, using version %s\n",
+			OPENSSL_VERSION_TEXT, SSLeay_version(SSLEAY_VERSION));
+#endif /* OPENSSL */
+
 	debug = DESC(DEBUG_LEVEL).optOccCt;
 	if (HAVE_OPT( MD5KEY ))
 		md5key++;
 
-#ifdef OPENSSL
-	passwd1 = hostbuf;
+#ifdef AUTOKEY
 	if (HAVE_OPT( PVT_PASSWD ))
-		passwd1 = strdup(OPT_ARG( PVT_PASSWD ));
+		passwd1 = estrdup(OPT_ARG( PVT_PASSWD ));
 
 	if (HAVE_OPT( GET_PVT_PASSWD ))
-		passwd2 = strdup(OPT_ARG( GET_PVT_PASSWD ));
+		passwd2 = estrdup(OPT_ARG( GET_PVT_PASSWD ));
 
 	if (HAVE_OPT( HOST_KEY ))
 		hostkey++;
 
 	if (HAVE_OPT( SIGN_KEY ))
-		sign = strdup(OPT_ARG( SIGN_KEY ));
+		sign = estrdup(OPT_ARG( SIGN_KEY ));
 
 	if (HAVE_OPT( GQ_PARAMS ))
 		gqkey++;
@@ -330,11 +349,21 @@ main(
 	if (HAVE_OPT( CERTIFICATE ))
 		scheme = OPT_ARG( CERTIFICATE );
 
-	if (HAVE_OPT( SUBJECT_NAME ))
-		hostname = strdup(OPT_ARG( SUBJECT_NAME ));
+	if (HAVE_OPT( SUBJECT_NAME )) {
+		if (*OPT_ARG(SUBJECT_NAME) != '@') {
+			certname = estrdup(OPT_ARG(SUBJECT_NAME));
+		} else {
+			strlcpy(str, certname, sizeof(str));
+			strlcat(str, OPT_ARG(SUBJECT_NAME), sizeof(str));
+			certname = estrdup(str);
+		}
+	}
 
 	if (HAVE_OPT( ISSUER_NAME ))
-		groupname = strdup(OPT_ARG( ISSUER_NAME ));
+		groupname = estrdup(OPT_ARG( ISSUER_NAME ));
+
+	if (HAVE_OPT( LIFETIME ))
+		lifetime = OPT_VALUE_LIFETIME;
 
 	if (HAVE_OPT( PVT_CERT ))
 		exten = EXT_KEY_PRIVATE;
@@ -350,7 +379,7 @@ main(
 	if (!RAND_status()) {
 		u_int	temp;
 
-		if (RAND_file_name(pathbuf, MAXFILENAME) == NULL) {
+		if (RAND_file_name(pathbuf, sizeof(pathbuf)) == NULL) {
 			fprintf(stderr, "RAND_file_name %s\n",
 			    ERR_error_string(ERR_get_error(), NULL));
 			exit (-1);
@@ -370,7 +399,7 @@ main(
 	/*
 	 * Load previous certificate if available.
 	 */
-	sprintf(filename, "ntpkey_cert_%s", hostname);
+	snprintf(filename, sizeof(filename), "ntpkey_cert_%s", hostname);
 	if ((fstr = fopen(filename, "r")) != NULL) {
 		cert = PEM_read_X509(fstr, NULL, NULL, NULL);
 		fclose(fstr);
@@ -418,22 +447,17 @@ main(
 					else if (strcmp(pathbuf,
 					    "Private") == 0)
 						exten = EXT_KEY_PRIVATE;
-					if (groupname == NULL)
-						groupname = ptr + 3;
+					if (certname == NULL)
+						certname = ptr + 3;
 				}
 			}
 		}
 	}
 	if (scheme == NULL)
 		scheme = "RSA-MD5";
-	if (groupname == NULL)
-		groupname = hostname;
 	fprintf(stderr, "Using host %s group %s\n", hostname,
 	    groupname);
-	if ((iffkey || gqkey || mvkey) && exten == NULL)
-		fprintf(stderr,
-		    "Warning: identity files may not be useful with a nontrusted certificate.\n");
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 	/*
 	 * Create new unencrypted MD5 keys file if requested. If this
@@ -444,7 +468,7 @@ main(
 		exit (0);
 	}
 
-#ifdef OPENSSL
+#ifdef AUTOKEY
 	/*
 	 * Create a new encrypted RSA host key file if requested;
 	 * otherwise, look for an existing host key file. If not found,
@@ -454,10 +478,10 @@ main(
 	if (hostkey)
 		pkey_host = genkey("RSA", "host");
 	if (pkey_host == NULL) {
-		sprintf(filename, "ntpkey_host_%s", hostname);
+		snprintf(filename, sizeof(filename), "ntpkey_host_%s", hostname);
 		pkey_host = readkey(filename, passwd1, &fstamp, NULL);
 		if (pkey_host != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using host key %s\n",
 			    filename);
 		} else {
@@ -477,13 +501,14 @@ main(
 	if (sign != NULL)
 		pkey_sign = genkey(sign, "sign");
 	if (pkey_sign == NULL) {
-		sprintf(filename, "ntpkey_sign_%s", hostname);
+		snprintf(filename, sizeof(filename), "ntpkey_sign_%s",
+			 hostname);
 		pkey_sign = readkey(filename, passwd1, &fstamp, NULL);
 		if (pkey_sign != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using sign key %s\n",
 			    filename);
-		} else if (pkey_host != NULL) {
+		} else {
 			pkey_sign = pkey_host;
 			fprintf(stderr, "Using host key as sign key\n");
 		}
@@ -497,10 +522,11 @@ main(
 	if (gqkey)
 		pkey_gqkey = gen_gqkey("gqkey");
 	if (pkey_gqkey == NULL) {
-		sprintf(filename, "ntpkey_gqkey_%s", groupname);
+		snprintf(filename, sizeof(filename), "ntpkey_gqkey_%s",
+		    groupname);
 		pkey_gqkey = readkey(filename, passwd1, &fstamp, NULL);
 		if (pkey_gqkey != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using GQ parameters %s\n",
 			    filename);
 		}
@@ -516,9 +542,8 @@ main(
 	if (pkey_gqkey != NULL && HAVE_OPT(ID_KEY)) {
 		RSA	*rsa;
 
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_gqpar_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_gqpar_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing GQ parameters %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -541,8 +566,8 @@ main(
 	if (pkey_gqkey != NULL && passwd2 != NULL) {
 		RSA	*rsa;
 
-		sprintf(filename, "ntpkey_gqkey_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_gqkey_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing GQ keys %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -564,10 +589,11 @@ main(
 	if (iffkey)
 		pkey_iffkey = gen_iffkey("iffkey");
 	if (pkey_iffkey == NULL) {
-		sprintf(filename, "ntpkey_iffkey_%s", groupname);
+		snprintf(filename, sizeof(filename), "ntpkey_iffkey_%s",
+		    groupname);
 		pkey_iffkey = readkey(filename, passwd1, &fstamp, NULL);
 		if (pkey_iffkey != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using IFF keys %s\n",
 			    filename);
 		}
@@ -581,9 +607,8 @@ main(
 	if (pkey_iffkey != NULL && HAVE_OPT(ID_KEY)) {
 		DSA	*dsa;
 
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_iffpar_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_iffpar_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing IFF parameters %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -605,9 +630,8 @@ main(
 	if (pkey_iffkey != NULL && passwd2 != NULL) {
 		DSA	*dsa;
 
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_iffkey_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_iffkey_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing IFF keys %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -629,11 +653,12 @@ main(
 	if (mvkey)
 		pkey_mvkey = gen_mvkey("mv", pkey_mvpar);
 	if (pkey_mvkey == NULL) {
-		sprintf(filename, "ntpkey_mvta_%s", groupname);
+		snprintf(filename, sizeof(filename), "ntpkey_mvta_%s",
+		    groupname);
 		pkey_mvkey = readkey(filename, passwd1, &fstamp,
-		   pkey_mvpar);
+		    pkey_mvpar);
 		if (pkey_mvkey != NULL) {
-			readlink(filename, filename, sizeof(filename));
+			followlink(filename, sizeof(filename));
 			fprintf(stderr, "Using MV keys %s\n",
 			    filename);
 		}
@@ -645,9 +670,8 @@ main(
 	 * associated with client key 1.
 	 */
 	if (pkey_mvkey != NULL && HAVE_OPT(ID_KEY)) {
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_mvpar_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_mvpar_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing MV parameters %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -664,9 +688,8 @@ main(
 	 * Write the encrypted MV server keys to the stdout stream.
 	 */
 	if (pkey_mvkey != NULL && passwd2 != NULL) {
-		epoch = fstamp - JAN_1970;
-		sprintf(filename, "ntpkey_mvkey_%s.%u", groupname,
-		    fstamp);
+		snprintf(filename, sizeof(filename),
+		    "ntpkey_mvkey_%s.%u", groupname, fstamp);
 		fprintf(stderr, "Writing MV keys %s to stdout\n",
 		    filename);
 		fprintf(stdout, "# %s\n# %s\n", filename,
@@ -680,16 +703,8 @@ main(
 	}
 
 	/*
-	 * Don't generate a certificate if no host keys or extracting
-	 * encrypted or nonencrypted keys to the standard output stream.
-	 */
-	if (pkey_host == NULL || HAVE_OPT(ID_KEY) || passwd2 != NULL)
-		exit (0);
-
-	/*
-	 * Decode the digest/signature scheme. If trusted, set the
-	 * subject and issuer names to the group name; if not set both
-	 * to the host name.
+	 * Decode the digest/signature scheme and create the
+	 * certificate. Do this every time we run the program.
 	 */
 	ectx = EVP_get_digestbyname(scheme);
 	if (ectx == NULL) {
@@ -698,31 +713,35 @@ main(
 		    scheme);
 			exit (-1);
 	}
-	if (exten == NULL)
-		x509(pkey_sign, ectx, grpkey, exten, hostname);
-	else
-		x509(pkey_sign, ectx, grpkey, exten, groupname);
-#endif /* OPENSSL */
+	x509(pkey_sign, ectx, grpkey, exten, certname);
+#endif	/* AUTOKEY */
 	exit (0);
 }
 
 
 /*
- * Generate semi-random MD5 keys compatible with NTPv3 and NTPv4
+ * Generate semi-random MD5 keys compatible with NTPv3 and NTPv4. Also,
+ * if OpenSSL is around, generate random SHA1 keys compatible with
+ * symmetric key cryptography.
  */
 int
 gen_md5(
 	char	*id		/* file name id */
 	)
 {
-	u_char	md5key[16+1];	/* MD5 key */
+	u_char	md5key[MD5SIZE + 1];	/* MD5 key */
 	FILE	*str;
 	int	i, j;
+#ifdef OPENSSL
+	u_char	keystr[MD5SIZE];
+	u_char	hexstr[2 * MD5SIZE + 1];
+	u_char	hex[] = "0123456789abcdef";
+#endif	/* OPENSSL */
 
 	str = fheader("MD5key", id, groupname);
 	ntp_srandom((u_long)epoch);
 	for (i = 1; i <= MD5KEYS; i++) {
-		for (j = 0; j < 16; j++) {
+		for (j = 0; j < MD5SIZE; j++) {
 			int temp;
 
 			while (1) {
@@ -736,15 +755,27 @@ gen_md5(
 			md5key[j] = (u_char)temp;
 		}
 		md5key[j] = '\0';
-		fprintf(str, "%2d MD5 %16s	# MD5 key\n", i,
+		fprintf(str, "%2d MD5 %s  # MD5 key\n", i,
 		    md5key);
 	}
+#ifdef OPENSSL
+	for (i = 1; i <= MD5KEYS; i++) {
+		RAND_bytes(keystr, 20);
+		for (j = 0; j < MD5SIZE; j++) {
+			hexstr[2 * j] = hex[keystr[j] >> 4];
+			hexstr[2 * j + 1] = hex[keystr[j] & 0xf];
+		}
+		hexstr[2 * MD5SIZE] = '\0';
+		fprintf(str, "%2d SHA1 %s  # SHA1 key\n", i + MD5KEYS,
+		    hexstr);
+	}
+#endif	/* OPENSSL */
 	fclose(str);
 	return (1);
 }
 
 
-#ifdef OPENSSL
+#ifdef AUTOKEY
 /*
  * readkey - load cryptographic parameters and keys
  *
@@ -1192,8 +1223,8 @@ gen_gqkey(
 		    ERR_error_string(ERR_get_error(), NULL));
 		return (NULL);
 	}
-	ctx = BN_CTX_new(); u = BN_new(); v = BN_new();
-	g = BN_new(); k = BN_new(); r = BN_new(); y = BN_new();
+	u = BN_new(); v = BN_new(); g = BN_new();
+	k = BN_new(); r = BN_new(); y = BN_new();
 
 	/*
 	 * Generate the group key b, which is saved in the e member of
@@ -1363,8 +1394,8 @@ gen_gqkey(
  *
  * How it works
  *
- * The scheme goes like this. Bob has the server values (p, E, q, gbar,
- * ghat) and Alice has the client values (p, xbar, xhat).
+ * The scheme goes like this. Bob has the server values (p, E, q,
+ * gbar, ghat) and Alice has the client values (p, xbar, xhat).
  *
  * Alice rolls new random nonce r mod p and sends to Bob in the MV
  * request message. Bob rolls random nonce k mod q, encrypts y = r E^k
@@ -1527,7 +1558,6 @@ gen_mvkey(
 	 */
 	for (i = 0; i <= n; i++) {
 		a[i] = BN_new();
-
 		BN_one(a[i]);
 	}
 	for (j = 1; j <= n; j++) {
@@ -1547,7 +1577,6 @@ gen_mvkey(
 	 */
 	for (i = 0; i <= n; i++) {
 		g[i] = BN_new();
-
 		BN_mod_exp(g[i], dsa->g, a[i], dsa->p, ctx);
 	}
 
@@ -1626,6 +1655,7 @@ gen_mvkey(
 		for (i = 1; i <= n; i++) {
 			if (i == j)
 				continue;
+
 			BN_mod_exp(u, x[i], v, dsa->q, ctx);
 			BN_add(xbar[j], xbar[j], u);
 		}
@@ -1642,9 +1672,7 @@ gen_mvkey(
 	 * additional keys, so we sail on with only token revocations.
 	 */
 	s = BN_new();
-
 	BN_copy(s, dsa->q);
-	BN_div(s, u, s, s1[10], ctx);
 	BN_div(s, u, s, s1[n], ctx);
 
 	/*
@@ -1656,7 +1684,6 @@ gen_mvkey(
 	 * changed.
 	 */
 	bige = BN_new(); gbar = BN_new(); ghat = BN_new();
-
 	BN_mod_exp(bige, biga, s, dsa->p, ctx);
 	BN_mod_exp(gbar, dsa->g, s, dsa->p, ctx);
 	BN_mod_mul(v, s, b, dsa->q, ctx);
@@ -1744,7 +1771,6 @@ gen_mvkey(
 	fprintf(stderr, "Generating %d MV client keys\n", n);
 	for (j = 1; j <= n; j++) {
 		sdsa = DSA_new();
-
 		sdsa->p = BN_dup(dsa->p);
 		sdsa->q = BN_dup(BN_value_one()); 
 		sdsa->g = BN_dup(BN_value_one()); 
@@ -1809,7 +1835,7 @@ x509	(
 	const EVP_MD *md,	/* generic digest algorithm */
 	char	*gqpub,		/* identity extension (hex string) */
 	char	*exten,		/* private cert extension */
-	char	*name		/* subject/issuer namd */
+	char	*name		/* subject/issuer name */
 	)
 {
 	X509	*cert;		/* X509 certificate */
@@ -1836,7 +1862,7 @@ x509	(
 	X509_set_serialNumber(cert, serial);
 	ASN1_INTEGER_free(serial);
 	X509_time_adj(X509_get_notBefore(cert), 0L, &epoch);
-	X509_time_adj(X509_get_notAfter(cert), YEAR, &epoch);
+	X509_time_adj(X509_get_notAfter(cert), lifetime * DAY, &epoch);
 	subj = X509_get_subject_name(cert);
 	X509_NAME_add_entry_by_txt(subj, "commonName", MBSTRING_ASC,
 	    (unsigned char *) name, strlen(name), -1, 0);
@@ -1924,7 +1950,7 @@ x509	(
 	 * Sign and verify.
 	 */
 	X509_sign(cert, pkey, md);
-	if (!X509_verify(cert, pkey)) {
+	if (X509_verify(cert, pkey) <= 0) {
 		fprintf(stderr, "Verify %s certificate fails\n%s\n", id,
 		    ERR_error_string(ERR_get_error(), NULL));
 		X509_free(cert);
@@ -1934,7 +1960,7 @@ x509	(
 	/*
 	 * Write the certificate encoded in PEM.
 	 */
-	sprintf(pathbuf, "%scert", id);
+	snprintf(pathbuf, sizeof(pathbuf), "%scert", id);
 	str = fheader(pathbuf, "cert", hostname);
 	PEM_write_X509(str, cert);
 	fclose(str);
@@ -2036,7 +2062,7 @@ genkey(
 	fprintf(stderr, "Invalid %s key type %s\n", id, type);
 	return (NULL);
 }
-#endif /* OPENSSL */
+#endif	/* AUTOKEY */
 
 
 /*
@@ -2053,13 +2079,14 @@ fheader	(
 	char	linkname[MAXFILENAME]; /* link name */
 	int	temp;
 
-	sprintf(filename, "ntpkey_%s_%s.%lu", file, owner, epoch +
-	    JAN_1970);
+	snprintf(filename, sizeof(filename), "ntpkey_%s_%s.%u", file,
+	    owner, fstamp); 
 	if ((str = fopen(filename, "w")) == NULL) {
 		perror("Write");
 		exit (-1);
 	}
-	sprintf(linkname, "ntpkey_%s_%s", ulink, owner);
+	snprintf(linkname, sizeof(linkname), "ntpkey_%s_%s", ulink,
+	    hostname);
 	remove(linkname);
 	temp = symlink(filename, linkname);
 	if (temp < 0)
